@@ -50,6 +50,8 @@
 #include <cmath>
 #include <wtf/BitVector.h>
 
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_BEGIN
+
 namespace JSC { namespace B3 {
 
 namespace {
@@ -85,6 +87,15 @@ public:
     }
     
 private:
+    template <class Fn>
+    void replaceWithBinaryCall(Fn &&function)
+    {
+        Value* functionAddress = m_insertionSet.insert<ConstPtrValue>(m_index, m_origin, tagCFunction<OperationPtrTag>(function));
+        Value* result = m_insertionSet.insert<CCallValue>(m_index, m_value->type(), m_origin, Effects::none(), functionAddress, m_value->child(0), m_value->child(1));
+        m_value->replaceWithIdentity(result);
+        m_changed = true;
+    }
+
     void processCurrentBlock()
     {
         for (m_index = 0; m_index < m_block->size(); ++m_index) {
@@ -127,10 +138,8 @@ private:
                     break;
                 }
                 
-                double(*fmodDouble)(double, double) = fmod;
-                fmodDouble = tagCFunction<B3CCallPtrTag>(fmodDouble);
                 if (m_value->type() == Double) {
-                    Value* functionAddress = m_insertionSet.insert<ConstPtrValue>(m_index, m_origin, fmodDouble);
+                    Value* functionAddress = m_insertionSet.insert<ConstPtrValue>(m_index, m_origin, tagCFunction<OperationPtrTag>(Math::fmodDouble));
                     Value* result = m_insertionSet.insert<CCallValue>(m_index, Double, m_origin,
                         Effects::none(),
                         functionAddress,
@@ -141,7 +150,7 @@ private:
                 } else if (m_value->type() == Float) {
                     Value* numeratorAsDouble = m_insertionSet.insert<Value>(m_index, FloatToDouble, m_origin, m_value->child(0));
                     Value* denominatorAsDouble = m_insertionSet.insert<Value>(m_index, FloatToDouble, m_origin, m_value->child(1));
-                    Value* functionAddress = m_insertionSet.insert<ConstPtrValue>(m_index, m_origin, fmodDouble);
+                    Value* functionAddress = m_insertionSet.insert<ConstPtrValue>(m_index, m_origin, tagCFunction<OperationPtrTag>(Math::fmodDouble));
                     Value* doubleMod = m_insertionSet.insert<CCallValue>(m_index, Double, m_origin,
                         Effects::none(),
                         functionAddress,
@@ -150,6 +159,11 @@ private:
                     Value* result = m_insertionSet.insert<Value>(m_index, DoubleToFloat, m_origin, doubleMod);
                     m_value->replaceWithIdentity(result);
                     m_changed = true;
+                } else if constexpr (isARM_THUMB2()) {
+                    if (m_value->type() == Int64)
+                        replaceWithBinaryCall(Math::i64_rem_s);
+                    else
+                        replaceWithBinaryCall(Math::i32_rem_s);
                 } else if (isARM64()) {
                     Value* divResult = m_insertionSet.insert<Value>(m_index, chill(Div), m_origin, m_value->child(0), m_value->child(1));
                     Value* multipliedBack = m_insertionSet.insert<Value>(m_index, Mul, m_origin, divResult, m_value->child(1));
@@ -161,6 +175,13 @@ private:
             }
 
             case UMod: {
+                if constexpr (isARM_THUMB2()) {
+                    if (m_value->child(0)->type() == Int64)
+                        replaceWithBinaryCall(Math::i64_rem_u);
+                    else
+                        replaceWithBinaryCall(Math::i32_rem_u);
+                    break;
+                }
                 if (isARM64()) {
                     Value* divResult = m_insertionSet.insert<Value>(m_index, UDiv, m_origin, m_value->child(0), m_value->child(1));
                     Value* multipliedBack = m_insertionSet.insert<Value>(m_index, Mul, m_origin, divResult, m_value->child(1));
@@ -171,37 +192,90 @@ private:
                 break;
             }
 
-            case Div: {
-                if (m_value->isChill())
-                    makeDivisionChill(Div);
+            case UDiv: {
+                if constexpr (!isARM_THUMB2())
+                    break;
+                if (m_value->type() == Int64)
+                    replaceWithBinaryCall(Math::i64_div_u);
+                else
+                    replaceWithBinaryCall(Math::i32_div_u);
+                break;
+            }
+            case FMax:
+            case FMin: {
+                if (isX86() || isARM_THUMB2()) {
+                    bool isMax = m_value->opcode() == FMax;
+
+                    Value* a = m_value->child(0);
+                    Value* b = m_value->child(1);
+
+                    Value* isEqualValue = m_insertionSet.insert<Value>(
+                        m_index, Equal, m_origin, a, b);
+
+                    BasicBlock* before = m_blockInsertionSet.splitForward(m_block, m_index, &m_insertionSet);
+
+                    BasicBlock* isEqual = m_blockInsertionSet.insertBefore(m_block);
+                    BasicBlock* notEqual = m_blockInsertionSet.insertBefore(m_block);
+                    BasicBlock* isLessThan = m_blockInsertionSet.insertBefore(m_block);
+                    BasicBlock* notLessThan = m_blockInsertionSet.insertBefore(m_block);
+                    BasicBlock* isGreaterThan = m_blockInsertionSet.insertBefore(m_block);
+                    BasicBlock* isNaN = m_blockInsertionSet.insertBefore(m_block);
+
+                    before->replaceLastWithNew<Value>(m_proc, Branch, m_origin, isEqualValue);
+                    before->setSuccessors(FrequentedBlock(isEqual), FrequentedBlock(notEqual));
+
+                    Value* lessThanValue = notEqual->appendNew<Value>(m_proc, LessThan, m_origin, a, b);
+                    notEqual->appendNew<Value>(m_proc, Branch, m_origin, lessThanValue);
+                    notEqual->setSuccessors(FrequentedBlock(isLessThan), FrequentedBlock(notLessThan));
+
+                    Value* greaterThanValue = notLessThan->appendNew<Value>(m_proc, GreaterThan, m_origin, a, b);
+                    notLessThan->appendNew<Value>(m_proc, Branch, m_origin, greaterThanValue);
+                    notLessThan->setSuccessors(FrequentedBlock(isGreaterThan), FrequentedBlock(isNaN));
+
+                    UpsilonValue* isLessThanResult = isLessThan->appendNew<UpsilonValue>(
+                        m_proc, m_origin, isMax ? b : a);
+                    isLessThan->appendNew<Value>(m_proc, Jump, m_origin);
+                    isLessThan->setSuccessors(FrequentedBlock(m_block));
+
+                    UpsilonValue* isGreaterThanResult = isGreaterThan->appendNew<UpsilonValue>(
+                        m_proc, m_origin, isMax ? a : b);
+                    isGreaterThan->appendNew<Value>(m_proc, Jump, m_origin);
+                    isGreaterThan->setSuccessors(FrequentedBlock(m_block));
+
+                    UpsilonValue* isEqualResult = isEqual->appendNew<UpsilonValue>(
+                        m_proc, m_origin, isEqual->appendNew<Value>(m_proc, isMax ? BitAnd : BitOr, m_origin, a, b));
+                    isEqual->appendNew<Value>(m_proc, Jump, m_origin);
+                    isEqual->setSuccessors(FrequentedBlock(m_block));
+
+                    UpsilonValue* isNaNResult = isNaN->appendNew<UpsilonValue>(
+                        m_proc, m_origin, isNaN->appendNew<Value>(m_proc, Add, m_origin, a, b));
+                    isNaN->appendNew<Value>(m_proc, Jump, m_origin);
+                    isNaN->setSuccessors(FrequentedBlock(m_block));
+
+                    Value* phi = m_insertionSet.insert<Value>(
+                        m_index, Phi, m_value->type(), m_origin);
+                    isLessThanResult->setPhi(phi);
+                    isGreaterThanResult->setPhi(phi);
+                    isEqualResult->setPhi(phi);
+                    isNaNResult->setPhi(phi);
+
+                    m_value->replaceWithIdentity(phi);
+                    before->updatePredecessorsAfter();
+                    m_changed = true;
+                }
                 break;
             }
 
-            case CheckMul: {
-                if (isARM64() && m_value->child(0)->type() == Int32) {
-                    CheckValue* checkMul = m_value->as<CheckValue>();
-
-                    Value* left = m_insertionSet.insert<Value>(m_index, SExt32, m_origin, m_value->child(0));
-                    Value* right = m_insertionSet.insert<Value>(m_index, SExt32, m_origin, m_value->child(1));
-                    Value* mulResult = m_insertionSet.insert<Value>(m_index, Mul, m_origin, left, right);
-                    Value* mulResult32 = m_insertionSet.insert<Value>(m_index, Trunc, m_origin, mulResult);
-                    Value* upperResult = m_insertionSet.insert<Value>(m_index, Trunc, m_origin,
-                        m_insertionSet.insert<Value>(m_index, SShr, m_origin, mulResult, m_insertionSet.insert<Const32Value>(m_index, m_origin, 32)));
-                    Value* signBit = m_insertionSet.insert<Value>(m_index, SShr, m_origin,
-                        mulResult32,
-                        m_insertionSet.insert<Const32Value>(m_index, m_origin, 31));
-                    Value* hasOverflowed = m_insertionSet.insert<Value>(m_index, NotEqual, m_origin, upperResult, signBit);
-
-                    CheckValue* check = m_insertionSet.insert<CheckValue>(m_index, Check, m_origin, hasOverflowed);
-                    check->setGenerator(checkMul->generator());
-                    check->clobberEarly(checkMul->earlyClobbered());
-                    check->clobberLate(checkMul->lateClobbered());
-                    auto children = checkMul->constrainedChildren();
-                    auto it = children.begin();
-                    for (std::advance(it, 2); it != children.end(); ++it)
-                        check->append(*it);
-
-                    m_value->replaceWithIdentity(mulResult32);
+            case Div: {
+                if (m_value->isChill())
+                    makeDivisionChill(Div);
+                else if (isARM_THUMB2() && (m_value->type() == Int64 || m_value->type() == Int32)) {
+                    BasicBlock* before = m_blockInsertionSet.splitForward(m_block, m_index);
+                    before->replaceLastWithNew<Value>(m_proc, Nop, m_origin);
+                    Value* result = callDivModHelper(before, Div, m_value->child(0), m_value->child(1));
+                    before->appendNew<Value>(m_proc, Jump, m_origin);
+                    before->setSuccessors(FrequentedBlock(m_block));
+                    m_value->replaceWithIdentity(result);
                     m_changed = true;
                 }
                 break;
@@ -308,6 +382,14 @@ private:
                     if (exempt)
                         break;
                 }
+
+                if (isARM64_LSE()) {
+                    if (m_value->opcode() == AtomicXchgSub) {
+                        m_value->setOpcodeUnsafely(AtomicXchgAdd);
+                        m_value->child(0) = m_insertionSet.insert<Value>(
+                            m_index, Neg, m_origin, m_value->child(0));
+                    }
+                }
                 
                 AtomicValue* atomic = m_value->as<AtomicValue>();
                 Width width = atomic->accessWidth();
@@ -346,7 +428,137 @@ private:
                 m_changed = true;
                 break;
             }
-                
+
+            case VectorPopcnt: {
+                if (!isX86())
+                    break;
+                ASSERT(m_value->as<SIMDValue>()->simdLane() == SIMDLane::i8x16);
+
+                // x86_64 does not natively support vector lanewise popcount, so we emulate it using multiple
+                // masks.
+
+                v128_t bottomNibbleConst;
+                v128_t popcntConst;
+                bottomNibbleConst.u64x2[0] = 0x0f0f0f0f0f0f0f0f;
+                bottomNibbleConst.u64x2[1] = 0x0f0f0f0f0f0f0f0f;
+                popcntConst.u64x2[0] = 0x0302020102010100;
+                popcntConst.u64x2[1] = 0x0403030203020201;
+                Value* bottomNibbleMask = m_insertionSet.insert<Const128Value>(m_index, m_origin, bottomNibbleConst);
+                Value* popcntMask = m_insertionSet.insert<Const128Value>(m_index, m_origin, popcntConst);
+
+                Value* four = m_insertionSet.insert<Const32Value>(m_index, m_origin, 4);
+                Value* v = m_value->child(0);
+                Value* upper = m_insertionSet.insert<SIMDValue>(m_index, m_origin, VectorAndnot, B3::V128, SIMDLane::v128, SIMDSignMode::None, v, bottomNibbleMask);
+                Value* lower = m_insertionSet.insert<SIMDValue>(m_index, m_origin, VectorAnd, B3::V128, SIMDLane::v128, SIMDSignMode::None, v, bottomNibbleMask);
+                upper = m_insertionSet.insert<SIMDValue>(m_index, m_origin, VectorShr, B3::V128, SIMDLane::i16x8, SIMDSignMode::Unsigned, upper, four);
+                lower = m_insertionSet.insert<SIMDValue>(m_index, m_origin, VectorSwizzle, B3::V128, SIMDLane::i8x16, SIMDSignMode::None, popcntMask, lower);
+                upper = m_insertionSet.insert<SIMDValue>(m_index, m_origin, VectorSwizzle, B3::V128, SIMDLane::i8x16, SIMDSignMode::None, popcntMask, upper);
+                Value* result = m_insertionSet.insert<SIMDValue>(m_index, m_origin, VectorAdd, B3::V128, SIMDLane::i8x16, SIMDSignMode::None, upper, lower);
+                m_value->replaceWithIdentity(result);
+                m_changed = true;
+                break;
+            }
+
+            case VectorNot: {
+                if (!isX86())
+                    break;
+                // x86_64 has no vector bitwise NOT instruction, so we expand vxv.not v into vxv.xor -1, v
+                // here to give B3/Air a chance to optimize out repeated usage of the mask.
+                v128_t mask;
+                mask.u64x2[0] = 0xffffffffffffffff;
+                mask.u64x2[1] = 0xffffffffffffffff;
+                Value* ones = m_insertionSet.insert<Const128Value>(m_index, m_origin, mask);
+                Value* result = m_insertionSet.insert<SIMDValue>(m_index, m_origin, VectorXor, B3::V128, SIMDLane::v128, SIMDSignMode::None, ones, m_value->child(0));
+                m_value->replaceWithIdentity(result);
+                m_changed = true;
+                break;
+            }
+
+            case VectorNeg: {
+                if (!isX86())
+                    break;
+                // x86_64 has no vector negate instruction. For integer vectors, we can replicate negation by
+                // subtracting from zero. For floating-point vectors, we need to toggle the sign using packed
+                // XOR.
+                SIMDValue* value = m_value->as<SIMDValue>();
+                switch (value->simdLane()) {
+                case SIMDLane::i8x16:
+                case SIMDLane::i16x8:
+                case SIMDLane::i32x4:
+                case SIMDLane::i64x2: {
+                    Value* zero = m_insertionSet.insert<Const128Value>(m_index, m_origin, v128_t());
+                    Value* result = m_insertionSet.insert<SIMDValue>(m_index, m_origin, VectorSub, B3::V128, value->simdInfo(), zero, m_value->child(0));
+                    m_value->replaceWithIdentity(result);
+                    m_changed = true;
+                    break;
+                }
+                case SIMDLane::f32x4: {
+                    Value* topBit = m_insertionSet.insert<Const32Value>(m_index, m_origin, 0x80000000u);
+                    Value* floatMask = m_insertionSet.insert<Value>(m_index, BitwiseCast, m_origin, topBit);
+                    Value* vectorMask = m_insertionSet.insert<SIMDValue>(m_index, m_origin, VectorSplat, B3::V128, SIMDLane::f32x4, SIMDSignMode::None, floatMask);
+                    Value* result = m_insertionSet.insert<SIMDValue>(m_index, m_origin, VectorXor, B3::V128, SIMDLane::v128, SIMDSignMode::None, m_value->child(0), vectorMask);
+                    m_value->replaceWithIdentity(result);
+                    m_changed = true;
+                    break;
+                }
+                case SIMDLane::f64x2: {
+                    Value* topBit = m_insertionSet.insert<Const64Value>(m_index, m_origin, 0x8000000000000000ull);
+                    Value* doubleMask = m_insertionSet.insert<Value>(m_index, BitwiseCast, m_origin, topBit);
+                    Value* vectorMask = m_insertionSet.insert<SIMDValue>(m_index, m_origin, VectorSplat, B3::V128, SIMDLane::f64x2, SIMDSignMode::None, doubleMask);
+                    Value* result = m_insertionSet.insert<SIMDValue>(m_index, m_origin, VectorXor, B3::V128, SIMDLane::v128, SIMDSignMode::None, m_value->child(0), vectorMask);
+                    m_value->replaceWithIdentity(result);
+                    m_changed = true;
+                    break;
+                }
+                default:
+                    RELEASE_ASSERT_NOT_REACHED();
+                }
+                break;
+            }
+
+            case VectorNotEqual:
+                if (isX86())
+                    invertedComparisonByXor(VectorEqual, m_value->child(0), m_value->child(1));
+                break;
+            case VectorAbove:
+                if (isX86())
+                    invertedComparisonByXor(VectorBelowOrEqual, m_value->child(0), m_value->child(1));
+                break;
+            case VectorBelow:
+                if (isX86())
+                    invertedComparisonByXor(VectorAboveOrEqual, m_value->child(0), m_value->child(1));
+                break;
+            case VectorGreaterThanOrEqual:
+                if (isX86() && m_value->as<SIMDValue>()->simdLane() == SIMDLane::i64x2) {
+                    // Note: rhs and lhs are reversed here, we are semantically negating LessThan. GreaterThan is
+                    // just better supported on AVX.
+                    invertedComparisonByXor(VectorGreaterThan, m_value->child(1), m_value->child(0));
+                }
+                break;
+            case VectorLessThanOrEqual:
+                if (isX86() && m_value->as<SIMDValue>()->simdLane() == SIMDLane::i64x2)
+                    invertedComparisonByXor(VectorGreaterThan, m_value->child(0), m_value->child(1));
+                break;
+            case VectorShr:
+            case VectorShl: {
+                if constexpr (!isARM64())
+                    break;
+                SIMDValue* value = m_value->as<SIMDValue>();
+                SIMDLane lane = value->simdLane();
+
+                int32_t mask = (elementByteSize(lane) * CHAR_BIT) - 1;
+                Value* shiftAmount = m_insertionSet.insert<Value>(m_index, BitAnd, m_origin, value->child(1), m_insertionSet.insertIntConstant(m_index, m_origin, Int32, mask));
+                if (value->opcode() == VectorShr) {
+                    // ARM64 doesn't have a version of this instruction for right shift. Instead, if the input to
+                    // left shift is negative, it's a right shift by the absolute value of that amount.
+                    shiftAmount = m_insertionSet.insert<Value>(m_index, Neg, m_origin, shiftAmount);
+                }
+                Value* shiftVector = m_insertionSet.insert<SIMDValue>(m_index, m_origin, VectorSplat, B3::V128, SIMDLane::i8x16, SIMDSignMode::None, shiftAmount);
+                Value* result = m_insertionSet.insert<SIMDValue>(m_index, m_origin, VectorShiftByVector, B3::V128, value->simdInfo(), value->child(0), shiftVector);
+                m_value->replaceWithIdentity(result);
+                m_changed = true;
+                break;
+            }
             default:
                 break;
             }
@@ -354,6 +566,42 @@ private:
         m_insertionSet.execute(m_block);
     }
 
+    void invertedComparisonByXor(Opcode opcodeToBeInverted, Value* lhs, Value* rhs)
+    {
+        v128_t allOnes;
+        allOnes.u64x2[0] = 0xffffffffffffffff;
+        allOnes.u64x2[1] = 0xffffffffffffffff;
+        Value* allOnesConst = m_insertionSet.insert<Const128Value>(m_index, m_origin, allOnes);
+        Value* compareResult = m_insertionSet.insert<SIMDValue>(m_index, m_origin, opcodeToBeInverted, B3::V128, m_value->as<SIMDValue>()->simdInfo(), lhs, rhs);
+        Value* result = m_insertionSet.insert<SIMDValue>(m_index, m_origin, VectorXor, B3::V128, SIMDLane::v128, SIMDSignMode::None, compareResult, allOnesConst);
+        m_value->replaceWithIdentity(result);
+        m_changed = true;
+    }
+
+#if USE(JSVALUE32_64)
+    Value* callDivModHelper(BasicBlock* block, Opcode nonChillOpcode, Value* num, Value* den)
+    {
+        Type type = num->type();
+        Value* functionAddress;
+        if (nonChillOpcode == Div) {
+            if (m_value->type() == Int64)
+                functionAddress = block->appendNew<ConstPtrValue>(m_proc, m_origin, tagCFunction<OperationPtrTag>(Math::i64_div_s));
+            else
+                functionAddress = block->appendNew<ConstPtrValue>(m_proc, m_origin, tagCFunction<OperationPtrTag>(Math::i32_div_s));
+        } else {
+            if (m_value->type() == Int64)
+                functionAddress = block->appendNew<ConstPtrValue>(m_proc, m_origin, tagCFunction<OperationPtrTag>(Math::i64_rem_s));
+            else
+                functionAddress = block->appendNew<ConstPtrValue>(m_proc, m_origin, tagCFunction<OperationPtrTag>(Math::i32_rem_s));
+        }
+        return block->appendNew<CCallValue>(m_proc, type, m_origin, Effects::none(), functionAddress, num, den);
+    }
+#else
+    Value* callDivModHelper(BasicBlock*, Opcode, Value*, Value*)
+    {
+        RELEASE_ASSERT_NOT_REACHED();
+    }
+#endif
     void makeDivisionChill(Opcode nonChillOpcode)
     {
         ASSERT(nonChillOpcode == Div || nonChillOpcode == Mod);
@@ -400,9 +648,14 @@ private:
             FrequentedBlock(normalDivCase, FrequencyClass::Normal),
             FrequentedBlock(shadyDenCase, FrequencyClass::Rare));
 
+        Value* innerResult;
+        if (isARM_THUMB2() && (m_value->type() == Int64 || m_value->type() == Int32))
+            innerResult = callDivModHelper(normalDivCase, nonChillOpcode, num, den);
+        else
+            innerResult = normalDivCase->appendNew<Value>(m_proc, nonChillOpcode, m_origin, num, den);
         UpsilonValue* normalResult = normalDivCase->appendNew<UpsilonValue>(
             m_proc, m_origin,
-            normalDivCase->appendNew<Value>(m_proc, nonChillOpcode, m_origin, num, den));
+            innerResult);
         normalDivCase->appendNew<Value>(m_proc, Jump, m_origin);
         normalDivCase->setSuccessors(FrequentedBlock(m_block));
 
@@ -504,7 +757,7 @@ private:
                 patchpoint->numGPScratchRegisters = 2;
                 // Technically, we don't have to clobber macro registers on X86_64. This is probably
                 // OK though.
-                patchpoint->clobber(RegisterSet::macroScratchRegisters());
+                patchpoint->clobber(RegisterSetBuilder::macroClobberedGPRs());
                 
                 BitVector handledIndices;
                 for (unsigned i = start; i < end; ++i) {
@@ -531,7 +784,7 @@ private:
                     [=] (CCallHelpers& jit, const StackmapGenerationParams& params) {
                         AllowMacroScratchRegisterUsage allowScratch(jit);
 
-                        using JumpTableCodePtr = MacroAssemblerCodePtr<JSSwitchPtrTag>;
+                        using JumpTableCodePtr = CodePtr<JSSwitchPtrTag>;
                         JumpTableCodePtr* jumpTable = static_cast<JumpTableCodePtr*>(
                             params.proc().addDataSection(sizeof(JumpTableCodePtr) * tableSize));
 
@@ -539,7 +792,7 @@ private:
                         GPRReg scratch = params.gpScratch(0);
 
                         jit.move(CCallHelpers::TrustedImmPtr(jumpTable), scratch);
-                        jit.load64(CCallHelpers::BaseIndex(scratch, index, CCallHelpers::ScalePtr), scratch);
+                        jit.loadPtr(CCallHelpers::BaseIndex(scratch, index, CCallHelpers::ScalePtr), scratch);
                         jit.farJump(scratch, JSSwitchPtrTag);
 
                         // These labels are guaranteed to be populated before either late paths or
@@ -642,12 +895,13 @@ private:
 
 bool lowerMacros(Procedure& proc)
 {
-    PhaseScope phaseScope(proc, "B3::lowerMacros");
+    PhaseScope phaseScope(proc, "B3::lowerMacros"_s);
     LowerMacros lowerMacros(proc);
     return lowerMacros.run();
 }
 
 } } // namespace JSC::B3
 
-#endif // ENABLE(B3_JIT)
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_END
 
+#endif // ENABLE(B3_JIT)
