@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2018 Apple Inc. All rights reserved.
+ * Copyright (C) 2013-2022 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,6 +31,8 @@
 #include "FTLState.h"
 #include "JSCPtrTag.h"
 
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_BEGIN
+
 namespace JSC { namespace FTL {
 
 using namespace B3;
@@ -44,15 +46,8 @@ JITCode::JITCode()
 JITCode::~JITCode()
 {
     if (FTL::shouldDumpDisassembly()) {
-        if (m_b3Code || m_arityCheckEntrypoint) {
-            dataLog("Destroying FTL JIT code at ");
-            CommaPrinter comma;
-            if (m_b3Code)
-                dataLog(comma, m_b3Code);
-            if (m_arityCheckEntrypoint)
-                dataLog(comma, m_arityCheckEntrypoint);
-            dataLog("\n");
-        }
+        if (m_b3Code)
+            dataLogLn("Destroying FTL JIT code at ", m_b3Code);
     }
 }
 
@@ -71,18 +66,18 @@ void JITCode::initializeAddressForCall(CodePtr<JSEntryPtrTag> address)
     m_addressForCall = address;
 }
 
-void JITCode::initializeArityCheckEntrypoint(CodeRef<JSEntryPtrTag> entrypoint)
+void JITCode::initializeAddressForArityCheck(CodePtr<JSEntryPtrTag> entrypoint)
 {
-    m_arityCheckEntrypoint = entrypoint;
+    m_addressForArityCheck = entrypoint;
 }
 
-JITCode::CodePtr<JSEntryPtrTag> JITCode::addressForCall(ArityCheckMode arityCheck)
+CodePtr<JSEntryPtrTag> JITCode::addressForCall(ArityCheckMode arityCheck)
 {
     switch (arityCheck) {
     case ArityCheckNotRequired:
         return m_addressForCall;
     case MustCheckArity:
-        return m_arityCheckEntrypoint.code();
+        return m_addressForArityCheck;
     }
     RELEASE_ASSERT_NOT_REACHED();
     return CodePtr<JSEntryPtrTag>();
@@ -91,9 +86,9 @@ JITCode::CodePtr<JSEntryPtrTag> JITCode::addressForCall(ArityCheckMode arityChec
 void* JITCode::executableAddressAtOffset(size_t offset)
 {
     if (!offset)
-        return m_addressForCall.executableAddress();
+        return m_addressForCall.taggedPtr();
 
-    char* executableAddress = m_addressForCall.untaggedExecutableAddress<char*>();
+    char* executableAddress = m_addressForCall.untaggedPtr<char*>();
     return tagCodePtr<JSEntryPtrTag>(executableAddress + offset);
 }
 
@@ -116,7 +111,7 @@ size_t JITCode::size()
 {
     // We don't know the size of FTL code, yet. Make a wild guess. This is mostly used for
     // GC load estimates.
-    return 1000;
+    return m_size;
 }
 
 bool JITCode::contains(void*)
@@ -136,7 +131,12 @@ DFG::CommonData* JITCode::dfgCommon()
     return &common;
 }
 
-void JITCode::shrinkToFit(const ConcurrentJSLocker&)
+const DFG::CommonData* JITCode::dfgCommon() const
+{
+    return &common;
+}
+
+void JITCode::shrinkToFit()
 {
     common.shrinkToFit();
     m_osrExit.shrinkToFit();
@@ -152,13 +152,13 @@ void JITCode::validateReferences(const TrackedReferences& trackedReferences)
         exit.m_descriptor->validateReferences(trackedReferences);
 }
 
-RegisterSet JITCode::liveRegistersToPreserveAtExceptionHandlingCallSite(CodeBlock*, CallSiteIndex callSiteIndex)
+RegisterSetBuilder JITCode::liveRegistersToPreserveAtExceptionHandlingCallSite(CodeBlock*, CallSiteIndex callSiteIndex)
 {
     for (OSRExit& exit : m_osrExit) {
         if (exit.m_exceptionHandlerCallSiteIndex.bits() == callSiteIndex.bits()) {
             RELEASE_ASSERT(exit.isExceptionHandler());
             RELEASE_ASSERT(exit.isGenericUnwindHandler());
-            return ValueRep::usedRegisters(exit.m_valueReps);
+            return ValueRep::usedRegisters(/* isSIMDContext = */ false, exit.m_valueReps);
         }
     }
     return { };
@@ -185,5 +185,6 @@ std::optional<CodeOrigin> JITCode::findPC(CodeBlock* codeBlock, void* pc)
 
 } } // namespace JSC::FTL
 
-#endif // ENABLE(FTL_JIT)
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_END
 
+#endif // ENABLE(FTL_JIT)
