@@ -8,6 +8,7 @@
 
 #include <limits>
 
+#include "common/span.h"
 #include "common/utilities.h"
 #include "compiler/preprocessor/numeric_lex.h"
 #include "compiler/translator/ImmutableStringBuilder.h"
@@ -74,6 +75,8 @@ bool IsInterpolationIn(TQualifier qualifier)
         case EvqNoPerspectiveIn:
         case EvqCentroidIn:
         case EvqSampleIn:
+        case EvqNoPerspectiveCentroidIn:
+        case EvqNoPerspectiveSampleIn:
             return true;
         default:
             return false;
@@ -89,6 +92,8 @@ bool IsInterpolationOut(TQualifier qualifier)
         case EvqNoPerspectiveOut:
         case EvqCentroidOut:
         case EvqSampleOut:
+        case EvqNoPerspectiveCentroidOut:
+        case EvqNoPerspectiveSampleOut:
             return true;
         default:
             return false;
@@ -213,7 +218,15 @@ float NumericLexFloat32OutOfRangeToInfinity(const std::string &str)
     {
         return std::numeric_limits<float>::infinity();
     }
-    else if (exponentLong < std::numeric_limits<float>::min_exponent10)
+    // In 32-bit float, min_exponent10 is -37 but min() is
+    // 1.1754943E-38. 10^-37 may be the "minimum negative integer such
+    // that 10 raised to that power is a normalized float", but being
+    // constrained to powers of ten it's above min() (which is 2^-126).
+    // Values below min() are flushed to zero near the end of this
+    // function anyway so (AFAICT) this comparison is only done to ensure
+    // that the exponent will not make the pow() call (below) overflow.
+    // Comparing against -38 (min_exponent10 - 1) will do the trick.
+    else if (exponentLong < std::numeric_limits<float>::min_exponent10 - 1)
     {
         return 0.0f;
     }
@@ -229,7 +242,7 @@ float NumericLexFloat32OutOfRangeToInfinity(const std::string &str)
     {
         return std::numeric_limits<float>::infinity();
     }
-    if (value < static_cast<double>(std::numeric_limits<float>::min()))
+    if (static_cast<float>(value) < std::numeric_limits<float>::min())
     {
         return 0.0f;
     }
@@ -278,6 +291,9 @@ GLenum GLVariableType(const TType &type)
 
             return kBoolGLType[type.getNominalSize() - 1];
 
+        case EbtYuvCscStandardEXT:
+            return GL_UNSIGNED_INT;
+
         case EbtSampler2D:
             return GL_SAMPLER_2D;
         case EbtSampler3D:
@@ -380,11 +396,16 @@ GLenum GLVariableType(const TType &type)
             return GL_UNSIGNED_INT_ATOMIC_COUNTER;
         case EbtSamplerVideoWEBGL:
             return GL_SAMPLER_VIDEO_IMAGE_WEBGL;
+        case EbtPixelLocalANGLE:
+        case EbtIPixelLocalANGLE:
+        case EbtUPixelLocalANGLE:
+            // TODO(anglebug.com/40096838): For now, we can expect PLS handles to be rewritten to
+            // images before anyone calls into here.
+            [[fallthrough]];
         default:
             UNREACHABLE();
+            return GL_NONE;
     }
-
-    return GL_NONE;
 }
 
 GLenum GLVariablePrecision(const TType &type)
@@ -399,9 +420,6 @@ GLenum GLVariablePrecision(const TType &type)
                 return GL_MEDIUM_FLOAT;
             case EbpLow:
                 return GL_LOW_FLOAT;
-            case EbpUndefined:
-                // Desktop specs do not use precision
-                return GL_NONE;
             default:
                 UNREACHABLE();
         }
@@ -416,9 +434,6 @@ GLenum GLVariablePrecision(const TType &type)
                 return GL_MEDIUM_INT;
             case EbpLow:
                 return GL_LOW_INT;
-            case EbpUndefined:
-                // Desktop specs do not use precision
-                return GL_NONE;
             default:
                 UNREACHABLE();
         }
@@ -433,7 +448,7 @@ ImmutableString ArrayString(const TType &type)
     if (!type.isArray())
         return ImmutableString("");
 
-    const TSpan<const unsigned int> &arraySizes     = type.getArraySizes();
+    const angle::Span<const unsigned int> &arraySizes = type.getArraySizes();
     constexpr const size_t kMaxDecimalDigitsPerSize = 10u;
     ImmutableStringBuilder arrayString(arraySizes.size() * (kMaxDecimalDigitsPerSize + 2u));
     for (auto arraySizeIter = arraySizes.rbegin(); arraySizeIter != arraySizes.rend();
@@ -442,7 +457,7 @@ ImmutableString ArrayString(const TType &type)
         arrayString << "[";
         if (*arraySizeIter > 0)
         {
-            arrayString.appendDecimal(*arraySizeIter);
+            arrayString << *arraySizeIter;
         }
         arrayString << "]";
     }
@@ -462,15 +477,10 @@ bool IsVaryingOut(TQualifier qualifier)
     switch (qualifier)
     {
         case EvqVaryingOut:
-        case EvqSmoothOut:
-        case EvqFlatOut:
-        case EvqNoPerspectiveOut:
-        case EvqCentroidOut:
         case EvqVertexOut:
         case EvqGeometryOut:
         case EvqTessControlOut:
         case EvqTessEvaluationOut:
-        case EvqSampleOut:
         case EvqPatchOut:
             return true;
 
@@ -478,7 +488,7 @@ bool IsVaryingOut(TQualifier qualifier)
             break;
     }
 
-    return false;
+    return IsInterpolationOut(qualifier);
 }
 
 bool IsVaryingIn(TQualifier qualifier)
@@ -486,15 +496,10 @@ bool IsVaryingIn(TQualifier qualifier)
     switch (qualifier)
     {
         case EvqVaryingIn:
-        case EvqSmoothIn:
-        case EvqFlatIn:
-        case EvqNoPerspectiveIn:
-        case EvqCentroidIn:
         case EvqFragmentIn:
         case EvqGeometryIn:
         case EvqTessControlIn:
         case EvqTessEvaluationIn:
-        case EvqSampleIn:
         case EvqPatchIn:
             return true;
 
@@ -502,7 +507,7 @@ bool IsVaryingIn(TQualifier qualifier)
             break;
     }
 
-    return false;
+    return IsInterpolationIn(qualifier);
 }
 
 bool IsVarying(TQualifier qualifier)
@@ -569,6 +574,14 @@ InterpolationType GetInterpolationType(TQualifier qualifier)
         case EvqNoPerspectiveOut:
             return INTERPOLATION_NOPERSPECTIVE;
 
+        case EvqNoPerspectiveCentroidIn:
+        case EvqNoPerspectiveCentroidOut:
+            return INTERPOLATION_NOPERSPECTIVE_CENTROID;
+
+        case EvqNoPerspectiveSampleIn:
+        case EvqNoPerspectiveSampleOut:
+            return INTERPOLATION_NOPERSPECTIVE_SAMPLE;
+
         case EvqSmoothIn:
         case EvqSmoothOut:
         case EvqVertexOut:
@@ -592,9 +605,7 @@ InterpolationType GetInterpolationType(TQualifier qualifier)
             return INTERPOLATION_SAMPLE;
         default:
             UNREACHABLE();
-#if !UNREACHABLE_IS_NORETURN
             return INTERPOLATION_SMOOTH;
-#endif
     }
 }
 
@@ -603,14 +614,20 @@ InterpolationType GetFieldInterpolationType(TQualifier qualifier)
 {
     switch (qualifier)
     {
+        case EvqSmooth:
+            return INTERPOLATION_SMOOTH;
         case EvqFlat:
             return INTERPOLATION_FLAT;
         case EvqNoPerspective:
             return INTERPOLATION_NOPERSPECTIVE;
-        case EvqSmooth:
-            return INTERPOLATION_SMOOTH;
         case EvqCentroid:
             return INTERPOLATION_CENTROID;
+        case EvqSample:
+            return INTERPOLATION_SAMPLE;
+        case EvqNoPerspectiveCentroid:
+            return INTERPOLATION_NOPERSPECTIVE_CENTROID;
+        case EvqNoPerspectiveSample:
+            return INTERPOLATION_NOPERSPECTIVE_SAMPLE;
         default:
             return GetInterpolationType(qualifier);
     }
@@ -672,9 +689,7 @@ TType GetShaderVariableBasicType(const sh::ShaderVariable &var)
             return TType(EbtUInt, 4);
         default:
             UNREACHABLE();
-#if !UNREACHABLE_IS_NORETURN
             return TType();
-#endif
     }
 }
 
@@ -717,6 +732,7 @@ bool IsBuiltinOutputVariable(TQualifier qualifier)
         case EvqClipDistance:
         case EvqCullDistance:
         case EvqLastFragData:
+        case EvqLastFragColor:
         case EvqSampleMask:
             return true;
         default:
@@ -734,6 +750,9 @@ bool IsBuiltinFragmentInputVariable(TQualifier qualifier)
         case EvqFrontFacing:
         case EvqHelperInvocation:
         case EvqLastFragData:
+        case EvqLastFragColor:
+        case EvqLastFragDepth:
+        case EvqLastFragStencil:
             return true;
         default:
             break;
@@ -746,6 +765,23 @@ bool IsShaderOutput(TQualifier qualifier)
     return IsVaryingOut(qualifier) || IsBuiltinOutputVariable(qualifier);
 }
 
+bool IsFragmentOutput(TQualifier qualifier)
+{
+    switch (qualifier)
+    {
+        case EvqFragmentOut:
+        case EvqFragmentInOut:
+            return true;
+        default:
+            return false;
+    }
+}
+
+bool IsOutputNULL(ShShaderOutput output)
+{
+    return output == SH_NULL_OUTPUT;
+}
+
 bool IsOutputESSL(ShShaderOutput output)
 {
     return output == SH_ESSL_OUTPUT;
@@ -778,25 +814,24 @@ bool IsOutputHLSL(ShShaderOutput output)
     {
         case SH_HLSL_3_0_OUTPUT:
         case SH_HLSL_4_1_OUTPUT:
-        case SH_HLSL_4_0_FL9_3_OUTPUT:
             return true;
         default:
             break;
     }
     return false;
 }
-bool IsOutputVulkan(ShShaderOutput output)
+bool IsOutputSPIRV(ShShaderOutput output)
 {
     return output == SH_SPIRV_VULKAN_OUTPUT;
 }
-bool IsOutputMetal(ShShaderOutput output)
-{
-    return output == SH_SPIRV_METAL_OUTPUT;
-}
-bool IsOutputMetalDirect(ShShaderOutput output)
+bool IsOutputMSL(ShShaderOutput output)
 {
     return output == SH_MSL_METAL_OUTPUT;
 }
+bool IsOutputWGSL(ShShaderOutput output)
+{
+    return output == SH_WGSL_OUTPUT;
+}
 
 bool IsInShaderStorageBlock(TIntermTyped *node)
 {
@@ -865,114 +900,6 @@ bool IsSpecWithFunctionBodyNewScope(ShShaderSpec shaderSpec, int shaderVersion)
     return (shaderVersion == 100 && !sh::IsWebGLBasedSpec(shaderSpec));
 }
 
-ImplicitTypeConversion GetConversion(TBasicType t1, TBasicType t2)
-{
-    if (t1 == t2)
-        return ImplicitTypeConversion::Same;
-
-    switch (t1)
-    {
-        case EbtInt:
-            switch (t2)
-            {
-                case EbtInt:
-                    UNREACHABLE();
-                    break;
-                case EbtUInt:
-                    return ImplicitTypeConversion::Invalid;
-                case EbtFloat:
-                    return ImplicitTypeConversion::Left;
-                default:
-                    return ImplicitTypeConversion::Invalid;
-            }
-            break;
-        case EbtUInt:
-            switch (t2)
-            {
-                case EbtInt:
-                    return ImplicitTypeConversion::Invalid;
-                case EbtUInt:
-                    UNREACHABLE();
-                    break;
-                case EbtFloat:
-                    return ImplicitTypeConversion::Left;
-                default:
-                    return ImplicitTypeConversion::Invalid;
-            }
-            break;
-        case EbtFloat:
-            switch (t2)
-            {
-                case EbtInt:
-                case EbtUInt:
-                    return ImplicitTypeConversion::Right;
-                case EbtFloat:
-                    UNREACHABLE();
-                    break;
-                default:
-                    return ImplicitTypeConversion::Invalid;
-            }
-            break;
-        default:
-            return ImplicitTypeConversion::Invalid;
-    }
-    return ImplicitTypeConversion::Invalid;
-}
-
-bool IsValidImplicitConversion(sh::ImplicitTypeConversion conversion, TOperator op)
-{
-    switch (conversion)
-    {
-        case sh::ImplicitTypeConversion::Same:
-            return true;
-        case sh::ImplicitTypeConversion::Left:
-            switch (op)
-            {
-                case EOpEqual:
-                case EOpNotEqual:
-                case EOpLessThan:
-                case EOpGreaterThan:
-                case EOpLessThanEqual:
-                case EOpGreaterThanEqual:
-                case EOpAdd:
-                case EOpSub:
-                case EOpMul:
-                case EOpDiv:
-                    return true;
-                default:
-                    break;
-            }
-            break;
-        case sh::ImplicitTypeConversion::Right:
-            switch (op)
-            {
-                case EOpAssign:
-                case EOpInitialize:
-                case EOpEqual:
-                case EOpNotEqual:
-                case EOpLessThan:
-                case EOpGreaterThan:
-                case EOpLessThanEqual:
-                case EOpGreaterThanEqual:
-                case EOpAdd:
-                case EOpSub:
-                case EOpMul:
-                case EOpDiv:
-                case EOpAddAssign:
-                case EOpSubAssign:
-                case EOpMulAssign:
-                case EOpDivAssign:
-                    return true;
-                default:
-                    break;
-            }
-            break;
-        case sh::ImplicitTypeConversion::Invalid:
-            break;
-    }
-    return false;
-}
-
 bool IsPrecisionApplicableToType(TBasicType type)
 {
     switch (type)
@@ -981,7 +908,7 @@ bool IsPrecisionApplicableToType(TBasicType type)
         case EbtUInt:
         case EbtFloat:
             // TODO: find all types where precision is applicable; for example samplers.
-            // http://anglebug.com/6132
+            // http://anglebug.com/42264661
             return true;
         default:
             return false;
@@ -990,7 +917,10 @@ bool IsPrecisionApplicableToType(TBasicType type)
 
 bool IsRedeclarableBuiltIn(const ImmutableString &name)
 {
-    return name == "gl_ClipDistance" || name == "gl_CullDistance" || name == "gl_LastFragData";
+    return name == "gl_ClipDistance" || name == "gl_CullDistance" || name == "gl_FragDepth" ||
+           name == "gl_LastFragData" || name == "gl_LastFragColorARM" ||
+           name == "gl_LastFragDepthARM" || name == "gl_LastFragStencilARM" ||
+           name == "gl_PerVertex" || name == "gl_Position" || name == "gl_PointSize";
 }
 
 size_t FindFieldIndex(const TFieldList &fieldList, const char *fieldName)
@@ -1006,4 +936,25 @@ size_t FindFieldIndex(const TFieldList &fieldList, const char *fieldName)
     return 0;
 }
 
+Declaration ViewDeclaration(TIntermDeclaration &declNode, uint32_t index)
+{
+    ASSERT(declNode.getChildCount() > index);
+    TIntermNode *childNode = declNode.getChildNode(index);
+    ASSERT(childNode);
+    TIntermSymbol *symbolNode;
+    if ((symbolNode = childNode->getAsSymbolNode()))
+    {
+        return {*symbolNode, nullptr};
+    }
+    else
+    {
+        TIntermBinary *initNode = childNode->getAsBinaryNode();
+        ASSERT(initNode);
+        ASSERT(initNode->getOp() == TOperator::EOpInitialize);
+        symbolNode = initNode->getLeft()->getAsSymbolNode();
+        ASSERT(symbolNode);
+        return {*symbolNode, initNode->getRight()};
+    }
+}
+
 }  // namespace sh
