@@ -11,9 +11,12 @@
 #ifndef LIBANGLE_UNIFORMLINKER_H_
 #define LIBANGLE_UNIFORMLINKER_H_
 
+#include <GLSLANG/ShaderVars.h>
+
 #include "angle_gl.h"
 #include "common/PackedEnums.h"
 #include "common/angleutils.h"
+#include "libANGLE/Uniform.h"
 #include "libANGLE/VaryingPacking.h"
 
 #include <functional>
@@ -37,22 +40,83 @@ class Context;
 class InfoLog;
 struct InterfaceBlock;
 enum class LinkMismatchError;
-struct LinkedUniform;
 class ProgramState;
 class ProgramPipelineState;
 class ProgramBindings;
 class ProgramAliasedBindings;
 class Shader;
-struct ShaderVariableBuffer;
-struct UnusedUniform;
+struct AtomicCounterBuffer;
 struct VariableLocation;
+struct Version;
 
-using AtomicCounterBuffer = ShaderVariableBuffer;
+using ShaderUniform = std::pair<ShaderType, const sh::ShaderVariable *>;
+
+// The link operation is responsible for finishing the link of uniform and interface blocks.
+// This way it can filter out unreferenced resources and still have access to the info.
+// TODO(jmadill): Integrate uniform linking/filtering as well as interface blocks.
+struct UnusedUniform
+{
+    UnusedUniform(std::string name,
+                  bool isSampler,
+                  bool isImage,
+                  bool isAtomicCounter,
+                  bool isFragmentInOut)
+    {
+        this->name            = name;
+        this->isSampler       = isSampler;
+        this->isImage         = isImage;
+        this->isAtomicCounter = isAtomicCounter;
+        this->isFragmentInOut = isFragmentInOut;
+    }
+
+    std::string name;
+    bool isSampler;
+    bool isImage;
+    bool isAtomicCounter;
+    bool isFragmentInOut;
+};
+
+struct UsedUniform : public sh::ShaderVariable
+{
+    UsedUniform();
+    UsedUniform(GLenum type,
+                GLenum precision,
+                const std::string &name,
+                const std::vector<unsigned int> &arraySizes,
+                const int binding,
+                const int offset,
+                const int location,
+                const int bufferIndex,
+                const sh::BlockMemberInfo &blockInfo);
+    UsedUniform(const UsedUniform &other);
+    UsedUniform &operator=(const UsedUniform &other);
+    ~UsedUniform();
+
+    bool isSampler() const { return typeInfo->isSampler; }
+    bool isImage() const { return typeInfo->isImageType; }
+    bool isAtomicCounter() const { return IsAtomicCounterType(type); }
+    size_t getElementSize() const { return typeInfo->externalSize; }
+
+    void setActive(ShaderType shaderType, bool used, uint32_t _id)
+    {
+        activeVariable.setActive(shaderType, used, _id);
+    }
+
+    ActiveVariable activeVariable;
+    const UniformTypeInfo *typeInfo;
+
+    // Identifies the containing buffer backed resource -- interface block or atomic counter buffer.
+    int bufferIndex;
+    sh::BlockMemberInfo blockInfo;
+    std::vector<unsigned int> outerArraySizes;
+    unsigned int outerArrayOffset;
+};
 
 class UniformLinker final : angle::NonCopyable
 {
   public:
-    UniformLinker(const ProgramState &state);
+    UniformLinker(const ShaderBitSet &activeShaderStages,
+                  const ShaderMap<std::vector<sh::ShaderVariable>> &shaderUniforms);
     ~UniformLinker();
 
     bool link(const Caps &caps,
@@ -60,18 +124,23 @@ class UniformLinker final : angle::NonCopyable
               const ProgramAliasedBindings &uniformLocationBindings);
 
     void getResults(std::vector<LinkedUniform> *uniforms,
-                    std::vector<UnusedUniform> *unusedUniforms,
-                    std::vector<VariableLocation> *uniformLocations);
+                    std::vector<std::string> *uniformNames,
+                    std::vector<std::string> *uniformMappedNames,
+                    std::vector<UnusedUniform> *unusedUniformsOutOrNull,
+                    std::vector<VariableLocation> *uniformLocationsOutOrNull);
 
   private:
     bool validateGraphicsUniforms(InfoLog &infoLog) const;
-
-    bool flattenUniformsAndCheckCapsForShader(Shader *shader,
+    bool validateGraphicsUniformsPerShader(ShaderType shaderToLink,
+                                           bool extendLinkedUniforms,
+                                           std::map<std::string, ShaderUniform> *linkedUniforms,
+                                           InfoLog &infoLog) const;
+    bool flattenUniformsAndCheckCapsForShader(ShaderType shaderType,
                                               const Caps &caps,
-                                              std::vector<LinkedUniform> &samplerUniforms,
-                                              std::vector<LinkedUniform> &imageUniforms,
-                                              std::vector<LinkedUniform> &atomicCounterUniforms,
-                                              std::vector<LinkedUniform> &inputAttachmentUniforms,
+                                              std::vector<UsedUniform> &samplerUniforms,
+                                              std::vector<UsedUniform> &imageUniforms,
+                                              std::vector<UsedUniform> &atomicCounterUniforms,
+                                              std::vector<UsedUniform> &inputAttachmentUniforms,
                                               std::vector<UnusedUniform> &unusedUniforms,
                                               InfoLog &infoLog);
 
@@ -86,8 +155,9 @@ class UniformLinker final : angle::NonCopyable
         int *maxUniformLocation);
     void pruneUnusedUniforms();
 
-    const ProgramState &mState;
-    std::vector<LinkedUniform> mUniforms;
+    ShaderBitSet mActiveShaderStages;
+    const ShaderMap<std::vector<sh::ShaderVariable>> &mShaderUniforms;
+    std::vector<UsedUniform> mUniforms;
     std::vector<UnusedUniform> mUnusedUniforms;
     std::vector<VariableLocation> mUniformLocations;
 };
@@ -113,6 +183,12 @@ class InterfaceBlockLinker : angle::NonCopyable
     void linkBlocks(const GetBlockSizeFunc &getBlockSize,
                     const GetBlockMemberInfoFunc &getMemberInfo) const;
 
+    const std::vector<sh::InterfaceBlock> &getShaderBlocks(ShaderType shaderType) const
+    {
+        ASSERT(mShaderBlocks[shaderType]);
+        return *mShaderBlocks[shaderType];
+    }
+
   protected:
     InterfaceBlockLinker();
     void init(std::vector<InterfaceBlock> *blocksOut,
@@ -144,6 +220,8 @@ class UniformBlockLinker final : public InterfaceBlockLinker
 
     void init(std::vector<InterfaceBlock> *blocksOut,
               std::vector<LinkedUniform> *uniformsOut,
+              std::vector<std::string> *uniformNamesOut,
+              std::vector<std::string> *uniformMappedNamesOut,
               std::vector<std::string> *unusedInterfaceBlocksOut);
 
   private:
@@ -155,7 +233,9 @@ class UniformBlockLinker final : public InterfaceBlockLinker
                                           ShaderType shaderType,
                                           int blockIndex) const override;
 
-    std::vector<LinkedUniform> *mUniformsOut = nullptr;
+    std::vector<LinkedUniform> *mUniformsOut         = nullptr;
+    std::vector<std::string> *mUniformNamesOut       = nullptr;
+    std::vector<std::string> *mUniformMappedNamesOut = nullptr;
 };
 
 class ShaderStorageBlockLinker final : public InterfaceBlockLinker
@@ -193,29 +273,17 @@ class AtomicCounterBufferLinker final : angle::NonCopyable
     std::vector<AtomicCounterBuffer> *mAtomicCounterBuffersOut = nullptr;
 };
 
-// The link operation is responsible for finishing the link of uniform and interface blocks.
-// This way it can filter out unreferenced resources and still have access to the info.
-// TODO(jmadill): Integrate uniform linking/filtering as well as interface blocks.
-struct UnusedUniform
+class PixelLocalStorageLinker final : angle::NonCopyable
 {
-    UnusedUniform(std::string name,
-                  bool isSampler,
-                  bool isImage,
-                  bool isAtomicCounter,
-                  bool isFragmentInOut)
-    {
-        this->name            = name;
-        this->isSampler       = isSampler;
-        this->isImage         = isImage;
-        this->isAtomicCounter = isAtomicCounter;
-        this->isFragmentInOut = isFragmentInOut;
-    }
+  public:
+    PixelLocalStorageLinker();
+    ~PixelLocalStorageLinker();
 
-    std::string name;
-    bool isSampler;
-    bool isImage;
-    bool isAtomicCounter;
-    bool isFragmentInOut;
+    void init(std::vector<ShPixelLocalStorageFormat> *pixelLocalStorageFormatsOut);
+    void link(const std::vector<ShPixelLocalStorageFormat> &pixelLocalStorageFormats) const;
+
+  private:
+    std::vector<ShPixelLocalStorageFormat> *mPixelLocalStorageFormatsOut = nullptr;
 };
 
 struct ProgramLinkedResources
@@ -225,24 +293,30 @@ struct ProgramLinkedResources
 
     void init(std::vector<InterfaceBlock> *uniformBlocksOut,
               std::vector<LinkedUniform> *uniformsOut,
+              std::vector<std::string> *uniformNamesOut,
+              std::vector<std::string> *uniformMappedNamesOut,
               std::vector<InterfaceBlock> *shaderStorageBlocksOut,
               std::vector<BufferVariable> *bufferVariablesOut,
-              std::vector<AtomicCounterBuffer> *atomicCounterBuffersOut);
+              std::vector<AtomicCounterBuffer> *atomicCounterBuffersOut,
+              std::vector<ShPixelLocalStorageFormat> *pixelLocalStorageFormatsOut);
 
     ProgramVaryingPacking varyingPacking;
     UniformBlockLinker uniformBlockLinker;
     ShaderStorageBlockLinker shaderStorageBlockLinker;
     AtomicCounterBufferLinker atomicCounterBufferLinker;
+    PixelLocalStorageLinker pixelLocalStorageLinker;
     std::vector<UnusedUniform> unusedUniforms;
     std::vector<std::string> unusedInterfaceBlocks;
 };
 
 struct LinkingVariables final : private angle::NonCopyable
 {
-    LinkingVariables(const ProgramState &state);
-    LinkingVariables(const ProgramPipelineState &state);
+    LinkingVariables();
     ~LinkingVariables();
 
+    void initForProgram(const ProgramState &state);
+    void initForProgramPipeline(const ProgramPipelineState &state);
+
     ShaderMap<std::vector<sh::ShaderVariable>> outputVaryings;
     ShaderMap<std::vector<sh::ShaderVariable>> inputVaryings;
     ShaderMap<std::vector<sh::ShaderVariable>> uniforms;
@@ -271,12 +345,15 @@ class ProgramLinkedResourcesLinker final : angle::NonCopyable
                        const ProgramLinkedResources &resources) const;
 
   private:
-    void getAtomicCounterBufferSizeMap(const ProgramState &programState,
+    void getAtomicCounterBufferSizeMap(const ProgramExecutable &executable,
                                        std::map<int, unsigned int> &sizeMapOut) const;
 
     CustomBlockLayoutEncoderFactory *mCustomEncoderFactory;
 };
 
+using ShaderInterfaceBlock = std::pair<ShaderType, const sh::InterfaceBlock *>;
+using InterfaceBlockMap    = std::map<std::string, ShaderInterfaceBlock>;
+
 bool LinkValidateProgramGlobalNames(InfoLog &infoLog,
                                     const ProgramExecutable &executable,
                                     const LinkingVariables &linkingVariables);
@@ -307,6 +384,13 @@ LinkMismatchError LinkValidateProgramVariables(const sh::ShaderVariable &variabl
                                                std::string *mismatchedStructOrBlockMemberName);
 void AddProgramVariableParentPrefix(const std::string &parentName,
                                     std::string *mismatchedFieldName);
+bool LinkValidateProgramInterfaceBlocks(const Caps &caps,
+                                        const Version &clientVersion,
+                                        bool webglCompatibility,
+                                        ShaderBitSet activeProgramStages,
+                                        const ProgramLinkedResources &resources,
+                                        InfoLog &infoLog,
+                                        GLuint *combinedShaderStorageBlocksCountOut);
 }  // namespace gl
 
 #endif  // LIBANGLE_UNIFORMLINKER_H_
