@@ -13,8 +13,9 @@
 #include "libANGLE/Error.h"
 #include "libANGLE/State.h"
 #include "libANGLE/angletypes.h"
+#include "libANGLE/renderer/gl/ProgramExecutableGL.h"
 #include "libANGLE/renderer/gl/functionsgl_typedefs.h"
-#include "platform/FeaturesGL_autogen.h"
+#include "platform/autogen/FeaturesGL_autogen.h"
 
 #include <array>
 #include <map>
@@ -35,6 +36,16 @@ class TransformFeedbackGL;
 class VertexArrayGL;
 class QueryGL;
 
+struct ExternalContextVertexAttribute
+{
+    bool enabled;
+    const angle::Format *format;
+    GLuint stride;
+    GLvoid *pointer;
+    GLuint buffer;
+    gl::VertexAttribCurrentValueData currentData;
+};
+
 // TODO(penghuang): use gl::State?
 struct ExternalContextState
 {
@@ -59,13 +70,19 @@ struct ExternalContextState
     GLfloat lineWidth;
     GLfloat polygonOffsetFactor;
     GLfloat polygonOffsetUnits;
+    GLfloat polygonOffsetClamp;
     GLfloat sampleCoverageValue;
     bool sampleCoverageInvert;
     GLenum blendEquationRgb;
     GLenum blendEquationAlpha;
+    bool enableBlendEquationAdvancedCoherent;
 
     bool enableDither;
+    GLenum polygonMode;
+    bool enablePolygonOffsetPoint;
+    bool enablePolygonOffsetLine;
     bool enablePolygonOffsetFill;
+    bool enableDepthClamp;
     bool enableSampleAlphaToCoverage;
     bool enableSampleCoverage;
     bool multisampleEnabled;
@@ -77,6 +94,8 @@ struct ExternalContextState
     GLenum blendDestAlpha;
     GLenum activeTexture;
     gl::Rectangle viewport;
+    GLenum clipOrigin;
+    GLenum clipDepthMode;
     bool scissorTest;
     gl::Rectangle scissorBox;
 
@@ -113,6 +132,9 @@ struct ExternalContextState
     std::vector<TextureBindings> textureBindings;
 
     GLenum vertexArrayBinding;
+
+    angle::FixedVector<ExternalContextVertexAttribute, gl::MAX_VERTEX_ATTRIBS>
+        defaultVertexArrayAttributes;
 };
 
 struct VertexAttributeGL
@@ -198,12 +220,14 @@ class StateManagerGL final : angle::NonCopyable
 
     void setViewport(const gl::Rectangle &viewport);
     void setDepthRange(float near, float far);
+    void setClipControl(gl::ClipOrigin origin, gl::ClipDepthMode depth);
 
     void setBlendEnabled(bool enabled);
     void setBlendEnabledIndexed(const gl::DrawBufferMask blendEnabledMask);
     void setBlendColor(const gl::ColorF &blendColor);
     void setBlendFuncs(const gl::BlendStateExt &blendStateExt);
     void setBlendEquations(const gl::BlendStateExt &blendStateExt);
+    void setBlendAdvancedCoherent(bool enabled);
     void setColorMask(bool red, bool green, bool blue, bool alpha);
     void setSampleAlphaToCoverageEnabled(bool enabled);
     void setSampleCoverageEnabled(bool enabled);
@@ -225,8 +249,12 @@ class StateManagerGL final : angle::NonCopyable
     void setCullFaceEnabled(bool enabled);
     void setCullFace(gl::CullFaceMode cullFace);
     void setFrontFace(GLenum frontFace);
+    void setPolygonMode(gl::PolygonMode mode);
+    void setPolygonOffsetPointEnabled(bool enabled);
+    void setPolygonOffsetLineEnabled(bool enabled);
     void setPolygonOffsetFillEnabled(bool enabled);
-    void setPolygonOffset(float factor, float units);
+    void setPolygonOffset(float factor, float units, float clamp);
+    void setDepthClampEnabled(bool enabled);
     void setRasterizerDiscardEnabled(bool enabled);
     void setLineWidth(float width);
 
@@ -259,7 +287,10 @@ class StateManagerGL final : angle::NonCopyable
 
     void setProvokingVertex(GLenum mode);
 
-    void setClipDistancesEnable(const gl::State::ClipDistanceEnableBits &enables);
+    void setClipDistancesEnable(const gl::ClipDistanceEnableBits &enables);
+
+    void setLogicOpEnabled(bool enabled);
+    void setLogicOp(gl::LogicalOperation opcode);
 
     void pauseTransformFeedback();
     angle::Result pauseAllQueries(const gl::Context *context);
@@ -269,19 +300,28 @@ class StateManagerGL final : angle::NonCopyable
     angle::Result onMakeCurrent(const gl::Context *context);
 
     angle::Result syncState(const gl::Context *context,
-                            const gl::State::DirtyBits &glDirtyBits,
-                            const gl::State::DirtyBits &bitMask);
+                            const gl::state::DirtyBits &glDirtyBits,
+                            const gl::state::DirtyBits &bitMask,
+                            const gl::state::ExtendedDirtyBits &extendedDirtyBits,
+                            const gl::state::ExtendedDirtyBits &extendedBitMask);
 
     ANGLE_INLINE void updateMultiviewBaseViewLayerIndexUniform(
-        const gl::Program *program,
+        const gl::ProgramExecutable *executable,
         const gl::FramebufferState &drawFramebufferState) const
     {
-        if (mIsMultiviewEnabled && program && program->usesMultiview())
+        if (mIsMultiviewEnabled && executable && executable->usesMultiview())
         {
-            updateMultiviewBaseViewLayerIndexUniformImpl(program, drawFramebufferState);
+            updateMultiviewBaseViewLayerIndexUniformImpl(executable, drawFramebufferState);
         }
     }
 
+    ANGLE_INLINE void updateEmulatedClipOriginUniform(const gl::ProgramExecutable *executable,
+                                                      const gl::ClipOrigin origin) const
+    {
+        ASSERT(executable);
+        GetImplAs<ProgramExecutableGL>(executable)->updateEmulatedClipOrigin(origin);
+    }
+
     GLuint getProgramID() const { return mProgram; }
     GLuint getVertexArrayID() const { return mVAO; }
     GLuint getFramebufferID(angle::FramebufferBinding binding) const
@@ -302,10 +342,17 @@ class StateManagerGL final : angle::NonCopyable
     void restoreNativeContext(const gl::Extensions &extensions, const ExternalContextState *state);
 
   private:
+    void forceBindVertexArray(GLuint vao, VertexArrayStateGL *vaoState);
+
     void setTextureCubemapSeamlessEnabled(bool enabled);
 
+    void setClipControlWithEmulatedClipOrigin(const gl::ProgramExecutable *executable,
+                                              GLenum frontFace,
+                                              gl::ClipOrigin origin,
+                                              gl::ClipDepthMode depth);
+
     angle::Result propagateProgramToVAO(const gl::Context *context,
-                                        const gl::Program *program,
+                                        const gl::ProgramExecutable *executable,
                                         VertexArrayGL *vao);
 
     void updateProgramTextureBindings(const gl::Context *context);
@@ -326,8 +373,11 @@ class StateManagerGL final : angle::NonCopyable
     void syncSamplersState(const gl::Context *context);
     void syncTransformFeedbackState(const gl::Context *context);
 
+    void updateEmulatedClipDistanceState(const gl::ProgramExecutable *executable,
+                                         const gl::ClipDistanceEnableBits enables) const;
+
     void updateMultiviewBaseViewLayerIndexUniformImpl(
-        const gl::Program *program,
+        const gl::ProgramExecutable *executable,
         const gl::FramebufferState &drawFramebufferState) const;
 
     void syncBlendFromNativeContext(const gl::Extensions &extensions, ExternalContextState *state);
@@ -369,6 +419,7 @@ class StateManagerGL final : angle::NonCopyable
 
     GLuint mProgram;
 
+    const bool mSupportsVertexArrayObjects;
     GLuint mVAO;
     std::vector<gl::VertexAttribCurrentValueData> mVertexAttribCurrentValues;
 
@@ -441,6 +492,8 @@ class StateManagerGL final : angle::NonCopyable
     // TODO(jmadill): Convert to std::array when available
     std::vector<GLenum> mFramebuffers;
     GLuint mRenderbuffer;
+    GLuint mPlaceholderFbo;
+    GLuint mPlaceholderRbo;
 
     bool mScissorTestEnabled;
     gl::Rectangle mScissor;
@@ -448,8 +501,12 @@ class StateManagerGL final : angle::NonCopyable
     float mNear;
     float mFar;
 
+    gl::ClipOrigin mClipOrigin;
+    gl::ClipDepthMode mClipDepthMode;
+
     gl::ColorF mBlendColor;
     gl::BlendStateExt mBlendStateExt;
+    bool mBlendAdvancedCoherent;
     const bool mIndependentBlendStates;
 
     bool mSampleAlphaToCoverageEnabled;
@@ -481,9 +538,14 @@ class StateManagerGL final : angle::NonCopyable
     bool mCullFaceEnabled;
     gl::CullFaceMode mCullFace;
     GLenum mFrontFace;
+    gl::PolygonMode mPolygonMode;
+    bool mPolygonOffsetPointEnabled;
+    bool mPolygonOffsetLineEnabled;
     bool mPolygonOffsetFillEnabled;
     GLfloat mPolygonOffsetFactor;
     GLfloat mPolygonOffsetUnits;
+    GLfloat mPolygonOffsetClamp;
+    bool mDepthClampEnabled;
     bool mRasterizerDiscardEnabled;
     float mLineWidth;
 
@@ -510,10 +572,14 @@ class StateManagerGL final : angle::NonCopyable
 
     GLenum mProvokingVertex;
 
-    gl::State::ClipDistanceEnableBits mEnabledClipDistances;
+    gl::ClipDistanceEnableBits mEnabledClipDistances;
     const size_t mMaxClipDistances;
 
-    gl::State::DirtyBits mLocalDirtyBits;
+    bool mLogicOpEnabled;
+    gl::LogicalOperation mLogicOp;
+
+    gl::state::DirtyBits mLocalDirtyBits;
+    gl::state::ExtendedDirtyBits mLocalExtendedDirtyBits;
     gl::AttributesMask mLocalDirtyCurrentValues;
 };
 
