@@ -16,6 +16,14 @@
 #include "libANGLE/renderer/gl/FunctionsGL.h"
 #include "libANGLE/renderer/gl/egl/functionsegl_typedefs.h"
 
+#if defined(ANGLE_HAS_LIBDRM)
+// clang-format off
+#include <fcntl.h>
+#include <unistd.h>
+#include <xf86drm.h>
+// clang-format on
+#endif  // defined(ANGLE_HAS_LIBDRM)
+
 namespace
 {
 
@@ -29,6 +37,20 @@ bool SetPtr(T *dst, void *src)
     }
     return false;
 }
+
+bool IsValidPlatformTypeForPlatformDisplayConnection(EGLAttrib platformType)
+{
+    switch (platformType)
+    {
+        case EGL_PLATFORM_SURFACELESS_MESA:
+        case EGL_PLATFORM_GBM_KHR:
+            return true;
+        default:
+            break;
+    }
+    return false;
+}
+
 }  // namespace
 
 namespace rx
@@ -85,7 +107,14 @@ struct FunctionsEGL::EGLDispatchTable
           getFrameTimestampSupportedANDROIDPtr(nullptr),
           getFrameTimestampsANDROIDPtr(nullptr),
 
-          dupNativeFenceFDANDROIDPtr(nullptr)
+          dupNativeFenceFDANDROIDPtr(nullptr),
+
+          queryDmaBufFormatsEXTPtr(nullptr),
+          queryDmaBufModifiersEXTPtr(nullptr),
+
+          queryDeviceAttribEXTPtr(nullptr),
+          queryDeviceStringEXTPtr(nullptr),
+          queryDisplayAttribEXTPtr(nullptr)
     {}
 
     // 1.0
@@ -148,6 +177,15 @@ struct FunctionsEGL::EGLDispatchTable
 
     // EGL_ANDROID_native_fence_sync
     PFNEGLDUPNATIVEFENCEFDANDROIDPROC dupNativeFenceFDANDROIDPtr;
+
+    // EGL_EXT_image_dma_buf_import_modifiers
+    PFNEGLQUERYDMABUFFORMATSEXTPROC queryDmaBufFormatsEXTPtr;
+    PFNEGLQUERYDMABUFMODIFIERSEXTPROC queryDmaBufModifiersEXTPtr;
+
+    // EGL_EXT_device_query
+    PFNEGLQUERYDEVICEATTRIBEXTPROC queryDeviceAttribEXTPtr;
+    PFNEGLQUERYDEVICESTRINGEXTPROC queryDeviceStringEXTPtr;
+    PFNEGLQUERYDISPLAYATTRIBEXTPROC queryDisplayAttribEXTPtr;
 };
 
 FunctionsEGL::FunctionsEGL()
@@ -159,8 +197,16 @@ FunctionsEGL::~FunctionsEGL()
     SafeDelete(mFnPtrs);
 }
 
-egl::Error FunctionsEGL::initialize(EGLNativeDisplayType nativeDisplay)
+egl::Error FunctionsEGL::initialize(EGLAttrib platformType, EGLNativeDisplayType nativeDisplay)
 {
+#define ANGLE_GET_PROC_OR_WARNING(MEMBER, NAME)                \
+    do                                                         \
+    {                                                          \
+        if (!SetPtr(MEMBER, getProcAddress(#NAME)))            \
+        {                                                      \
+            WARN() << "Could not load EGL entry point " #NAME; \
+        }                                                      \
+    } while (0)
 #define ANGLE_GET_PROC_OR_ERROR(MEMBER, NAME)                                           \
     do                                                                                  \
     {                                                                                   \
@@ -194,15 +240,43 @@ egl::Error FunctionsEGL::initialize(EGLNativeDisplayType nativeDisplay)
     ANGLE_GET_PROC_OR_ERROR(&mFnPtrs->surfaceAttribPtr, eglSurfaceAttrib);
     ANGLE_GET_PROC_OR_ERROR(&mFnPtrs->swapIntervalPtr, eglSwapInterval);
 
-    mEGLDisplay = mFnPtrs->getDisplayPtr(nativeDisplay);
+    // Querying EGL_EXTENSIONS string and loading it into the mExtensions
+    // vector will at this point retrieve the client extensions since mEGLDisplay is still
+    // EGL_NO_DISPLAY. This is desired, and mExtensions will later be reinitialized with the display
+    // extensions once the display is created and initialized.
+    queryExtensions();
+
+#if defined(ANGLE_HAS_LIBDRM)
+    mEGLDisplay = getPreferredDisplay(&majorVersion, &minorVersion);
+#endif  // defined(ANGLE_HAS_LIBDRM)
+
+    if (mEGLDisplay == EGL_NO_DISPLAY)
+    {
+        if (IsValidPlatformTypeForPlatformDisplayConnection(platformType))
+        {
+            mEGLDisplay = getPlatformDisplay(platformType, nativeDisplay);
+        }
+        else
+        {
+            mEGLDisplay = mFnPtrs->getDisplayPtr(nativeDisplay);
+        }
+    }
+
+    if (mEGLDisplay != EGL_NO_DISPLAY &&
+        mFnPtrs->initializePtr(mEGLDisplay, &majorVersion, &minorVersion) != EGL_TRUE)
+    {
+        mEGLDisplay = EGL_NO_DISPLAY;
+    }
+    if (mEGLDisplay == EGL_NO_DISPLAY)
+    {
+        // If no display was available, try to fallback to the first available
+        // native device object's display.
+        mEGLDisplay = getNativeDisplay(&majorVersion, &minorVersion);
+    }
     if (mEGLDisplay == EGL_NO_DISPLAY)
     {
         return egl::EglNotInitialized() << "Failed to get system egl display";
     }
-    if (mFnPtrs->initializePtr(mEGLDisplay, &majorVersion, &minorVersion) != EGL_TRUE)
-    {
-        return egl::Error(mFnPtrs->getErrorPtr(), "Failed to initialize system egl");
-    }
     if (majorVersion < 1 || (majorVersion == 1 && minorVersion < 4))
     {
         return egl::EglNotInitialized() << "Unsupported EGL version (require at least 1.4).";
@@ -212,14 +286,15 @@ egl::Error FunctionsEGL::initialize(EGLNativeDisplayType nativeDisplay)
         return egl::Error(mFnPtrs->getErrorPtr(), "Failed to bind API in system egl");
     }
 
+    vendorString  = queryString(EGL_VENDOR);
+    versionString = queryString(EGL_VERSION);
+
     ANGLE_GET_PROC_OR_ERROR(&mFnPtrs->getCurrentContextPtr, eglGetCurrentContext);
 
-    const char *extensions = queryString(EGL_EXTENSIONS);
-    if (!extensions)
+    if (!queryExtensions())
     {
-        return egl::Error(mFnPtrs->getErrorPtr(), "Faild to query extensions in system egl");
+        return egl::Error(mFnPtrs->getErrorPtr(), "Failed to query extensions in system egl");
     }
-    angle::SplitStringAlongWhitespace(extensions, &mExtensions);
 
     if (hasExtension("EGL_KHR_image_base"))
     {
@@ -282,6 +357,25 @@ egl::Error FunctionsEGL::initialize(EGLNativeDisplayType nativeDisplay)
         }
     }
 
+    if (hasExtension("EGL_EXT_image_dma_buf_import_modifiers"))
+    {
+        // https://anglebug.com/42266130
+        // Some drivers, notably older versions of ANGLE, announce this extension without
+        // implementing the following functions. DisplayEGL checks for this case and disables the
+        // extension.
+        ANGLE_GET_PROC_OR_WARNING(&mFnPtrs->queryDmaBufFormatsEXTPtr, eglQueryDmaBufFormatsEXT);
+        ANGLE_GET_PROC_OR_WARNING(&mFnPtrs->queryDmaBufModifiersEXTPtr, eglQueryDmaBufModifiersEXT);
+    }
+
+    // EGL_EXT_device_query is only advertised in extension string in the
+    // no-display case, see getNativeDisplay().
+    if (SetPtr(&mFnPtrs->queryDeviceAttribEXTPtr, getProcAddress("eglQueryDeviceAttribEXT")) &&
+        SetPtr(&mFnPtrs->queryDeviceStringEXTPtr, getProcAddress("eglQueryDeviceStringEXT")) &&
+        SetPtr(&mFnPtrs->queryDisplayAttribEXTPtr, getProcAddress("eglQueryDisplayAttribEXT")))
+    {
+        mExtensions.push_back("EGL_EXT_device_query");
+    }
+
 #undef ANGLE_GET_PROC_OR_ERROR
 
     return egl::NoError();
@@ -297,6 +391,207 @@ egl::Error FunctionsEGL::terminate()
     return egl::Error(mFnPtrs->getErrorPtr());
 }
 
+bool FunctionsEGL::queryExtensions()
+{
+    const char *extensions = queryString(EGL_EXTENSIONS);
+    if (!extensions)
+    {
+        return false;
+    }
+
+    angle::SplitStringAlongWhitespace(extensions, &mExtensions);
+
+    return true;
+}
+
+EGLDisplay FunctionsEGL::getPlatformDisplay(EGLAttrib platformType,
+                                            EGLNativeDisplayType nativeDisplay)
+{
+    PFNEGLGETPLATFORMDISPLAYEXTPROC getPlatformDisplayEXTPtr;
+    if (!hasExtension("EGL_EXT_platform_base") ||
+        !SetPtr(&getPlatformDisplayEXTPtr, getProcAddress("eglGetPlatformDisplayEXT")))
+    {
+        return EGL_NO_DISPLAY;
+    }
+
+    ASSERT(IsValidPlatformTypeForPlatformDisplayConnection(platformType));
+    switch (platformType)
+    {
+        case EGL_PLATFORM_SURFACELESS_MESA:
+            if (!hasExtension("EGL_MESA_platform_surfaceless"))
+                return EGL_NO_DISPLAY;
+            break;
+        case EGL_PLATFORM_GBM_KHR:
+            break;
+        default:
+            UNREACHABLE();
+            return EGL_NO_DISPLAY;
+    }
+
+    return getPlatformDisplayEXTPtr(static_cast<EGLenum>(platformType),
+                                    reinterpret_cast<void *>(nativeDisplay), nullptr);
+}
+
+std::vector<EGLDeviceEXT> FunctionsEGL::queryDevices(int *major, int *minor)
+{
+    // Only called after confirming we have the necessary extension.
+    PFNEGLQUERYDEVICESEXTPROC queryDevicesEXTPtr;
+    if (!SetPtr(&queryDevicesEXTPtr, getProcAddress("eglQueryDevicesEXT")))
+    {
+        return {};
+    }
+
+    // Get a list of native device objects.
+    const EGLint kMaxDevices = 32;
+    EGLint numDevices        = 0;
+    std::vector<EGLDeviceEXT> devices(kMaxDevices, EGL_NO_DEVICE_EXT);
+    if (!queryDevicesEXTPtr(kMaxDevices, devices.data(), &numDevices))
+    {
+        return {};
+    }
+
+    devices.resize(numDevices);
+    return devices;
+}
+
+EGLDisplay FunctionsEGL::getNativeDisplay(int *major, int *minor)
+{
+    // This fallback mechanism makes use of:
+    // - EGL_EXT_device_enumeration or EGL_EXT_device_base for eglQueryDevicesEXT
+    // - EGL_EXT_platform_base for eglGetPlatformDisplayEXT
+    // - EGL_EXT_platform_device for EGL_PLATFORM_DEVICE_EXT
+    PFNEGLGETPLATFORMDISPLAYEXTPROC getPlatformDisplayEXTPtr;
+    bool hasQueryDevicesEXT =
+        hasExtension("EGL_EXT_device_enumeration") || hasExtension("EGL_EXT_device_base");
+    bool hasPlatformBaseEXT   = hasExtension("EGL_EXT_platform_base");
+    bool hasPlatformDeviceEXT = hasExtension("EGL_EXT_platform_device");
+    if (!hasQueryDevicesEXT || !hasPlatformBaseEXT || !hasPlatformDeviceEXT)
+    {
+        return EGL_NO_DISPLAY;
+    }
+
+    if (!SetPtr(&getPlatformDisplayEXTPtr, getProcAddress("eglGetPlatformDisplayEXT")))
+    {
+        return EGL_NO_DISPLAY;
+    }
+
+    std::vector<EGLDeviceEXT> devices = queryDevices(major, minor);
+    if (devices.size() == 0)
+    {
+        return EGL_NO_DISPLAY;
+    }
+
+    // Look for the first native device that gives us a valid display.
+    for (const EGLDeviceEXT device : devices)
+    {
+        EGLDisplay display = getPlatformDisplayEXTPtr(EGL_PLATFORM_DEVICE_EXT, device, nullptr);
+        if (mFnPtrs->getErrorPtr() == EGL_SUCCESS &&
+            mFnPtrs->initializePtr(display, major, minor) == EGL_TRUE)
+        {
+            return display;
+        }
+    }
+
+    return EGL_NO_DISPLAY;
+}
+
+#if defined(ANGLE_HAS_LIBDRM)
+EGLDeviceEXT FunctionsEGL::getPreferredEGLDevice(const std::vector<EGLDeviceEXT> &devices)
+{
+    // Only called after confirming we have the necessary extension.
+    PFNEGLQUERYDEVICESTRINGEXTPROC queryDeviceStringEXTPtr;
+    if (!SetPtr(&queryDeviceStringEXTPtr, getProcAddress("eglQueryDeviceStringEXT")))
+    {
+        return EGL_NO_DEVICE_EXT;
+    }
+
+    std::map<EGLDeviceEXT, std::string> device_drivers;
+    for (const EGLDeviceEXT device : devices)
+    {
+        const char *filename = queryDeviceStringEXTPtr(device, EGL_DRM_DEVICE_FILE_EXT);
+        if (!filename)
+        {
+            continue;
+        }
+
+        int fd = open(filename, O_RDWR);
+        if (!fd)
+        {
+            continue;
+        }
+
+        drmVersionPtr version = drmGetVersion(fd);
+        if (version)
+        {
+            const std::string driver_name(version->name, version->name_len);
+            device_drivers.insert({device, driver_name});
+        }
+
+        drmFreeVersion(version);
+        close(fd);
+    }
+
+    const char *preferred_drivers[3] = {"i915", "amdgpu", "virtio_gpu"};
+    for (const char *preferred_driver : preferred_drivers)
+    {
+        for (const EGLDeviceEXT device : devices)
+        {
+            const auto driver = device_drivers.find(device);
+            if (driver != device_drivers.end() && driver->second == preferred_driver)
+            {
+                return device;
+            }
+        }
+    }
+
+    return EGL_NO_DEVICE_EXT;
+}
+
+EGLDisplay FunctionsEGL::getPreferredDisplay(int *major, int *minor)
+{
+    // This  mechanism makes use of:
+    // - EGL_EXT_device_enumeration or EGL_EXT_device_base for eglQueryDevicesEXT
+    // - EGL_EXT_platform_base for eglGetPlatformDisplayEXT
+    // - EGL_EXT_platform_device for EGL_PLATFORM_DEVICE_EXT
+    // - EGL_EXT_device_query for eglQueryDeviceStringEXT
+    PFNEGLGETPLATFORMDISPLAYEXTPROC getPlatformDisplayEXTPtr;
+    bool hasQueryDevicesEXT =
+        hasExtension("EGL_EXT_device_enumeration") || hasExtension("EGL_EXT_device_base");
+    bool hasPlatformBaseEXT   = hasExtension("EGL_EXT_platform_base");
+    bool hasPlatformDeviceEXT = hasExtension("EGL_EXT_platform_device");
+    bool hasDeviceQueryEXT    = hasExtension("EGL_EXT_device_query");
+    if (!hasQueryDevicesEXT || !hasPlatformBaseEXT || !hasPlatformDeviceEXT || !hasDeviceQueryEXT)
+    {
+        return EGL_NO_DISPLAY;
+    }
+
+    if (!SetPtr(&getPlatformDisplayEXTPtr, getProcAddress("eglGetPlatformDisplayEXT")))
+    {
+        return EGL_NO_DISPLAY;
+    }
+
+    std::vector<EGLDeviceEXT> devices = queryDevices(major, minor);
+    if (devices.size() == 0)
+    {
+        return EGL_NO_DISPLAY;
+    }
+
+    EGLDeviceEXT device = getPreferredEGLDevice(devices);
+    if (device == EGL_NO_DEVICE_EXT)
+    {
+        return EGL_NO_DISPLAY;
+    }
+
+    EGLDisplay display = getPlatformDisplayEXTPtr(EGL_PLATFORM_DEVICE_EXT, device, nullptr);
+    if (mFnPtrs->getErrorPtr() == EGL_SUCCESS)
+    {
+        return display;
+    }
+
+    return EGL_NO_DISPLAY;
+}
+#endif  // defined(ANGLE_HAS_LIBDRM)
+
 class FunctionsGLEGL : public FunctionsGL
 {
   public:
@@ -323,6 +618,12 @@ bool FunctionsEGL::hasExtension(const char *extension) const
     return std::find(mExtensions.begin(), mExtensions.end(), extension) != mExtensions.end();
 }
 
+bool FunctionsEGL::hasDmaBufImportModifierFunctions() const
+{
+    return mFnPtrs->queryDmaBufFormatsEXTPtr != nullptr &&
+           mFnPtrs->queryDmaBufModifiersEXTPtr != nullptr;
+}
+
 EGLDisplay FunctionsEGL::getDisplay() const
 {
     return mEGLDisplay;
@@ -528,4 +829,38 @@ EGLint FunctionsEGL::dupNativeFenceFDANDROID(EGLSync sync) const
     return mFnPtrs->dupNativeFenceFDANDROIDPtr(mEGLDisplay, sync);
 }
 
+EGLint FunctionsEGL::queryDmaBufFormatsEXT(EGLint maxFormats,
+                                           EGLint *formats,
+                                           EGLint *numFormats) const
+{
+    return mFnPtrs->queryDmaBufFormatsEXTPtr(mEGLDisplay, maxFormats, formats, numFormats);
+}
+
+EGLint FunctionsEGL::queryDmaBufModifiersEXT(EGLint format,
+                                             EGLint maxModifiers,
+                                             EGLuint64KHR *modifiers,
+                                             EGLBoolean *externalOnly,
+                                             EGLint *numModifiers) const
+{
+    return mFnPtrs->queryDmaBufModifiersEXTPtr(mEGLDisplay, format, maxModifiers, modifiers,
+                                               externalOnly, numModifiers);
+}
+
+EGLBoolean FunctionsEGL::queryDeviceAttribEXT(EGLDeviceEXT device,
+                                              EGLint attribute,
+                                              EGLAttrib *value) const
+{
+    return mFnPtrs->queryDeviceAttribEXTPtr(device, attribute, value);
+}
+
+const char *FunctionsEGL::queryDeviceStringEXT(EGLDeviceEXT device, EGLint name) const
+{
+    return mFnPtrs->queryDeviceStringEXTPtr(device, name);
+}
+
+EGLBoolean FunctionsEGL::queryDisplayAttribEXT(EGLint attribute, EGLAttrib *value) const
+{
+    return mFnPtrs->queryDisplayAttribEXTPtr(mEGLDisplay, attribute, value);
+}
+
 }  // namespace rx
