@@ -31,6 +31,7 @@ class ContextMtl;
 namespace mtl
 {
 
+class ContextDevice;
 class CommandQueue;
 class BlitCommandEncoder;
 class Resource;
@@ -51,10 +52,12 @@ class Resource : angle::NonCopyable
     // Check whether the resource still being used by GPU including the pending (uncommitted)
     // command buffer.
     bool isBeingUsedByGPU(Context *context) const;
-    // Checks whether the last command buffer that uses the given resource has been committed or not
+    // Checks whether the last command buffer that uses the given resource has been committed or
+    // not
     bool hasPendingWorks(Context *context) const;
+    bool hasPendingRenderWorks(Context *context) const;
 
-    void setUsedByCommandBufferWithQueueSerial(uint64_t serial, bool writing);
+    void setUsedByCommandBufferWithQueueSerial(uint64_t serial, bool writing, bool isRenderCommand);
 
     uint64_t getCommandBufferQueueSerial() const { return mUsageRef->cmdBufferQueueSerial; }
 
@@ -63,35 +66,68 @@ class Resource : angle::NonCopyable
     bool isCPUReadMemNeedSync() const { return mUsageRef->cpuReadMemNeedSync; }
     void resetCPUReadMemNeedSync() { mUsageRef->cpuReadMemNeedSync = false; }
 
+    bool isCPUReadMemSyncPending() const { return mUsageRef->cpuReadMemSyncPending; }
+    void setCPUReadMemSyncPending(bool value) const { mUsageRef->cpuReadMemSyncPending = value; }
+    void resetCPUReadMemSyncPending() { mUsageRef->cpuReadMemSyncPending = false; }
+
     bool isCPUReadMemDirty() const { return mUsageRef->cpuReadMemDirty; }
     void resetCPUReadMemDirty() { mUsageRef->cpuReadMemDirty = false; }
 
+    uint64_t getLastReadingRenderEncoderSerial() const
+    {
+        return mUsageRef->lastReadingRenderEncoderSerial;
+    }
+    uint64_t getLastWritingRenderEncoderSerial() const
+    {
+        return mUsageRef->lastWritingRenderEncoderSerial;
+    }
+
+    uint64_t getLastRenderEncoderSerial() const
+    {
+        return std::max(mUsageRef->lastReadingRenderEncoderSerial,
+                        mUsageRef->lastWritingRenderEncoderSerial);
+    }
+
+    virtual size_t estimatedByteSize() const = 0;
+    virtual id getID() const                 = 0;
+
   protected:
+    struct UsageRef;
+
     Resource();
     // Share the GPU usage ref with other resource
     Resource(Resource *other);
+    Resource(std::shared_ptr<UsageRef> otherUsageRef);
 
     void reset();
 
-  private:
     struct UsageRef
     {
         // The id of the last command buffer that is using this resource.
         uint64_t cmdBufferQueueSerial = 0;
 
         // This flag means the resource was issued to be modified by GPU, if CPU wants to read
-        // its content, explicit synchornization call must be invoked.
+        // its content, explicit synchronization call must be invoked.
         bool cpuReadMemNeedSync = false;
 
+        // This flag is set when synchronization for the resource has been
+        // encoded on the GPU, and a map operation must wait
+        // until it's completed.
+        bool cpuReadMemSyncPending = false;
+
         // This flag is useful for BufferMtl to know whether it should update the shadow copy
         bool cpuReadMemDirty = false;
+
+        // The id of the last render encoder to read/write to this resource
+        uint64_t lastReadingRenderEncoderSerial = 0;
+        uint64_t lastWritingRenderEncoderSerial = 0;
     };
 
     // One resource object might just be a view of another resource. For example, a texture 2d
-    // object might be a view of one face of a cube texture object. Another example is one texture
-    // object of size 2x2 might be a mipmap view of a texture object size 4x4. Thus, if one object
-    // is being used by a command buffer, it means the other object is being used also. In this
-    // case, the two objects must share the same UsageRef property.
+    // object might be a view of one face of a cube texture object. Another example is one
+    // texture object of size 2x2 might be a mipmap view of a texture object size 4x4. Thus, if
+    // one object is being used by a command buffer, it means the other object is being used
+    // also. In this case, the two objects must share the same UsageRef property.
     std::shared_ptr<UsageRef> mUsageRef;
 };
 
@@ -110,11 +146,12 @@ class Texture final : public Resource,
                                        TextureRef *refOut);
 
     // On macOS, memory will still be allocated for this texture.
-    static angle::Result MakeMemoryLess2DTexture(ContextMtl *context,
-                                                 const Format &format,
-                                                 uint32_t width,
-                                                 uint32_t height,
-                                                 TextureRef *refOut);
+    static angle::Result MakeMemoryLess2DMSTexture(ContextMtl *context,
+                                                   const Format &format,
+                                                   uint32_t width,
+                                                   uint32_t height,
+                                                   uint32_t samples,
+                                                   TextureRef *refOut);
 
     static angle::Result MakeCubeTexture(ContextMtl *context,
                                          const Format &format,
@@ -152,15 +189,6 @@ class Texture final : public Resource,
                                        bool renderTargetOnly,
                                        bool allowFormatView,
                                        TextureRef *refOut);
-
-    static angle::Result MakeIOSurfaceTexture(ContextMtl *context,
-                                              const Format &format,
-                                              uint32_t width,
-                                              uint32_t height,
-                                              IOSurfaceRef ref,
-                                              uint32_t plane,
-                                              TextureRef *refOut);
-
     static TextureRef MakeFromMetal(id<MTLTexture> metalTexture);
 
     // Allow CPU to read & write data directly to this texture?
@@ -168,6 +196,8 @@ class Texture final : public Resource,
     // Allow shaders to read/sample this texture?
     // Texture created with renderTargetOnly flag won't be readable
     bool isShaderReadable() const;
+    // Allow shaders to write this texture?
+    bool isShaderWritable() const;
 
     bool supportFormatView() const;
 
@@ -198,21 +228,36 @@ class Texture final : public Resource,
     TextureRef createCubeFaceView(uint32_t face);
     // Create a view of one slice at a level.
     TextureRef createSliceMipView(uint32_t slice, const MipmapNativeLevel &level);
+    // Same as createSliceMipView but the target format must be compatible, for example sRGB to linear. In this
+    // case texture doesn't need format view usage flag.
+    TextureRef createSliceMipViewWithCompatibleFormat(uint32_t slice,
+                                                      const MipmapNativeLevel &level,
+                                                      MTLPixelFormat format);
+    // Create a levels range view
+    TextureRef createMipsView(const MipmapNativeLevel &baseLevel, uint32_t levels);
     // Create a view of a level.
     TextureRef createMipView(const MipmapNativeLevel &level);
     // Create a view with different format
     TextureRef createViewWithDifferentFormat(MTLPixelFormat format);
-    // Same as above but the target format must be compatible, for example sRGB to linear. In this
-    // case texture doesn't need format view usage flag.
+    // Create a view for a shader image binding.
+    TextureRef createShaderImageView2D(const MipmapNativeLevel &level,
+                                       int layer,
+                                       MTLPixelFormat format);
+    // Same as above but the target format must be compatible, for example sRGB to linear. In
+    // this case texture doesn't need format view usage flag.
     TextureRef createViewWithCompatibleFormat(MTLPixelFormat format);
     // Create a swizzled view
-    TextureRef createSwizzleView(const TextureSwizzleChannels &swizzle);
+    TextureRef createMipsSwizzleView(const MipmapNativeLevel &baseLevel,
+                                     uint32_t levels,
+                                     MTLPixelFormat format,
+                                     const MTLTextureSwizzleChannels &swizzle);
 
     MTLTextureType textureType() const;
     MTLPixelFormat pixelFormat() const;
 
     uint32_t mipmapLevels() const;
     uint32_t arrayLength() const;
+    uint32_t cubeFaces() const;
     uint32_t cubeFacesOrArrayLength() const;
 
     uint32_t width(const MipmapNativeLevel &level) const;
@@ -229,9 +274,14 @@ class Texture final : public Resource,
 
     uint32_t samples() const;
 
+    bool hasIOSurface() const;
+    bool sameTypeAndDimemsionsAs(const TextureRef &other) const;
+
     angle::Result resize(ContextMtl *context, uint32_t width, uint32_t height);
 
-    // For render target
+    // Get the color write mask to restrict writing to certain color channels in this texture. It's
+    // used for textures having emulated mtl::Format such as RGB which should always have alpha
+    // value being one.
     MTLColorWriteMask getColorWritableMask() const { return *mColorWritableMask; }
     void setColorWritableMask(MTLColorWriteMask mask) { *mColorWritableMask = mask; }
 
@@ -254,11 +304,23 @@ class Texture final : public Resource,
     // Get linear color
     TextureRef getLinearColorView();
 
+    TextureRef parentTexture();
+    MipmapNativeLevel parentRelativeLevel();
+    uint32_t parentRelativeSlice();
+
     // Change the wrapped metal object. Special case for swapchain image
     void set(id<MTLTexture> metalTexture);
 
     // Explicitly sync content between CPU and GPU
     void syncContent(ContextMtl *context, mtl::BlitCommandEncoder *encoder);
+    void setEstimatedByteSize(size_t bytes) { mEstimatedByteSize = bytes; }
+    size_t estimatedByteSize() const override { return mEstimatedByteSize; }
+    id getID() const override { return get(); }
+
+    // Should we disable MTLLoadActionLoad & MTLStoreActionStore when using this texture
+    // as render pass' attachment. This is usually used for memoryless textures and
+    // EXT_multisampled_render_to_texture.
+    bool shouldNotLoadStore() const { return mShouldNotLoadStore; }
 
   private:
     using ParentClass = WrappedObject<id<MTLTexture>>;
@@ -289,6 +351,13 @@ class Texture final : public Resource,
                                      TextureRef *refOut);
 
     Texture(id<MTLTexture> metalTexture);
+
+    // Create a texture that shares ownership of usageRef, underlying MTLTexture and colorWriteMask
+    // with the original texture.
+    Texture(std::shared_ptr<UsageRef> usageRef,
+            id<MTLTexture> metalTexture,
+            std::shared_ptr<MTLColorWriteMask> colorWriteMask);
+
     Texture(ContextMtl *context,
             MTLTextureDescriptor *desc,
             uint32_t mips,
@@ -308,13 +377,22 @@ class Texture final : public Resource,
             bool renderTargetOnly);
 
     // Create a texture view
-    Texture(Texture *original, MTLPixelFormat format);
-    Texture(Texture *original, MTLTextureType type, NSRange mipmapLevelRange, NSRange slices);
-    Texture(Texture *original, const TextureSwizzleChannels &swizzle);
+    Texture(Texture *original, MTLPixelFormat pixelFormat);
+    Texture(Texture *original,
+            MTLPixelFormat pixelFormat,
+            MTLTextureType textureType,
+            NSRange levels,
+            NSRange slices);
+    Texture(Texture *original,
+            MTLPixelFormat pixelFormat,
+            MTLTextureType textureType,
+            NSRange levels,
+            NSRange slices,
+            const MTLTextureSwizzleChannels &swizzle);
 
     void syncContentIfNeeded(ContextMtl *context);
 
-    AutoObjCObj<MTLTextureDescriptor> mCreationDesc;
+    angle::ObjCPtr<MTLTextureDescriptor> mCreationDesc;
 
     // This property is shared between this object and its views:
     std::shared_ptr<MTLColorWriteMask> mColorWritableMask;
@@ -325,39 +403,36 @@ class Texture final : public Resource,
     TextureRef mStencilView;
     // Readable copy of texture
     TextureRef mReadCopy;
+
+    TextureRef mParentTexture;
+
+    size_t mEstimatedByteSize = 0;
+
+    bool mShouldNotLoadStore = false;
 };
 
-class Buffer final : public Resource,
-                     public WrappedObject<id<MTLBuffer>>,
-                     public std::enable_shared_from_this<Buffer>
+class Buffer final : public Resource, public WrappedObject<id<MTLBuffer>>
 {
   public:
+    static MTLStorageMode getStorageModeForSharedBuffer(ContextMtl *contextMtl);
+    using Usage = gl::BufferUsage;
+    static MTLStorageMode getStorageModeForUsage(ContextMtl *context, Usage usage);
+
     static angle::Result MakeBuffer(ContextMtl *context,
                                     size_t size,
                                     const uint8_t *data,
                                     BufferRef *bufferOut);
 
-    static angle::Result MakeBufferWithSharedMemOpt(ContextMtl *context,
-                                                    bool forceUseSharedMem,
-                                                    size_t size,
-                                                    const uint8_t *data,
-                                                    BufferRef *bufferOut);
+    static angle::Result MakeBufferWithStorageMode(ContextMtl *context,
+                                                   MTLStorageMode storageMode,
+                                                   size_t size,
+                                                   const uint8_t *data,
+                                                   BufferRef *bufferOut);
 
-    static angle::Result MakeBufferWithResOpt(ContextMtl *context,
-                                              MTLResourceOptions resourceOptions,
-                                              size_t size,
-                                              const uint8_t *data,
-                                              BufferRef *bufferOut);
-
-    angle::Result reset(ContextMtl *context, size_t size, const uint8_t *data);
-    angle::Result resetWithSharedMemOpt(ContextMtl *context,
-                                        bool forceUseSharedMem,
-                                        size_t size,
-                                        const uint8_t *data);
-    angle::Result resetWithResOpt(ContextMtl *context,
-                                  MTLResourceOptions resourceOptions,
-                                  size_t size,
-                                  const uint8_t *data);
+    angle::Result reset(ContextMtl *context,
+                        MTLStorageMode storageMode,
+                        size_t size,
+                        const uint8_t *data);
 
     const uint8_t *mapReadOnly(ContextMtl *context);
     uint8_t *map(ContextMtl *context);
@@ -370,19 +445,26 @@ class Buffer final : public Resource,
     void flush(ContextMtl *context, size_t offsetWritten, size_t sizeWritten);
 
     size_t size() const;
-    bool useSharedMem() const;
+    MTLStorageMode storageMode() const;
 
     // Explicitly sync content between CPU and GPU
     void syncContent(ContextMtl *context, mtl::BlitCommandEncoder *encoder);
 
+    size_t estimatedByteSize() const override { return size(); }
+    id getID() const override { return get(); }
+
+    size_t getNumContextSwitchesAtLastUse() { return mContextSwitchesAtLastUse; }
+    void setNumContextSwitchesAtLastUse(size_t num) { mContextSwitchesAtLastUse = num; }
+    size_t getNumCommandBufferCommitsAtLastUse() { return mCommandBufferCommitsAtLastUse; }
+    void setNumCommandBufferCommitsAtLastUse(size_t num) { mCommandBufferCommitsAtLastUse = num; }
+
   private:
-    Buffer(ContextMtl *context, bool forceUseSharedMem, size_t size, const uint8_t *data);
-    Buffer(ContextMtl *context,
-           MTLResourceOptions resourceOptions,
-           size_t size,
-           const uint8_t *data);
+    Buffer(ContextMtl *context, MTLStorageMode storageMode, size_t size, const uint8_t *data);
 
     bool mMapReadOnly = true;
+    // For garbage collecting shadow buffers in BufferManager.
+    size_t mContextSwitchesAtLastUse      = 0;
+    size_t mCommandBufferCommitsAtLastUse = 0;
 };
 
 class NativeTexLevelArray
