@@ -17,40 +17,56 @@
 namespace angle
 {
 class PoolAllocator;
+class SharedRingBufferAllocator;
 }  // namespace angle
 
+#if ANGLE_ENABLE_VULKAN_SHARED_RING_BUFFER_CMD_ALLOC
+using SecondaryCommandMemoryAllocator = angle::SharedRingBufferAllocator;
+#else
+using SecondaryCommandMemoryAllocator = angle::PoolAllocator;
+#endif
+
 namespace rx
 {
 class ContextVk;
 
 namespace vk
 {
-class Context;
+class ErrorContext;
 class RenderPassDesc;
+class SecondaryCommandPool;
 
 class VulkanSecondaryCommandBuffer : public priv::CommandBuffer
 {
   public:
     VulkanSecondaryCommandBuffer() = default;
 
-    static angle::Result InitializeCommandPool(Context *context,
-                                               CommandPool *pool,
+    static angle::Result InitializeCommandPool(ErrorContext *context,
+                                               SecondaryCommandPool *pool,
                                                uint32_t queueFamilyIndex,
-                                               bool hasProtectedContent);
+                                               ProtectionType protectionType);
     static angle::Result InitializeRenderPassInheritanceInfo(
         ContextVk *contextVk,
         const Framebuffer &framebuffer,
         const RenderPassDesc &renderPassDesc,
-        VkCommandBufferInheritanceInfo *inheritanceInfoOut);
+        VkCommandBufferInheritanceInfo *inheritanceInfoOut,
+        VkCommandBufferInheritanceRenderingInfo *renderingInfoOut,
+        gl::DrawBuffersArray<VkFormat> *colorFormatStorageOut);
 
-    angle::Result initialize(vk::Context *context,
-                             vk::CommandPool *pool,
+    angle::Result initialize(ErrorContext *context,
+                             SecondaryCommandPool *pool,
                              bool isRenderPassCommandBuffer,
-                             angle::PoolAllocator *allocator);
+                             SecondaryCommandMemoryAllocator *allocator);
 
-    angle::Result begin(vk::Context *context,
+    void destroy();
+
+    void attachAllocator(SecondaryCommandMemoryAllocator *source) {}
+
+    void detachAllocator(SecondaryCommandMemoryAllocator *destination) {}
+
+    angle::Result begin(ErrorContext *context,
                         const VkCommandBufferInheritanceInfo &inheritanceInfo);
-    angle::Result end(vk::Context *context);
+    angle::Result end(ErrorContext *context);
     VkResult reset();
 
     void executeCommands(PrimaryCommandBuffer *primary) { primary->executeCommands(1, this); }
@@ -154,13 +170,24 @@ class VulkanSecondaryCommandBuffer : public priv::CommandBuffer
                        VkPipelineStageFlags dstStageMask,
                        const VkBufferMemoryBarrier *bufferMemoryBarrier);
 
+    void bufferBarrier2(const VkBufferMemoryBarrier2 *bufferMemoryBarrier2);
+
     void imageBarrier(VkPipelineStageFlags srcStageMask,
                       VkPipelineStageFlags dstStageMask,
                       const VkImageMemoryBarrier &imageMemoryBarrier);
 
+    void imageBarrier2(const VkImageMemoryBarrier2 &imageMemoryBarrier2);
+
+    void imageWaitEvent(const VkEvent &event,
+                        VkPipelineStageFlags srcStageMask,
+                        VkPipelineStageFlags dstStageMask,
+                        const VkImageMemoryBarrier &imageMemoryBarrier);
+
     void memoryBarrier(VkPipelineStageFlags srcStageMask,
                        VkPipelineStageFlags dstStageMask,
-                       const VkMemoryBarrier *memoryBarrier);
+                       const VkMemoryBarrier &memoryBarrier);
+
+    void memoryBarrier2(const VkMemoryBarrier2 &memoryBarrier);
 
     void nextSubpass(VkSubpassContents subpassContents);
 
@@ -174,6 +201,14 @@ class VulkanSecondaryCommandBuffer : public priv::CommandBuffer
                          uint32_t imageMemoryBarrierCount,
                          const VkImageMemoryBarrier *imageMemoryBarriers);
 
+    void pipelineBarrier2(VkDependencyFlags dependencyFlags,
+                          uint32_t memoryBarrierCount,
+                          const VkMemoryBarrier2 *memoryBarriers2,
+                          uint32_t bufferMemoryBarrierCount,
+                          const VkBufferMemoryBarrier2 *bufferMemoryBarriers2,
+                          uint32_t imageMemoryBarrierCount,
+                          const VkImageMemoryBarrier2 *imageMemoryBarriers2);
+
     void pushConstants(const PipelineLayout &layout,
                        VkShaderStageFlags flag,
                        uint32_t offset,
@@ -181,6 +216,10 @@ class VulkanSecondaryCommandBuffer : public priv::CommandBuffer
                        const void *data);
 
     void setEvent(VkEvent event, VkPipelineStageFlags stageMask);
+    void setVertexInput(uint32_t vertexBindingDescriptionCount,
+                        const VkVertexInputBindingDescription2EXT *vertexBindingDescriptions,
+                        uint32_t vertexAttributeDescriptionCount,
+                        const VkVertexInputAttributeDescription2EXT *vertexAttributeDescriptions);
     void resetEvent(VkEvent event, VkPipelineStageFlags stageMask);
     void resetQueryPool(const QueryPool &queryPool, uint32_t firstQuery, uint32_t queryCount);
     void resolveImage(const Image &srcImage,
@@ -204,6 +243,10 @@ class VulkanSecondaryCommandBuffer : public priv::CommandBuffer
                         const QueryPool &queryPool,
                         uint32_t query);
 
+    void writeTimestamp2(VkPipelineStageFlagBits2 pipelineStage,
+                         const QueryPool &queryPool,
+                         uint32_t query);
+
     // VK_EXT_transform_feedback
     void beginTransformFeedback(uint32_t firstCounterBuffer,
                                 uint32_t counterBufferCount,
@@ -221,9 +264,14 @@ class VulkanSecondaryCommandBuffer : public priv::CommandBuffer
 
     void open() const {}
     void close() const {}
-    bool empty() const { return !mAnyCommand; }
+    bool empty() const
+    {
+        ASSERT(valid());
+        return !mAnyCommand;
+    }
     uint32_t getRenderPassWriteCommandCount() const
     {
+        ASSERT(valid());
         return mCommandTracker.getRenderPassWriteCommandCount();
     }
     std::string dumpCommands(const char *separator) const { return ""; }
@@ -231,6 +279,7 @@ class VulkanSecondaryCommandBuffer : public priv::CommandBuffer
   private:
     void onRecordCommand() { mAnyCommand = true; }
 
+    SecondaryCommandPool *mCommandPool = nullptr;
     CommandBufferCommandTracker mCommandTracker;
     bool mAnyCommand = false;
 };
@@ -271,6 +320,15 @@ ANGLE_INLINE void VulkanSecondaryCommandBuffer::writeTimestamp(
     CommandBuffer::writeTimestamp(pipelineStage, queryPool, query);
 }
 
+ANGLE_INLINE void VulkanSecondaryCommandBuffer::writeTimestamp2(
+    VkPipelineStageFlagBits2 pipelineStage,
+    const QueryPool &queryPool,
+    uint32_t query)
+{
+    onRecordCommand();
+    CommandBuffer::writeTimestamp2(pipelineStage, queryPool, query);
+}
+
 ANGLE_INLINE void VulkanSecondaryCommandBuffer::clearColorImage(
     const Image &image,
     VkImageLayout imageLayout,
@@ -498,14 +556,36 @@ ANGLE_INLINE void VulkanSecondaryCommandBuffer::pipelineBarrier(
                                    imageMemoryBarrierCount, imageMemoryBarriers);
 }
 
+ANGLE_INLINE void VulkanSecondaryCommandBuffer::pipelineBarrier2(
+    VkDependencyFlags dependencyFlags,
+    uint32_t memoryBarrierCount,
+    const VkMemoryBarrier2 *memoryBarriers2,
+    uint32_t bufferMemoryBarrierCount,
+    const VkBufferMemoryBarrier2 *bufferMemoryBarriers2,
+    uint32_t imageMemoryBarrierCount,
+    const VkImageMemoryBarrier2 *imageMemoryBarriers2)
+{
+    onRecordCommand();
+    CommandBuffer::pipelineBarrier2(dependencyFlags, memoryBarrierCount, memoryBarriers2,
+                                    bufferMemoryBarrierCount, bufferMemoryBarriers2,
+                                    imageMemoryBarrierCount, imageMemoryBarriers2);
+}
+
 ANGLE_INLINE void VulkanSecondaryCommandBuffer::memoryBarrier(VkPipelineStageFlags srcStageMask,
                                                               VkPipelineStageFlags dstStageMask,
-                                                              const VkMemoryBarrier *memoryBarrier)
+                                                              const VkMemoryBarrier &memoryBarrier)
 {
     onRecordCommand();
     CommandBuffer::memoryBarrier(srcStageMask, dstStageMask, memoryBarrier);
 }
 
+ANGLE_INLINE void VulkanSecondaryCommandBuffer::memoryBarrier2(
+    const VkMemoryBarrier2 &memoryBarrier2)
+{
+    onRecordCommand();
+    CommandBuffer::memoryBarrier2(memoryBarrier2);
+}
+
 ANGLE_INLINE void VulkanSecondaryCommandBuffer::bufferBarrier(
     VkPipelineStageFlags srcStageMask,
     VkPipelineStageFlags dstStageMask,
@@ -516,6 +596,13 @@ ANGLE_INLINE void VulkanSecondaryCommandBuffer::bufferBarrier(
                                    bufferMemoryBarrier, 0, nullptr);
 }
 
+ANGLE_INLINE void VulkanSecondaryCommandBuffer::bufferBarrier2(
+    const VkBufferMemoryBarrier2 *bufferMemoryBarrier2)
+{
+    onRecordCommand();
+    CommandBuffer::pipelineBarrier2(0, 0, nullptr, 1, bufferMemoryBarrier2, 0, nullptr);
+}
+
 ANGLE_INLINE void VulkanSecondaryCommandBuffer::imageBarrier(
     VkPipelineStageFlags srcStageMask,
     VkPipelineStageFlags dstStageMask,
@@ -525,6 +612,24 @@ ANGLE_INLINE void VulkanSecondaryCommandBuffer::imageBarrier(
     CommandBuffer::imageBarrier(srcStageMask, dstStageMask, imageMemoryBarrier);
 }
 
+ANGLE_INLINE void VulkanSecondaryCommandBuffer::imageBarrier2(
+    const VkImageMemoryBarrier2 &imageMemoryBarrier2)
+{
+    onRecordCommand();
+    CommandBuffer::imageBarrier2(imageMemoryBarrier2);
+}
+
+ANGLE_INLINE void VulkanSecondaryCommandBuffer::imageWaitEvent(
+    const VkEvent &event,
+    VkPipelineStageFlags srcStageMask,
+    VkPipelineStageFlags dstStageMask,
+    const VkImageMemoryBarrier &imageMemoryBarrier)
+{
+    onRecordCommand();
+    CommandBuffer::waitEvents(1, &event, srcStageMask, dstStageMask, 0, nullptr, 0, nullptr, 1,
+                              &imageMemoryBarrier);
+}
+
 ANGLE_INLINE void VulkanSecondaryCommandBuffer::nextSubpass(VkSubpassContents subpassContents)
 {
     onRecordCommand();
@@ -548,6 +653,17 @@ ANGLE_INLINE void VulkanSecondaryCommandBuffer::setEvent(VkEvent event,
     CommandBuffer::setEvent(event, stageMask);
 }
 
+ANGLE_INLINE void VulkanSecondaryCommandBuffer::setVertexInput(
+    uint32_t vertexBindingDescriptionCount,
+    const VkVertexInputBindingDescription2EXT *vertexBindingDescriptions,
+    uint32_t vertexAttributeDescriptionCount,
+    const VkVertexInputAttributeDescription2EXT *vertexAttributeDescriptions)
+{
+    onRecordCommand();
+    CommandBuffer::setVertexInput(vertexBindingDescriptionCount, vertexBindingDescriptions,
+                                  vertexAttributeDescriptionCount, vertexAttributeDescriptions);
+}
+
 ANGLE_INLINE void VulkanSecondaryCommandBuffer::resetEvent(VkEvent event,
                                                            VkPipelineStageFlags stageMask)
 {
