@@ -458,8 +458,8 @@ TEST_P(EGLDisplaySelectionTestMultiDisplay, BackendDefaultVulkan)
 #endif
     ANGLE_SKIP_TEST_IF(missingBackends);
 
-    // http://anglebug.com/6999
-    ANGLE_SKIP_TEST_IF(IsOSX());
+    // http://anglebug.com/42265471
+    ANGLE_SKIP_TEST_IF(IsMac());
 
     runMultiDisplayBackendDefault(EGL_PLATFORM_ANGLE_TYPE_VULKAN_ANGLE, IsVulkan);
 }
