@@ -169,7 +169,7 @@ struct GLColorT
     T R, G, B, A;
 };
 
-using GLColor16UI = GLColorT<uint16_t>;
+using GLColor16   = GLColorT<uint16_t>;
 using GLColor32F  = GLColorT<float>;
 using GLColor32I  = GLColorT<int32_t>;
 using GLColor32UI = GLColorT<uint32_t>;
@@ -244,6 +244,9 @@ bool IsFormatEmulated(GLenum target);
 
 #define EXPECT_PIXEL_ALPHA_EQ(x, y, a) EXPECT_EQ(a, angle::ReadColor(x, y).A)
 
+#define EXPECT_PIXEL_ALPHA_NEAR(x, y, a, abs_error) \
+    EXPECT_NEAR(a, angle::ReadColor(x, y).A, abs_error);
+
 #define EXPECT_PIXEL_ALPHA32F_EQ(x, y, a) EXPECT_EQ(a, angle::ReadColor32F(x, y).A)
 
 #define EXPECT_PIXEL_COLOR_EQ(x, y, angleColor) EXPECT_EQ(angleColor, angle::ReadColor(x, y))
@@ -302,6 +305,15 @@ bool IsFormatEmulated(GLenum target);
 #define EXPECT_PIXEL_NEAR(x, y, r, g, b, a, abs_error) \
     EXPECT_PIXEL_NEAR_HELPER(x, y, r, g, b, a, abs_error, GLubyte, GL_RGBA, GL_UNSIGNED_BYTE)
 
+#define EXPECT_PIXEL_16_NEAR(x, y, r, g, b, a, abs_error) \
+    EXPECT_PIXEL_NEAR_HELPER(x, y, r, g, b, a, abs_error, GLushort, GL_RGBA, GL_UNSIGNED_SHORT)
+
+#define EXPECT_PIXEL_8S_NEAR(x, y, r, g, b, a, abs_error) \
+    EXPECT_PIXEL_NEAR_HELPER(x, y, r, g, b, a, abs_error, GLbyte, GL_RGBA, GL_BYTE)
+
+#define EXPECT_PIXEL_16S_NEAR(x, y, r, g, b, a, abs_error) \
+    EXPECT_PIXEL_NEAR_HELPER(x, y, r, g, b, a, abs_error, GLshort, GL_RGBA, GL_SHORT)
+
 #define EXPECT_PIXEL_32F_NEAR(x, y, r, g, b, a, abs_error) \
     EXPECT_PIXEL_NEAR_HELPER(x, y, r, g, b, a, abs_error, GLfloat, GL_RGBA, GL_FLOAT)
 
@@ -311,16 +323,25 @@ bool IsFormatEmulated(GLenum target);
 #define EXPECT_PIXEL_8UI(x, y, r, g, b, a) \
     EXPECT_PIXEL_EQ_HELPER(x, y, r, g, b, a, GLubyte, GL_RGBA_INTEGER, GL_UNSIGNED_BYTE)
 
-#define EXPECT_PIXEL_16UI(x, y, r, g, b, a) \
-    EXPECT_PIXEL_EQ_HELPER(x, y, r, g, b, a, GLushort, GL_RGBA, GL_UNSIGNED_SHORT)
+#define EXPECT_PIXEL_32UI(x, y, r, g, b, a) \
+    EXPECT_PIXEL_EQ_HELPER(x, y, r, g, b, a, GLuint, GL_RGBA_INTEGER, GL_UNSIGNED_INT)
 
-#define EXPECT_PIXEL_16UI_COLOR(x, y, color) \
-    EXPECT_PIXEL_16UI(x, y, color.R, color.G, color.B, color.A)
+#define EXPECT_PIXEL_32I(x, y, r, g, b, a) \
+    EXPECT_PIXEL_EQ_HELPER(x, y, r, g, b, a, GLint, GL_RGBA_INTEGER, GL_INT)
+
+#define EXPECT_PIXEL_32UI_COLOR(x, y, color) \
+    EXPECT_PIXEL_32UI(x, y, color.R, color.G, color.B, color.A)
+
+#define EXPECT_PIXEL_32I_COLOR(x, y, color) \
+    EXPECT_PIXEL_32I(x, y, color.R, color.G, color.B, color.A)
 
 // TODO(jmadill): Figure out how we can use GLColor's nice printing with EXPECT_NEAR.
 #define EXPECT_PIXEL_COLOR_NEAR(x, y, angleColor, abs_error) \
     EXPECT_PIXEL_NEAR(x, y, angleColor.R, angleColor.G, angleColor.B, angleColor.A, abs_error)
 
+#define EXPECT_PIXEL_COLOR16_NEAR(x, y, angleColor, abs_error) \
+    EXPECT_PIXEL_16_NEAR(x, y, angleColor.R, angleColor.G, angleColor.B, angleColor.A, abs_error)
+
 #define EXPECT_PIXEL_COLOR32F_NEAR(x, y, angleColor, abs_error) \
     EXPECT_PIXEL32F_NEAR(x, y, angleColor.R, angleColor.G, angleColor.B, angleColor.A, abs_error)
 
@@ -391,6 +412,9 @@ class ANGLETestBase
         return mCurrentParams->eglParameters.debugLayersEnabled != EGL_FALSE;
     }
 
+    void *operator new(size_t size);
+    void operator delete(void *ptr);
+
   protected:
     void ANGLETestSetUp();
     void ANGLETestPreTearDown();
@@ -463,6 +487,12 @@ class ANGLETestBase
                             bool useVertexBuffer,
                             float layer);
 
+    // The layer parameter chooses the 2DArray texture layer to sample from.
+    void draw2DArrayTexturedQuad(GLfloat positionAttribZ,
+                                 GLfloat positionAttribXYScale,
+                                 bool useVertexBuffer,
+                                 float layer);
+
     void setWindowWidth(int width);
     void setWindowHeight(int height);
     void setConfigRedBits(int bits);
@@ -502,9 +532,6 @@ class ANGLETestBase
 
     void ignoreD3D11SDKLayersWarnings();
 
-    // Allows a test to be more restrictive about platform warnings.
-    void treatPlatformWarningsAsErrors();
-
     OSWindow *getOSWindow() { return mFixture->osWindow; }
 
     GLuint get2DTexturedQuadProgram();
@@ -512,6 +539,9 @@ class ANGLETestBase
     // Has a float uniform "u_layer" to choose the 3D texture layer.
     GLuint get3DTexturedQuadProgram();
 
+    // Has a float uniform "u_layer" to choose the 2DArray texture layer.
+    GLuint get2DArrayTexturedQuadProgram();
+
     class [[nodiscard]] ScopedIgnorePlatformMessages : angle::NonCopyable
     {
       public:
@@ -588,6 +618,7 @@ class ANGLETestBase
     // Used for texture rendering.
     GLuint m2DTexturedQuadProgram;
     GLuint m3DTexturedQuadProgram;
+    GLuint m2DArrayTexturedQuadProgram;
 
     bool mDeferContextInit;
     bool mAlwaysForceNewDisplay;
@@ -653,6 +684,14 @@ class ANGLETest : public ANGLETestBase, public ::testing::TestWithParam<Params>
     }
 };
 
+enum class APIExtensionVersion
+{
+    Core,
+    OES,
+    EXT,
+    KHR,
+};
+
 template <typename Params>
 ANGLETest<Params>::ANGLETest()
     : ANGLETestBase(std::get<angle::PlatformParameters>(this->GetParam()))
@@ -672,11 +711,15 @@ class ANGLETestEnvironment : public testing::Environment
 
   private:
     static angle::Library *GetAngleEGLLibrary();
+    static angle::Library *GetAngleVulkanSecondariesEGLLibrary();
+    static angle::Library *GetMesaEGLLibrary();
     static angle::Library *GetSystemEGLLibrary();
     static angle::Library *GetSystemWGLLibrary();
 
     // For loading entry points.
     static std::unique_ptr<angle::Library> gAngleEGLLibrary;
+    static std::unique_ptr<angle::Library> gAngleVulkanSecondariesEGLLibrary;
+    static std::unique_ptr<angle::Library> gMesaEGLLibrary;
     static std::unique_ptr<angle::Library> gSystemEGLLibrary;
     static std::unique_ptr<angle::Library> gSystemWGLLibrary;
 };
