@@ -65,7 +65,7 @@ bool compileTestShader(GLenum type,
                        ShShaderOutput output,
                        const std::string &shaderString,
                        ShBuiltInResources *resources,
-                       ShCompileOptions compileOptions,
+                       const ShCompileOptions &compileOptions,
                        std::string *translatedCode,
                        std::string *infoLog)
 {
@@ -78,9 +78,10 @@ bool compileTestShader(GLenum type,
 
     const char *shaderStrings[] = {shaderString.c_str()};
 
-    bool compilationSuccess =
-        translator->compile(shaderStrings, 1, SH_OBJECT_CODE | compileOptions);
-    TInfoSink &infoSink = translator->getInfoSink();
+    ShCompileOptions options = compileOptions;
+    options.objectCode       = true;
+    bool compilationSuccess  = translator->compile(shaderStrings, 1, options);
+    TInfoSink &infoSink      = translator->getInfoSink();
     if (translatedCode)
     {
         *translatedCode = infoSink.obj.isBinary() ? kBinaryBlob : infoSink.obj.c_str();
@@ -97,7 +98,7 @@ bool compileTestShader(GLenum type,
                        ShShaderSpec spec,
                        ShShaderOutput output,
                        const std::string &shaderString,
-                       ShCompileOptions compileOptions,
+                       const ShCompileOptions &compileOptions,
                        std::string *translatedCode,
                        std::string *infoLog)
 {
@@ -108,16 +109,19 @@ bool compileTestShader(GLenum type,
                              translatedCode, infoLog);
 }
 
-MatchOutputCodeTest::MatchOutputCodeTest(GLenum shaderType,
-                                         ShCompileOptions defaultCompileOptions,
-                                         ShShaderOutput outputType)
-    : mShaderType(shaderType), mDefaultCompileOptions(defaultCompileOptions)
+MatchOutputCodeTest::MatchOutputCodeTest(GLenum shaderType, ShShaderOutput outputType)
+    : mShaderType(shaderType), mDefaultCompileOptions{}
 {
     sh::InitBuiltInResources(&mResources);
     mResources.FragmentPrecisionHigh = 1;
     mOutputCode[outputType]          = std::string();
 }
 
+void MatchOutputCodeTest::setDefaultCompileOptions(const ShCompileOptions &defaultCompileOptions)
+{
+    mDefaultCompileOptions = defaultCompileOptions;
+}
+
 void MatchOutputCodeTest::addOutputType(const ShShaderOutput outputType)
 {
     mOutputCode[outputType] = std::string();
@@ -134,7 +138,7 @@ void MatchOutputCodeTest::compile(const std::string &shaderString)
 }
 
 void MatchOutputCodeTest::compile(const std::string &shaderString,
-                                  const ShCompileOptions compileOptions)
+                                  const ShCompileOptions &compileOptions)
 {
     std::string infoLog;
     for (auto &code : mOutputCode)
@@ -150,7 +154,7 @@ void MatchOutputCodeTest::compile(const std::string &shaderString,
 
 bool MatchOutputCodeTest::compileWithSettings(ShShaderOutput output,
                                               const std::string &shaderString,
-                                              const ShCompileOptions compileOptions,
+                                              const ShCompileOptions &compileOptions,
                                               std::string *translatedCode,
                                               std::string *infoLog)
 {
@@ -334,6 +338,24 @@ bool MatchOutputCodeTest::notFoundInCode(const char *stringToFind) const
     return true;
 }
 
+std::string MatchOutputCodeTest::outputCode(ShShaderOutput output) const
+{
+    const auto code = mOutputCode.find(output);
+    EXPECT_NE(mOutputCode.end(), code);
+    if (code == mOutputCode.end())
+    {
+        return {};
+    }
+
+    // No meaningful check for binary blobs
+    if (IsBinaryBlob(code->second))
+    {
+        return {};
+    }
+
+    return code->second;
+}
+
 const TIntermAggregate *FindFunctionCallNode(TIntermNode *root, const TString &functionMangledName)
 {
     FunctionCallFinder finder(functionMangledName.c_str());
