@@ -16,17 +16,15 @@
 
 // Disable ANGLE-specific warnings that pop up in fuchsia headers.
 ANGLE_DISABLE_DESTRUCTOR_OVERRIDE_WARNING
+ANGLE_DISABLE_SUGGEST_OVERRIDE_WARNINGS
 
-#include <fuchsia/ui/policy/cpp/fidl.h>
-#include <fuchsia/ui/scenic/cpp/fidl.h>
+#include <fuchsia/element/cpp/fidl.h>
 #include <fuchsia_egl.h>
 #include <lib/async-loop/cpp/loop.h>
-#include <lib/ui/scenic/cpp/commands.h>
-#include <lib/ui/scenic/cpp/resources.h>
-#include <lib/ui/scenic/cpp/session.h>
 #include <zircon/types.h>
 #include <string>
 
+ANGLE_REENABLE_SUGGEST_OVERRIDE_WARNINGS
 ANGLE_REENABLE_DESTRUCTOR_OVERRIDE_WARNING
 
 struct FuchsiaEGLWindowDeleter
@@ -41,7 +39,6 @@ class ANGLE_UTIL_EXPORT ScenicWindow : public OSWindow
     ~ScenicWindow() override;
 
     // OSWindow:
-    bool initialize(const std::string &name, int width, int height) override;
     void disableErrorMessageDialog() override;
     void destroy() override;
     void resetNativeWindow() override;
@@ -61,14 +58,8 @@ class ANGLE_UTIL_EXPORT ScenicWindow : public OSWindow
     // surface's image pipe as a child of our window.
     void present();
 
-    // FIDL callbacks:
-    void onScenicEvents(std::vector<fuchsia::ui::scenic::Event> events);
-    void onScenicError(zx_status_t status);
-    void onFramePresented(fuchsia::scenic::scheduling::FramePresentedInfo info);
-    void onViewMetrics(const fuchsia::ui::gfx::Metrics &metrics);
-    void onViewProperties(const fuchsia::ui::gfx::ViewProperties &properties);
-
   private:
+    bool initializeImpl(const std::string &name, int width, int height) override;
     void updateViewSize();
 
     // ScenicWindow async loop.
@@ -76,32 +67,7 @@ class ANGLE_UTIL_EXPORT ScenicWindow : public OSWindow
 
     // System services.
     zx::channel mServiceRoot;
-    fuchsia::ui::scenic::ScenicPtr mScenic;
-    fuchsia::ui::policy::PresenterPtr mPresenter;
-
-    // Scenic session & resources.
-    scenic::Session mScenicSession;
-    scenic::ShapeNode mShape;
-    scenic::Material mMaterial;
-
-    // Whether our scenic session has disconnected.
-    bool mLostSession = false;
-
-    // Present limiting.
-    static constexpr int kMaxInFlightPresents = 2;
-    int mInFlightPresents                     = 0;
-
-    // Scenic view.
-    std::unique_ptr<scenic::View> mView;
-
-    // View geometry.
-    float mDisplayHeightDips = 0.f;
-    float mDisplayWidthDips  = 0.f;
-    float mDisplayScaleX     = 0.f;
-    float mDisplayScaleY     = 0.f;
-    bool mHasViewProperties  = false;
-    bool mHasViewMetrics     = false;
-    bool mViewSizeDirty      = false;
+    fuchsia::element::GraphicalPresenterPtr mPresenter;
 
     // EGL native window.
     std::unique_ptr<fuchsia_egl_window, FuchsiaEGLWindowDeleter> mFuchsiaEGLWindow;
