@@ -35,16 +35,24 @@
 #include <fnmatch.h>
 #include <libgen.h>
 #include <stdio.h>
+#include <sys/file.h>
 #include <sys/stat.h>
 #include <sys/statvfs.h>
 #include <sys/types.h>
 #include <unistd.h>
 #include <wtf/EnumTraits.h>
+#include <wtf/MallocSpan.h>
 #include <wtf/SafeStrerror.h>
 #include <wtf/text/CString.h>
+#include <wtf/text/MakeString.h>
+#include <wtf/text/ParsingUtilities.h>
 #include <wtf/text/StringBuilder.h>
 #include <wtf/text/WTFString.h>
 
+#if USE(GLIB)
+#include <glib.h>
+#endif
+
 namespace WTF {
 
 namespace FileSystemImpl {
@@ -56,12 +64,12 @@ PlatformFileHandle openFile(const String& path, FileOpenMode mode, FileAccessPer
     if (fsRep.isNull())
         return invalidPlatformFileHandle;
 
-    int platformFlag = 0;
+    int platformFlag = O_CLOEXEC;
     switch (mode) {
     case FileOpenMode::Read:
         platformFlag |= O_RDONLY;
         break;
-    case FileOpenMode::Write:
+    case FileOpenMode::Truncate:
         platformFlag |= (O_WRONLY | O_CREAT | O_TRUNC);
         break;
     case FileOpenMode::ReadWrite:
@@ -94,6 +102,11 @@ void closeFile(PlatformFileHandle& handle)
     }
 }
 
+int posixFileDescriptor(PlatformFileHandle handle)
+{
+    return handle;
+}
+
 long long seekFile(PlatformFileHandle handle, long long offset, FileSeekOrigin origin)
 {
     int whence = SEEK_SET;
@@ -124,20 +137,20 @@ bool flushFile(PlatformFileHandle handle)
     return !fsync(handle);
 }
 
-int writeToFile(PlatformFileHandle handle, const void* data, int length)
+int64_t writeToFile(PlatformFileHandle handle, std::span<const uint8_t> data)
 {
     do {
-        int bytesWritten = write(handle, data, static_cast<size_t>(length));
+        auto bytesWritten = write(handle, data.data(), data.size());
         if (bytesWritten >= 0)
             return bytesWritten;
     } while (errno == EINTR);
     return -1;
 }
 
-int readFromFile(PlatformFileHandle handle, void* data, int length)
+int64_t readFromFile(PlatformFileHandle handle, std::span<uint8_t> data)
 {
     do {
-        int bytesRead = read(handle, data, static_cast<size_t>(length));
+        auto bytesRead = read(handle, data.data(), data.size());
         if (bytesRead >= 0)
             return bytesRead;
     } while (errno == EINTR);
@@ -172,22 +185,44 @@ std::optional<uint64_t> fileSize(PlatformFileHandle handle)
 
 std::optional<WallTime> fileCreationTime(const String& path)
 {
-#if OS(DARWIN) || OS(OPENBSD) || OS(NETBSD) || OS(FREEBSD)
+#if (OS(LINUX) && HAVE(STATX)) || OS(DARWIN) || OS(OPENBSD) || OS(NETBSD) || OS(FREEBSD)
     CString fsRep = fileSystemRepresentation(path);
-
     if (!fsRep.data() || fsRep.data()[0] == '\0')
         return std::nullopt;
 
+#if OS(LINUX) && HAVE(STATX)
+    struct statx fileInfo;
+
+    if (statx(-1, fsRep.data(), 0, STATX_BTIME, &fileInfo) == -1)
+        return std::nullopt;
+
+    return WallTime::fromRawSeconds(fileInfo.stx_btime.tv_sec);
+#elif OS(DARWIN) || OS(OPENBSD) || OS(NETBSD) || OS(FREEBSD)
     struct stat fileInfo;
 
-    if (stat(fsRep.data(), &fileInfo))
+    if (stat(fsRep.data(), &fileInfo) == -1)
         return std::nullopt;
 
     return WallTime::fromRawSeconds(fileInfo.st_birthtime);
-#else
+#endif
+#endif
+
     UNUSED_PARAM(path);
     return std::nullopt;
-#endif
+}
+
+std::optional<PlatformFileID> fileID(PlatformFileHandle handle)
+{
+    struct stat fileInfo;
+    if (fstat(handle, &fileInfo))
+        return std::nullopt;
+
+    return fileInfo.st_ino;
+}
+
+bool fileIDsAreEqual(std::optional<PlatformFileID> a, std::optional<PlatformFileID> b)
+{
+    return a == b;
 }
 
 std::optional<uint32_t> volumeFileBlockSize(const String& path)
@@ -215,29 +250,40 @@ CString fileSystemRepresentation(const String& path)
 #endif
 
 #if !PLATFORM(COCOA)
-String openTemporaryFile(StringView prefix, PlatformFileHandle& handle, StringView suffix)
+static const char* temporaryFileDirectory()
 {
-    // FIXME: Suffix is not supported, but OK for now since the code using it is macOS-port-only.
+#if USE(GLIB)
+    return g_get_tmp_dir();
+#else
+    if (auto* tmpDir = getenv("TMPDIR"))
+        return tmpDir;
+
+    return "/tmp";
+#endif
+}
+
+std::pair<String, PlatformFileHandle> openTemporaryFile(StringView prefix, StringView suffix)
+{
+    PlatformFileHandle handle = invalidPlatformFileHandle;
+    // Suffix is not supported because that's incompatible with mkostemp, mkostemps would be needed for that.
+    // This is OK for now since the code using it is built on macOS only.
     ASSERT_UNUSED(suffix, suffix.isEmpty());
 
-    char buffer[PATH_MAX];
-    const char* tmpDir = getenv("TMPDIR");
+    const char* directory = temporaryFileDirectory();
+    CString prefixUTF8 = prefix.utf8();
+    size_t length = strlen(directory) + 1 + prefixUTF8.length() + 1 + 6 + 1;
+    auto buffer = MallocSpan<char>::malloc(length);
+    snprintf(buffer.mutableSpan().data(), length, "%s/%s-XXXXXX", directory, prefixUTF8.data());
 
-    if (!tmpDir)
-        tmpDir = "/tmp";
-
-    if (snprintf(buffer, PATH_MAX, "%s/%sXXXXXX", tmpDir, prefix.utf8().data()) >= PATH_MAX)
-        goto end;
-
-    handle = mkstemp(buffer);
+    handle = mkostemp(buffer.mutableSpan().data(), O_CLOEXEC);
     if (handle < 0)
         goto end;
 
-    return String::fromUTF8(buffer);
+    return { String::fromUTF8(buffer.span().data()), handle };
 
 end:
     handle = invalidPlatformFileHandle;
-    return String();
+    return { String(), handle };
 }
 #endif // !PLATFORM(COCOA)
 
@@ -254,6 +300,8 @@ std::optional<int32_t> getFileDeviceId(const String& path)
     return fileStat.st_dev;
 }
 
+// On macOS, stat() used by std::filesystem is much slower than access() when sandboxed.
+// This fast path exists to avoid calls to stat(). It's not needed on other platforms.
 #if ENABLE(FILESYSTEM_POSIX_FAST_PATH)
 
 bool fileExists(const String& path)
@@ -281,17 +329,17 @@ bool makeAllDirectories(const String& path)
     if (!access(fullPath.data(), F_OK))
         return true;
 
-    char* p = fullPath.mutableData() + 1;
+    auto p = fullPath.mutableSpanIncludingNullTerminator().subspan(1);
     if (p[length - 1] == '/')
         p[length - 1] = '\0';
-    for (; *p; ++p) {
-        if (*p == '/') {
-            *p = '\0';
+    for (; p[0]; skip(p, 1)) {
+        if (p[0] == '/') {
+            p[0] = '\0';
             if (access(fullPath.data(), F_OK)) {
                 if (mkdir(fullPath.data(), S_IRWXU))
                     return false;
             }
-            *p = '/';
+            p[0] = '/';
         }
     }
     if (access(fullPath.data(), F_OK)) {
