@@ -26,16 +26,19 @@
 #include "config.h"
 #include "HTMLSourceElement.h"
 
+#include "CSSSerializationContext.h"
 #include "ElementInlines.h"
 #include "Event.h"
 #include "EventNames.h"
 #include "HTMLImageElement.h"
 #include "HTMLNames.h"
 #include "HTMLPictureElement.h"
+#include "HTMLSrcsetParser.h"
 #include "Logging.h"
-#include "MediaList.h"
 #include "MediaQueryParser.h"
-#include <wtf/IsoMallocInlines.h>
+#include "MediaQueryParserContext.h"
+#include "NodeName.h"
+#include <wtf/TZoneMallocInlines.h>
 
 #if ENABLE(VIDEO)
 #include "HTMLMediaElement.h"
@@ -47,12 +50,12 @@
 
 namespace WebCore {
 
-WTF_MAKE_ISO_ALLOCATED_IMPL(HTMLSourceElement);
+WTF_MAKE_TZONE_OR_ISO_ALLOCATED_IMPL(HTMLSourceElement);
 
 using namespace HTMLNames;
 
 inline HTMLSourceElement::HTMLSourceElement(const QualifiedName& tagName, Document& document)
-    : HTMLElement(tagName, document)
+    : HTMLElement(tagName, document, TypeFlag::HasDidMoveToNewDocument)
     , ActiveDOMObject(document)
 {
     LOG(Media, "HTMLSourceElement::HTMLSourceElement - %p", this);
@@ -77,16 +80,16 @@ Node::InsertedIntoAncestorResult HTMLSourceElement::insertedIntoAncestor(Inserti
     RefPtr<Element> parent = parentElement();
     if (parent == &parentOfInsertedTree) {
 #if ENABLE(VIDEO)
-        if (is<HTMLMediaElement>(*parent))
-            downcast<HTMLMediaElement>(*parent).sourceWasAdded(*this);
+        if (RefPtr mediaElement = dynamicDowncast<HTMLMediaElement>(*parent))
+            mediaElement->sourceWasAdded(*this);
         else
 #endif
 #if ENABLE(MODEL_ELEMENT)
-        if (is<HTMLModelElement>(*parent))
-            downcast<HTMLModelElement>(*parent).sourcesChanged();
+        if (RefPtr modelElement = dynamicDowncast<HTMLModelElement>(*parent))
+            modelElement->sourcesChanged();
         else
 #endif
-        if (is<HTMLPictureElement>(*parent)) {
+        if (RefPtr pictureElement = dynamicDowncast<HTMLPictureElement>(*parent)) {
             // The new source element only is a relevant mutation if it precedes any img element.
             m_shouldCallSourcesChanged = true;
             for (const Node* node = previousSibling(); node; node = node->previousSibling()) {
@@ -94,7 +97,7 @@ Node::InsertedIntoAncestorResult HTMLSourceElement::insertedIntoAncestor(Inserti
                     m_shouldCallSourcesChanged = false;
             }
             if (m_shouldCallSourcesChanged)
-                downcast<HTMLPictureElement>(*parent).sourcesChanged();
+                pictureElement->sourcesChanged();
         }
     }
     return InsertedIntoAncestorResult::Done;
@@ -105,13 +108,13 @@ void HTMLSourceElement::removedFromAncestor(RemovalType removalType, ContainerNo
     HTMLElement::removedFromAncestor(removalType, oldParentOfRemovedTree);
     if (!parentNode() && is<Element>(oldParentOfRemovedTree)) {
 #if ENABLE(VIDEO)
-        if (is<HTMLMediaElement>(oldParentOfRemovedTree))
-            downcast<HTMLMediaElement>(oldParentOfRemovedTree).sourceWasRemoved(*this);
+        if (RefPtr medialElement = dynamicDowncast<HTMLMediaElement>(oldParentOfRemovedTree))
+            medialElement->sourceWasRemoved(*this);
         else
 #endif
 #if ENABLE(MODEL_ELEMENT)
-        if (is<HTMLModelElement>(oldParentOfRemovedTree))
-            downcast<HTMLModelElement>(oldParentOfRemovedTree).sourcesChanged();
+        if (RefPtr model = dynamicDowncast<HTMLModelElement>(oldParentOfRemovedTree))
+            model->sourcesChanged();
         else
 #endif
         if (m_shouldCallSourcesChanged) {
@@ -121,6 +124,12 @@ void HTMLSourceElement::removedFromAncestor(RemovalType removalType, ContainerNo
     }
 }
 
+void HTMLSourceElement::didMoveToNewDocument(Document& oldDocument, Document& newDocument)
+{
+    HTMLElement::didMoveToNewDocument(oldDocument, newDocument);
+    ActiveDOMObject::didMoveToNewDocument(newDocument);
+}
+
 void HTMLSourceElement::scheduleErrorEvent()
 {
     LOG(Media, "HTMLSourceElement::scheduleErrorEvent - %p", this);
@@ -141,9 +150,10 @@ bool HTMLSourceElement::isURLAttribute(const Attribute& attribute) const
     return attribute.name() == srcAttr || HTMLElement::isURLAttribute(attribute);
 }
 
-const char* HTMLSourceElement::activeDOMObjectName() const
+bool HTMLSourceElement::attributeContainsURL(const Attribute& attribute) const
 {
-    return "HTMLSourceElement";
+    return attribute.name() == srcsetAttr
+        || HTMLElement::attributeContainsURL(attribute);
 }
 
 void HTMLSourceElement::stop()
@@ -151,44 +161,88 @@ void HTMLSourceElement::stop()
     cancelPendingErrorEvent();
 }
 
-void HTMLSourceElement::parseAttribute(const QualifiedName& name, const AtomString& value)
+void HTMLSourceElement::attributeChanged(const QualifiedName& name, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason attributeModificationReason)
 {
-    HTMLElement::parseAttribute(name, value);
-    if (name == srcsetAttr || name == sizesAttr || name == mediaAttr || name == typeAttr) {
+    HTMLElement::attributeChanged(name, oldValue, newValue, attributeModificationReason);
+
+    switch (name.nodeName()) {
+    case AttributeNames::typeAttr: {
+        RefPtr parent = parentElement();
+        if (m_shouldCallSourcesChanged && parent)
+            downcast<HTMLPictureElement>(*parent).sourcesChanged();
+#if ENABLE(MODEL_ELEMENT)
+        if (RefPtr parentModelElement = dynamicDowncast<HTMLModelElement>(parent.get()))
+            parentModelElement->sourcesChanged();
+#endif
+        break;
+    }
+    case AttributeNames::srcsetAttr:
+    case AttributeNames::sizesAttr:
+    case AttributeNames::mediaAttr: {
         if (name == mediaAttr)
             m_cachedParsedMediaAttribute = std::nullopt;
-        RefPtr parent = parentNode();
-        if (m_shouldCallSourcesChanged)
+        RefPtr parent = parentElement();
+        if (m_shouldCallSourcesChanged && parent)
             downcast<HTMLPictureElement>(*parent).sourcesChanged();
+        break;
     }
+    case AttributeNames::widthAttr:
+    case AttributeNames::heightAttr:
+        if (RefPtr parent = dynamicDowncast<HTMLPictureElement>(parentElement()))
+            parent->sourceDimensionAttributesChanged(*this);
+        break;
 #if ENABLE(MODEL_ELEMENT)
-    if (name == srcAttr ||  name == typeAttr) {
-        RefPtr<Element> parent = parentElement();
-        if (is<HTMLModelElement>(parent))
-            downcast<HTMLModelElement>(*parent).sourcesChanged();
-    }
+    case AttributeNames::srcAttr:
+        if (RefPtr parent = dynamicDowncast<HTMLModelElement>(parentElement()))
+            parent->sourcesChanged();
+        break;
 #endif
+    default:
+        break;
+    }
 }
 
-const MediaQuerySet* HTMLSourceElement::parsedMediaAttribute(Document& document) const
+const MQ::MediaQueryList& HTMLSourceElement::parsedMediaAttribute(Document& document) const
 {
     if (!m_cachedParsedMediaAttribute) {
-        RefPtr<const MediaQuerySet> parsedAttribute;
         auto& value = attributeWithoutSynchronization(mediaAttr);
-        if (!value.isNull())
-            parsedAttribute = MediaQuerySet::create(value, MediaQueryParserContext(document));
-        m_cachedParsedMediaAttribute = WTFMove(parsedAttribute);
+        m_cachedParsedMediaAttribute = MQ::MediaQueryParser::parse(value, MediaQueryParserContext { document });
     }
-    return m_cachedParsedMediaAttribute.value().get();
+    return m_cachedParsedMediaAttribute.value();
 }
 
-void HTMLSourceElement::attributeChanged(const QualifiedName& name, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason reason)
+Attribute HTMLSourceElement::replaceURLsInAttributeValue(const Attribute& attribute, const CSS::SerializationContext& serializationContext) const
 {
-    if (name == widthAttr || name == heightAttr) {
-        if (RefPtr parent = parentNode(); is<HTMLPictureElement>(parent))
-            downcast<HTMLPictureElement>(*parent).sourceDimensionAttributesChanged(*this);
-    }
-    HTMLElement::attributeChanged(name, oldValue, newValue, reason);
+    if (attribute.name() != srcsetAttr)
+        return attribute;
+
+    if (serializationContext.replacementURLStrings.isEmpty())
+        return attribute;
+
+    return Attribute { srcsetAttr, AtomString { replaceURLsInSrcsetAttribute(*this, StringView(attribute.value()), serializationContext) } };
+}
+
+void HTMLSourceElement::addCandidateSubresourceURLs(ListHashSet<URL>& urls) const
+{
+    getURLsFromSrcsetAttribute(*this, attributeWithoutSynchronization(srcsetAttr), urls);
+}
+
+Ref<Element> HTMLSourceElement::cloneElementWithoutAttributesAndChildren(Document& document, CustomElementRegistry*)
+{
+    Ref clone = create(document);
+#if ENABLE(ATTACHMENT_ELEMENT)
+    cloneAttachmentAssociatedElementWithoutAttributesAndChildren(clone, document);
+#endif
+    return clone;
+}
+
+void HTMLSourceElement::copyNonAttributePropertiesFromElement(const Element& source)
+{
+#if ENABLE(ATTACHMENT_ELEMENT)
+    auto& sourceElement = downcast<HTMLSourceElement>(source);
+    copyAttachmentAssociatedPropertiesFromElement(sourceElement);
+#endif
+    Element::copyNonAttributePropertiesFromElement(source);
 }
 
 }
