@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008 Apple Inc. All Rights Reserved.
+ * Copyright (C) 2008-2022 Apple Inc. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -32,9 +32,7 @@
 #include "ChromeClient.h"
 #include "DocumentLoader.h"
 #include "EventNames.h"
-#include "Frame.h"
 #include "FrameLoader.h"
-#include "FrameLoaderClient.h"
 #include "HTMLBodyElement.h"
 #include "HTMLEmbedElement.h"
 #include "HTMLHeadElement.h"
@@ -44,18 +42,21 @@
 #include "HTMLSourceElement.h"
 #include "HTMLVideoElement.h"
 #include "KeyboardEvent.h"
+#include "LocalFrame.h"
+#include "LocalFrameLoaderClient.h"
+#include "MouseEvent.h"
 #include "NodeList.h"
 #include "Page.h"
 #include "RawDataDocumentParser.h"
 #include "ScriptController.h"
 #include "ShadowRoot.h"
-#include "TypedElementDescendantIterator.h"
-#include <wtf/IsoMallocInlines.h>
+#include "TypedElementDescendantIteratorInlines.h"
+#include <wtf/TZoneMallocInlines.h>
 #include <wtf/text/StringBuilder.h>
 
 namespace WebCore {
 
-WTF_MAKE_ISO_ALLOCATED_IMPL(MediaDocument);
+WTF_MAKE_TZONE_OR_ISO_ALLOCATED_IMPL(MediaDocument);
 
 using namespace HTMLNames;
 
@@ -74,62 +75,57 @@ private:
     {
     }
 
-    void appendBytes(DocumentWriter&, const uint8_t*, size_t) final;
+    void appendBytes(DocumentWriter&, std::span<const uint8_t>) final;
     void createDocumentStructure();
 
-    HTMLMediaElement* m_mediaElement { nullptr };
+    WeakPtr<HTMLMediaElement> m_mediaElement;
     String m_outgoingReferrer;
 };
     
 void MediaDocumentParser::createDocumentStructure()
 {
-    auto& document = *this->document();
+    Ref document = *this->document();
 
-    auto rootElement = HTMLHtmlElement::create(document);
-    document.appendChild(rootElement);
-    document.setCSSTarget(rootElement.ptr());
-    rootElement->insertedByParser();
+    Ref rootElement = HTMLHtmlElement::create(document);
+    document->appendChild(rootElement);
+    document->setCSSTarget(rootElement.ptr());
 
-    if (document.frame())
-        document.frame()->injectUserScripts(UserScriptInjectionTime::DocumentStart);
+    if (RefPtr frame = document->frame())
+        frame->injectUserScripts(UserScriptInjectionTime::DocumentStart);
 
 #if PLATFORM(IOS_FAMILY)
-    auto headElement = HTMLHeadElement::create(document);
+    Ref headElement = HTMLHeadElement::create(document);
     rootElement->appendChild(headElement);
 
-    auto metaElement = HTMLMetaElement::create(document);
+    Ref metaElement = HTMLMetaElement::create(document);
     metaElement->setAttributeWithoutSynchronization(nameAttr, "viewport"_s);
     metaElement->setAttributeWithoutSynchronization(contentAttr, "width=device-width,initial-scale=1"_s);
     headElement->appendChild(metaElement);
 #endif
 
-    auto body = HTMLBodyElement::create(document);
+    Ref body = HTMLBodyElement::create(document);
     rootElement->appendChild(body);
 
-    auto videoElement = HTMLVideoElement::create(document);
-    m_mediaElement = videoElement.ptr();
+    Ref videoElement = HTMLVideoElement::create(document);
+    m_mediaElement = videoElement.get();
     videoElement->setAttributeWithoutSynchronization(controlsAttr, emptyAtom());
     videoElement->setAttributeWithoutSynchronization(autoplayAttr, emptyAtom());
-    videoElement->setAttributeWithoutSynchronization(srcAttr, AtomString { document.url().string() });
-    if (RefPtr loader = document.loader())
+    videoElement->setAttributeWithoutSynchronization(srcAttr, AtomString { document->url().string() });
+    if (RefPtr loader = document->loader())
         videoElement->setAttributeWithoutSynchronization(typeAttr, AtomString { loader->responseMIMEType() });
 
-#if !ENABLE(MODERN_MEDIA_CONTROLS)
-    videoElement->setAttribute(styleAttr, "max-width: 100%; max-height: 100%;"_s);
-#endif // !ENABLE(MODERN_MEDIA_CONTROLS)
-
     body->appendChild(videoElement);
-    document.setHasVisuallyNonEmptyCustomContent();
+    document->setHasVisuallyNonEmptyCustomContent();
 
-    RefPtr<Frame> frame = document.frame();
+    RefPtr frame = document->frame();
     if (!frame)
         return;
 
-    frame->loader().activeDocumentLoader()->setMainResourceDataBufferingPolicy(DataBufferingPolicy::DoNotBufferData);
-    frame->loader().setOutgoingReferrer(document.completeURL(m_outgoingReferrer));
+    frame->loader().protectedActiveDocumentLoader()->setMainResourceDataBufferingPolicy(DataBufferingPolicy::DoNotBufferData);
+    frame->protectedLoader()->setOutgoingReferrer(document->completeURL(m_outgoingReferrer));
 }
 
-void MediaDocumentParser::appendBytes(DocumentWriter&, const uint8_t*, size_t)
+void MediaDocumentParser::appendBytes(DocumentWriter&, std::span<const uint8_t>)
 {
     if (m_mediaElement)
         return;
@@ -138,18 +134,16 @@ void MediaDocumentParser::appendBytes(DocumentWriter&, const uint8_t*, size_t)
     finish();
 }
     
-MediaDocument::MediaDocument(Frame* frame, const Settings& settings, const URL& url)
+MediaDocument::MediaDocument(LocalFrame* frame, const Settings& settings, const URL& url)
     : HTMLDocument(frame, settings, url, { }, { DocumentClass::Media })
 {
-    setCompatibilityMode(DocumentCompatibilityMode::QuirksMode);
+    setCompatibilityMode(DocumentCompatibilityMode::NoQuirksMode);
     lockCompatibilityMode();
     if (frame)
         m_outgoingReferrer = frame->loader().outgoingReferrer();
 }
 
-MediaDocument::~MediaDocument()
-{
-}
+MediaDocument::~MediaDocument() = default;
 
 Ref<DocumentParser> MediaDocument::createParser()
 {
@@ -158,67 +152,12 @@ Ref<DocumentParser> MediaDocument::createParser()
 
 static inline HTMLVideoElement* descendantVideoElement(ContainerNode& node)
 {
-    if (is<HTMLVideoElement>(node))
-        return downcast<HTMLVideoElement>(&node);
+    if (auto* video = dynamicDowncast<HTMLVideoElement>(node))
+        return video;
 
     return descendantsOfType<HTMLVideoElement>(node).first();
 }
 
-static inline HTMLVideoElement* ancestorVideoElement(Node* node)
-{
-    while (node && !is<HTMLVideoElement>(*node))
-        node = node->parentOrShadowHostNode();
-
-    return downcast<HTMLVideoElement>(node);
-}
-
-void MediaDocument::defaultEventHandler(Event& event)
-{
-#if !ENABLE(MODERN_MEDIA_CONTROLS)
-    // Match the default Quicktime plugin behavior to allow
-    // clicking and double-clicking to pause and play the media.
-    if (!is<Node>(event.target()))
-        return;
-    auto& targetNode = downcast<Node>(*event.target());
-
-    if (RefPtr video = ancestorVideoElement(&targetNode)) {
-        if (event.type() == eventNames().clickEvent) {
-            if (!video->canPlay()) {
-                video->pause();
-                event.setDefaultHandled();
-            }
-        } else if (event.type() == eventNames().dblclickEvent) {
-            if (video->canPlay()) {
-                video->play();
-                event.setDefaultHandled();
-            }
-        }
-    }
-
-    if (!is<ContainerNode>(targetNode))
-        return;
-    auto& targetContainer = downcast<ContainerNode>(targetNode);
-
-    if (event.type() == eventNames().keydownEvent && is<KeyboardEvent>(event)) {
-        RefPtr video = descendantVideoElement(targetContainer);
-        if (!video)
-            return;
-
-        auto& keyboardEvent = downcast<KeyboardEvent>(event);
-        if (keyboardEvent.keyIdentifier() == "U+0020"_s) { // space
-            if (video->paused()) {
-                if (video->canPlay())
-                    video->play();
-            } else
-                video->pause();
-            keyboardEvent.setDefaultHandled();
-        }
-    }
-#else // !ENABLE(MODERN_MEDIA_CONTROLS)
-    UNUSED_PARAM(event);
-#endif
-}
-
 void MediaDocument::replaceMediaElementTimerFired()
 {
     RefPtr htmlBody = bodyOrFrameset();
