@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2010 Google Inc. All rights reserved.
- * Copyright (C) 2011-2018 Apple Inc. All rights reserved.
+ * Copyright (C) 2010-2014 Google Inc. All rights reserved.
+ * Copyright (C) 2011-2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -40,15 +40,20 @@
 #include "InputTypeNames.h"
 #include "KeyboardEvent.h"
 #include "LocalizedStrings.h"
+#include "NodeName.h"
 #include "PlatformLocale.h"
+#include "RenderBoxModelObjectInlines.h"
 #include "RenderTextControl.h"
 #include "StepRange.h"
 #include <limits>
 #include <wtf/ASCIICType.h>
 #include <wtf/MathExtras.h>
+#include <wtf/TZoneMallocInlines.h>
 
 namespace WebCore {
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(NumberInputType);
+
 using namespace HTMLNames;
 
 static const int numberDefaultStep = 1;
@@ -94,12 +99,12 @@ const AtomString& NumberInputType::formControlType() const
     return InputTypeNames::number();
 }
 
-void NumberInputType::setValue(const String& sanitizedValue, bool valueChanged, TextFieldEventBehavior eventBehavior)
+void NumberInputType::setValue(const String& sanitizedValue, bool valueChanged, TextFieldEventBehavior eventBehavior, TextControlSetValueSelection selection)
 {
     ASSERT(element());
     if (!valueChanged && sanitizedValue.isEmpty() && !element()->innerTextValue().isEmpty())
         updateInnerTextValue();
-    TextFieldInputType::setValue(sanitizedValue, valueChanged, eventBehavior);
+    TextFieldInputType::setValue(sanitizedValue, valueChanged, eventBehavior, selection);
 }
 
 double NumberInputType::valueAsDouble() const
@@ -110,10 +115,6 @@ double NumberInputType::valueAsDouble() const
 
 ExceptionOr<void> NumberInputType::setValueAsDouble(double newValue, TextFieldEventBehavior eventBehavior) const
 {
-    // FIXME: We should use numeric_limits<double>::max for number input type.
-    const double floatMax = std::numeric_limits<float>::max();
-    if (newValue < -floatMax || newValue > floatMax)
-        return Exception { InvalidStateError };
     ASSERT(element());
     element()->setValue(serializeForNumberType(newValue), eventBehavior);
     return { };
@@ -121,10 +122,6 @@ ExceptionOr<void> NumberInputType::setValueAsDouble(double newValue, TextFieldEv
 
 ExceptionOr<void> NumberInputType::setValueAsDecimal(const Decimal& newValue, TextFieldEventBehavior eventBehavior) const
 {
-    // FIXME: We should use numeric_limits<double>::max for number input type.
-    const Decimal floatMax = Decimal::fromDouble(std::numeric_limits<float>::max());
-    if (newValue < -floatMax || newValue > floatMax)
-        return Exception { InvalidStateError };
     ASSERT(element());
     element()->setValue(serializeForNumberType(newValue), eventBehavior);
     return { };
@@ -147,12 +144,9 @@ StepRange NumberInputType::createStepRange(AnyStepHandling anyStepHandling) cons
     static NeverDestroyed<const StepRange::StepDescription> stepDescription(numberDefaultStep, numberDefaultStepBase, numberStepScaleFactor);
 
     ASSERT(element());
-    Decimal stepBase = parseToDecimalForNumberType(element()->attributeWithoutSynchronization(minAttr), Decimal::nan());
-    if (stepBase.isNaN())
-        stepBase = parseToDecimalForNumberType(element()->attributeWithoutSynchronization(valueAttr), numberDefaultStepBase);
+    const Decimal stepBase = findStepBase(numberDefaultStepBase);
 
-    // FIXME: We should use numeric_limits<double>::max for number input type.
-    const Decimal floatMax = Decimal::fromDouble(std::numeric_limits<float>::max());
+    const Decimal doubleMax = Decimal::doubleMax();
     const Element& element = *this->element();
 
     RangeLimitations rangeLimitations = RangeLimitations::Invalid;
@@ -165,8 +159,8 @@ StepRange NumberInputType::createStepRange(AnyStepHandling anyStepHandling) cons
         }
         return defaultValue;
     };
-    Decimal minimum = extractBound(minAttr, -floatMax);
-    Decimal maximum = extractBound(maxAttr, floatMax);
+    Decimal minimum = extractBound(minAttr, -doubleMax);
+    Decimal maximum = extractBound(maxAttr, doubleMax);
 
     const Decimal step = StepRange::parseStep(anyStepHandling, stepDescription, element.attributeWithoutSynchronization(stepAttr));
     return StepRange(stepBase, rangeLimitations, minimum, maximum, step, stepDescription);
@@ -178,7 +172,7 @@ bool NumberInputType::sizeShouldIncludeDecoration(int defaultSize, int& preferre
 
     ASSERT(element());
     auto& stepString = element()->attributeWithoutSynchronization(stepAttr);
-    if (equalLettersIgnoringASCIICase(stepString, "any"))
+    if (equalLettersIgnoringASCIICase(stepString, "any"_s))
         return false;
 
     const Decimal minimum = parseToDecimalForNumberType(element()->attributeWithoutSynchronization(minAttr));
@@ -294,18 +288,26 @@ bool NumberInputType::supportsPlaceholder() const
 void NumberInputType::attributeChanged(const QualifiedName& name)
 {
     ASSERT(element());
-    if (name == maxAttr || name == minAttr) {
+    switch (name.nodeName()) {
+    case AttributeNames::maxAttr:
+    case AttributeNames::minAttr:
         if (auto* element = this->element()) {
             element->invalidateStyleForSubtree();
             if (auto* renderer = element->renderer())
                 renderer->setNeedsLayoutAndPrefWidthsRecalc();
         }
-    } else if (name == stepAttr) {
+        break;
+    case AttributeNames::classAttr:
+    case AttributeNames::stepAttr:
         if (auto* element = this->element()) {
             if (auto* renderer = element->renderer())
                 renderer->setNeedsLayoutAndPrefWidthsRecalc();
         }
+        break;
+    default:
+        break;
     }
+
     TextFieldInputType::attributeChanged(name);
 }
 
