@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2006 Eric Seidel (eric@webkit.org)
- * Copyright (C) 2008-2017 Apple Inc. All rights reserved.
+ * Copyright (C) 2008-2023 Apple Inc. All rights reserved.
  * Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
  * Copyright (C) 2012 Samsung Electronics. All rights reserved.
  *
@@ -29,6 +29,8 @@
 #pragma once
 
 #include "ChromeClient.h"
+#include "CryptoClient.h"
+#include <wtf/TZoneMalloc.h>
 #include <wtf/UniqueRef.h>
 
 // Empty client classes for use by WebCore.
@@ -44,129 +46,149 @@ class HTMLImageElement;
 class PageConfiguration;
 
 class EmptyChromeClient : public ChromeClient {
-    WTF_MAKE_FAST_ALLOCATED;
+    WTF_MAKE_TZONE_ALLOCATED(EmptyChromeClient);
 
     void chromeDestroyed() override { }
 
     void setWindowRect(const FloatRect&) final { }
-    FloatRect windowRect() final { return FloatRect(); }
+    FloatRect windowRect() const final { return FloatRect(); }
 
-    FloatRect pageRect() final { return FloatRect(); }
+    FloatRect pageRect() const final { return FloatRect(); }
 
     void focus() final { }
     void unfocus() final { }
 
-    bool canTakeFocus(FocusDirection) final { return false; }
+    bool canTakeFocus(FocusDirection) const final { return false; }
     void takeFocus(FocusDirection) final { }
 
     void focusedElementChanged(Element*) final { }
     void focusedFrameChanged(Frame*) final { }
 
-    Page* createWindow(Frame&, const FrameLoadRequest&, const WindowFeatures&, const NavigationAction&) final { return nullptr; }
+    RefPtr<Page> createWindow(LocalFrame&, const String&, const WindowFeatures&, const NavigationAction&) final { return nullptr; }
     void show() final { }
 
-    bool canRunModal() final { return false; }
+    bool canRunModal() const final { return false; }
     void runModal() final { }
 
     void setToolbarsVisible(bool) final { }
-    bool toolbarsVisible() final { return false; }
+    bool toolbarsVisible() const final { return false; }
 
     void setStatusbarVisible(bool) final { }
-    bool statusbarVisible() final { return false; }
+    bool statusbarVisible() const final { return false; }
 
     void setScrollbarsVisible(bool) final { }
-    bool scrollbarsVisible() final { return false; }
+    bool scrollbarsVisible() const final { return false; }
 
     void setMenubarVisible(bool) final { }
-    bool menubarVisible() final { return false; }
+    bool menubarVisible() const final { return false; }
 
     void setResizable(bool) final { }
 
     void addMessageToConsole(MessageSource, MessageLevel, const String&, unsigned, unsigned, const String&) final { }
+    void addMessageWithArgumentsToConsole(MessageSource, MessageLevel, const String&, std::span<const String>, unsigned, unsigned, const String&) final { }
 
     bool canRunBeforeUnloadConfirmPanel() final { return false; }
-    bool runBeforeUnloadConfirmPanel(const String&, Frame&) final { return true; }
+    bool runBeforeUnloadConfirmPanel(const String&, LocalFrame&) final { return true; }
 
-    void closeWindowSoon() final { }
+    void closeWindow() final { }
 
-    void runJavaScriptAlert(Frame&, const String&) final { }
-    bool runJavaScriptConfirm(Frame&, const String&) final { return false; }
-    bool runJavaScriptPrompt(Frame&, const String&, const String&, String&) final { return false; }
+    void rootFrameAdded(const LocalFrame&) final { }
+    void rootFrameRemoved(const LocalFrame&) final { }
+
+    void runJavaScriptAlert(LocalFrame&, const String&) final { }
+    bool runJavaScriptConfirm(LocalFrame&, const String&) final { return false; }
+    bool runJavaScriptPrompt(LocalFrame&, const String&, const String&, String&) final { return false; }
 
     bool selectItemWritingDirectionIsNatural() final { return false; }
     bool selectItemAlignmentFollowsMenuWritingDirection() final { return false; }
     RefPtr<PopupMenu> createPopupMenu(PopupMenuClient&) const final;
     RefPtr<SearchPopupMenu> createSearchPopupMenu(PopupMenuClient&) const final;
 
-    void setStatusbarText(const String&) final { }
-
     KeyboardUIMode keyboardUIMode() final { return KeyboardAccessDefault; }
 
+    bool hoverSupportedByPrimaryPointingDevice() const final { return false; };
+    bool hoverSupportedByAnyAvailablePointingDevice() const final { return false; }
+    std::optional<PointerCharacteristics> pointerCharacteristicsOfPrimaryPointingDevice() const final { return std::nullopt; };
+    OptionSet<PointerCharacteristics> pointerCharacteristicsOfAllAvailablePointingDevices() const final { return { }; }
+
     void invalidateRootView(const IntRect&) final { }
     void invalidateContentsAndRootView(const IntRect&) override { }
     void invalidateContentsForSlowScroll(const IntRect&) final { }
     void scroll(const IntSize&, const IntRect&, const IntRect&) final { }
 
     IntPoint screenToRootView(const IntPoint& p) const final { return p; }
+    IntPoint rootViewToScreen(const IntPoint& p) const final { return p; }
     IntRect rootViewToScreen(const IntRect& r) const final { return r; }
     IntPoint accessibilityScreenToRootView(const IntPoint& p) const final { return p; };
     IntRect rootViewToAccessibilityScreen(const IntRect& r) const final { return r; };
+#if PLATFORM(IOS_FAMILY)
+    void relayAccessibilityNotification(const String&, const RetainPtr<NSData>&) const final { };
+#endif
 
     void didFinishLoadingImageForElement(HTMLImageElement&) final { }
 
     PlatformPageClient platformPageClient() const final { return 0; }
-    void contentsSizeChanged(Frame&, const IntSize&) const final { }
+    void contentsSizeChanged(LocalFrame&, const IntSize&) const final { }
     void intrinsicContentsSizeChanged(const IntSize&) const final { }
 
-    void mouseDidMoveOverElement(const HitTestResult&, unsigned, const String&, TextDirection) final { }
+    void mouseDidMoveOverElement(const HitTestResult&, OptionSet<PlatformEventModifier>, const String&, TextDirection) final { }
 
-    void print(Frame&) final { }
+    void print(LocalFrame&, const StringWithDirection&) final { }
 
-    void exceededDatabaseQuota(Frame&, const String&, DatabaseDetails) final { }
+    void exceededDatabaseQuota(LocalFrame&, const String&, DatabaseDetails) final { }
 
     void reachedMaxAppCacheSize(int64_t) final { }
     void reachedApplicationCacheOriginQuota(SecurityOrigin&, int64_t) final { }
 
-#if ENABLE(INPUT_TYPE_COLOR)
-    std::unique_ptr<ColorChooser> createColorChooser(ColorChooserClient&, const Color&) final;
-#endif
+    RefPtr<ColorChooser> createColorChooser(ColorChooserClient&, const Color&) final;
 
-#if ENABLE(DATALIST_ELEMENT)
-    std::unique_ptr<DataListSuggestionPicker> createDataListSuggestionPicker(DataListSuggestionsClient&) final;
-#endif
+    RefPtr<DataListSuggestionPicker> createDataListSuggestionPicker(DataListSuggestionsClient&) final;
+    bool canShowDataListSuggestionLabels() const final { return false; }
 
-    void runOpenPanel(Frame&, FileChooser&) final;
+    RefPtr<DateTimeChooser> createDateTimeChooser(DateTimeChooserClient&) final;
+
+    void setTextIndicator(const TextIndicatorData&) const final;
+
+    DisplayRefreshMonitorFactory* displayRefreshMonitorFactory() const final;
+
+    void runOpenPanel(LocalFrame&, FileChooser&) final;
     void showShareSheet(ShareDataWithParsedURL&, CompletionHandler<void(bool)>&&) final;
     void loadIconForFiles(const Vector<String>&, FileIconLoader&) final { }
 
-    void elementDidFocus(Element&) final { }
+    void elementDidFocus(Element&, const FocusOptions&) final { }
     void elementDidBlur(Element&) final { }
 
     void setCursor(const Cursor&) final { }
     void setCursorHiddenUntilMouseMoves(bool) final { }
 
-    void scrollRectIntoView(const IntRect&) const final { }
+    void scrollContainingScrollViewsToRevealRect(const IntRect&) const final { }
+    void scrollMainFrameToRevealRect(const IntRect&) const final { }
 
-    void attachRootGraphicsLayer(Frame&, GraphicsLayer*) final { }
+    void attachRootGraphicsLayer(LocalFrame&, GraphicsLayer*) final { }
     void attachViewOverlayGraphicsLayer(GraphicsLayer*) final { }
     void setNeedsOneShotDrawingSynchronization() final { }
-    void scheduleCompositingLayerFlush() final { }
+    void triggerRenderingUpdate() final { }
 
 #if PLATFORM(WIN)
-    void setLastSetCursorToCurrentCursor() final { }
     void AXStartFrameLoad() final { }
     void AXFinishFrameLoad() final { }
 #endif
 
+#if PLATFORM(PLAYSTATION)
+    void postAccessibilityNotification(AccessibilityObject&, AXNotification) final { }
+    void postAccessibilityNodeTextChangeNotification(AccessibilityObject*, AXTextChange, unsigned, const String&) final { }
+    void postAccessibilityFrameLoadingEventNotification(AccessibilityObject*, AXLoadingEvent) final { }
+#endif
+
 #if ENABLE(IOS_TOUCH_EVENTS)
     void didPreventDefaultForEvent() final { }
 #endif
 
 #if PLATFORM(IOS_FAMILY)
     void didReceiveMobileDocType(bool) final { }
-    void setNeedsScrollNotifications(Frame&, bool) final { }
-    void didFinishContentChangeObserving(Frame&, WKContentChange) final { }
-    void notifyRevealedSelectionByScrollingFrame(Frame&) final { }
+    void setNeedsScrollNotifications(LocalFrame&, bool) final { }
+    void didFinishContentChangeObserving(LocalFrame&, WKContentChange) final { }
+    void notifyRevealedSelectionByScrollingFrame(LocalFrame&) final { }
     void didLayout(LayoutType) final { }
     void didStartOverflowScroll() final { }
     void didEndOverflowScroll() final { }
@@ -177,12 +199,14 @@ class EmptyChromeClient : public ChromeClient {
     void addOrUpdateScrollingLayer(Node*, PlatformLayer*, PlatformLayer*, const IntSize&, bool, bool) final { }
     void removeScrollingLayer(Node*, PlatformLayer*, PlatformLayer*) final { }
 
-    void webAppOrientationsUpdated() final { };
-    void showPlaybackTargetPicker(bool, RouteSharingPolicy, const String&) final { };
+    void webAppOrientationsUpdated() final { }
+    void showPlaybackTargetPicker(bool, RouteSharingPolicy, const String&) final { }
+
+    bool showDataDetectorsUIForElement(const Element&, const Event&) final { return false; }
 #endif // PLATFORM(IOS_FAMILY)
 
 #if ENABLE(ORIENTATION_EVENTS)
-    int deviceOrientation() const final { return 0; }
+    IntDegrees deviceOrientation() const final { return 0; }
 #endif
 
 #if PLATFORM(IOS_FAMILY)
@@ -193,13 +217,22 @@ class EmptyChromeClient : public ChromeClient {
     
     bool isEmptyChromeClient() const final { return true; }
 
-    void didAssociateFormControls(const Vector<RefPtr<Element>>&, Frame&) final { }
+    void didAssociateFormControls(const Vector<RefPtr<Element>>&, LocalFrame&) final { }
     bool shouldNotifyOnFormChanges() final { return false; }
 
-    RefPtr<Icon> createIconForFiles(const Vector<String>& /* filenames */) final { return nullptr; }
+    RefPtr<Icon> createIconForFiles(const Vector<String>& /* filenames */) final;
+
+    void requestCookieConsent(CompletionHandler<void(CookieConsentDecisionResult)>&&) final;
 };
 
 DiagnosticLoggingClient& emptyDiagnosticLoggingClient();
-WEBCORE_EXPORT PageConfiguration pageConfigurationWithEmptyClients(PAL::SessionID);
+WEBCORE_EXPORT PageConfiguration pageConfigurationWithEmptyClients(std::optional<PageIdentifier>, PAL::SessionID);
+
+class EmptyCryptoClient: public CryptoClient {
+    WTF_MAKE_TZONE_ALLOCATED(EmptyCryptoClient);
+public:
+    EmptyCryptoClient() = default;
+    ~EmptyCryptoClient() = default;
+};
 
 }
