@@ -36,10 +36,14 @@
 #include "ScrollingStateScrollingNode.h"
 #include "ScrollingStateTree.h"
 #include "ScrollingTree.h"
+#include "ScrollingTreeScrollingNodeDelegate.h"
+#include <wtf/TZoneMallocInlines.h>
 #include <wtf/text/TextStream.h>
 
 namespace WebCore {
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(ScrollingTreeScrollingNode);
+
 ScrollingTreeScrollingNode::ScrollingTreeScrollingNode(ScrollingTree& scrollingTree, ScrollingNodeType nodeType, ScrollingNodeID nodeID)
     : ScrollingTreeNode(scrollingTree, nodeType, nodeID)
 {
@@ -47,69 +51,80 @@ ScrollingTreeScrollingNode::ScrollingTreeScrollingNode(ScrollingTree& scrollingT
 
 ScrollingTreeScrollingNode::~ScrollingTreeScrollingNode() = default;
 
-void ScrollingTreeScrollingNode::commitStateBeforeChildren(const ScrollingStateNode& stateNode)
+bool ScrollingTreeScrollingNode::commitStateBeforeChildren(const ScrollingStateNode& stateNode)
 {
-    const ScrollingStateScrollingNode& state = downcast<ScrollingStateScrollingNode>(stateNode);
+    auto* state = dynamicDowncast<ScrollingStateScrollingNode>(stateNode);
+    if (!state)
+        return false;
 
-    if (state.hasChangedProperty(ScrollingStateNode::Property::ScrollableAreaSize))
-        m_scrollableAreaSize = state.scrollableAreaSize();
+    if (state->hasChangedProperty(ScrollingStateNode::Property::ScrollableAreaSize))
+        m_scrollableAreaSize = state->scrollableAreaSize();
 
-    if (state.hasChangedProperty(ScrollingStateNode::Property::TotalContentsSize)) {
-        if (scrollingTree().isRubberBandInProgressForNode(scrollingNodeID()))
+    if (state->hasChangedProperty(ScrollingStateNode::Property::TotalContentsSize)) {
+        if (scrollingTree()->isRubberBandInProgressForNode(scrollingNodeID()))
             m_totalContentsSizeForRubberBand = m_totalContentsSize;
         else
-            m_totalContentsSizeForRubberBand = state.totalContentsSize();
+            m_totalContentsSizeForRubberBand = state->totalContentsSize();
 
-        m_totalContentsSize = state.totalContentsSize();
+        m_totalContentsSize = state->totalContentsSize();
     }
 
-    if (state.hasChangedProperty(ScrollingStateNode::Property::ReachableContentsSize))
-        m_reachableContentsSize = state.reachableContentsSize();
+    if (state->hasChangedProperty(ScrollingStateNode::Property::ReachableContentsSize))
+        m_reachableContentsSize = state->reachableContentsSize();
 
-    if (state.hasChangedProperty(ScrollingStateNode::Property::ScrollPosition)) {
-        m_lastCommittedScrollPosition = state.scrollPosition();
-        if (m_isFirstCommit && !state.hasScrollPositionRequest())
+    if (state->hasChangedProperty(ScrollingStateNode::Property::ScrollPosition)) {
+        m_lastCommittedScrollPosition = state->scrollPosition();
+        if (m_isFirstCommit && !state->hasScrollPositionRequest())
             m_currentScrollPosition = m_lastCommittedScrollPosition;
     }
 
-    if (state.hasChangedProperty(ScrollingStateNode::Property::ScrollOrigin))
-        m_scrollOrigin = state.scrollOrigin();
+    if (state->hasChangedProperty(ScrollingStateNode::Property::ScrollOrigin))
+        m_scrollOrigin = state->scrollOrigin();
 
-    if (state.hasChangedProperty(ScrollingStateNode::Property::SnapOffsetsInfo))
-        m_snapOffsetsInfo = state.snapOffsetsInfo();
+    if (state->hasChangedProperty(ScrollingStateNode::Property::SnapOffsetsInfo))
+        m_snapOffsetsInfo = state->snapOffsetsInfo();
 
-    if (state.hasChangedProperty(ScrollingStateNode::Property::CurrentHorizontalSnapOffsetIndex))
-        m_currentHorizontalSnapPointIndex = state.currentHorizontalSnapPointIndex();
+    if (state->hasChangedProperty(ScrollingStateNode::Property::CurrentHorizontalSnapOffsetIndex))
+        m_currentHorizontalSnapPointIndex = state->currentHorizontalSnapPointIndex();
 
-    if (state.hasChangedProperty(ScrollingStateNode::Property::CurrentVerticalSnapOffsetIndex))
-        m_currentVerticalSnapPointIndex = state.currentVerticalSnapPointIndex();
+    if (state->hasChangedProperty(ScrollingStateNode::Property::CurrentVerticalSnapOffsetIndex))
+        m_currentVerticalSnapPointIndex = state->currentVerticalSnapPointIndex();
 
-    if (state.hasChangedProperty(ScrollingStateNode::Property::ScrollableAreaParams))
-        m_scrollableAreaParameters = state.scrollableAreaParameters();
+    if (state->hasChangedProperty(ScrollingStateNode::Property::ScrollableAreaParams))
+        m_scrollableAreaParameters = state->scrollableAreaParameters();
 
 #if ENABLE(SCROLLING_THREAD)
-    if (state.hasChangedProperty(ScrollingStateNode::Property::ReasonsForSynchronousScrolling))
-        m_synchronousScrollingReasons = state.synchronousScrollingReasons();
+    if (state->hasChangedProperty(ScrollingStateNode::Property::ReasonsForSynchronousScrolling))
+        m_synchronousScrollingReasons = state->synchronousScrollingReasons();
 #endif
 
-    if (state.hasChangedProperty(ScrollingStateNode::Property::ScrollContainerLayer))
-        m_scrollContainerLayer = state.scrollContainerLayer();
+    if (state->hasChangedProperty(ScrollingStateNode::Property::ScrollContainerLayer))
+        m_scrollContainerLayer = state->scrollContainerLayer();
 
-    if (state.hasChangedProperty(ScrollingStateNode::Property::ScrolledContentsLayer))
-        m_scrolledContentsLayer = state.scrolledContentsLayer();
+    if (state->hasChangedProperty(ScrollingStateNode::Property::ScrolledContentsLayer))
+        m_scrolledContentsLayer = state->scrolledContentsLayer();
+
+    return true;
 }
 
-void ScrollingTreeScrollingNode::commitStateAfterChildren(const ScrollingStateNode& stateNode)
+bool ScrollingTreeScrollingNode::commitStateAfterChildren(const ScrollingStateNode& stateNode)
 {
-    const ScrollingStateScrollingNode& scrollingStateNode = downcast<ScrollingStateScrollingNode>(stateNode);
-    if (scrollingStateNode.hasChangedProperty(ScrollingStateNode::Property::RequestedScrollPosition))
-        handleScrollPositionRequest(scrollingStateNode.requestedScrollData());
+    auto* scrollingStateNode = dynamicDowncast<ScrollingStateScrollingNode>(stateNode);
+    if (!scrollingStateNode)
+        return false;
+
+    if (scrollingStateNode->hasChangedProperty(ScrollingStateNode::Property::RequestedScrollPosition))
+        handleScrollPositionRequest(scrollingStateNode->requestedScrollData());
+
+    if (scrollingStateNode->hasChangedProperty(ScrollingStateNode::Property::KeyboardScrollData))
+        requestKeyboardScroll(scrollingStateNode->keyboardScrollData());
 
     // This synthetic bit is added back in ScrollingTree::propagateSynchronousScrollingReasons().
 #if ENABLE(SCROLLING_THREAD)
     m_synchronousScrollingReasons.remove(SynchronousScrollingReason::DescendantScrollersHaveSynchronousScrolling);
 #endif
     m_isFirstCommit = false;
+    return true;
 }
 
 void ScrollingTreeScrollingNode::didCompleteCommitForNode()
@@ -119,17 +134,81 @@ void ScrollingTreeScrollingNode::didCompleteCommitForNode()
 
 bool ScrollingTreeScrollingNode::isLatchedNode() const
 {
-    return scrollingTree().latchedNodeID() == scrollingNodeID();
+    return scrollingTree()->latchedNodeID() == scrollingNodeID();
+}
+
+bool ScrollingTreeScrollingNode::shouldRubberBandOnSide(BoxSide side, RectEdges<bool> pinnedEdges) const
+{
+    if (!pinnedEdges[side])
+        return false;
+
+    auto mainFrameRubberBandingBehavior = scrollingTree()->clientAllowsMainFrameRubberBandingOnSide(side);
+
+    if (isRootNode() && mainFrameRubberBandingBehavior == RubberBandingBehavior::Never)
+        return false;
+
+    switch (side) {
+    case BoxSide::Top:
+    case BoxSide::Bottom:
+        if (!overscrollBehaviorAllowsVerticalRubberBand())
+            return false;
+
+        // The root allows rubberbanding if it doesn't have enough content, but only if a scrollbar is allowed.
+        if (isRootNode() && canHaveVerticalScrollbar()) {
+            if (!allowsVerticalScrolling() && mainFrameRubberBandingBehavior == RubberBandingBehavior::BasedOnSize)
+                return false;
+
+            return true;
+        }
+
+        if (!allowsVerticalScrolling())
+            return false;
+
+        return verticalOverscrollBehaviorPreventsPropagation();
+
+    case BoxSide::Left:
+    case BoxSide::Right:
+        if (!overscrollBehaviorAllowsHorizontalRubberBand())
+            return false;
+
+        // The root allows rubberbanding if it doesn't have enough content, but only if a scrollbar is allowed.
+        if (isRootNode() && canHaveHorizontalScrollbar()) {
+            if (!allowsHorizontalScrolling() && mainFrameRubberBandingBehavior == RubberBandingBehavior::BasedOnSize)
+                return false;
+
+            return true;
+        }
+
+        if (!allowsHorizontalScrolling())
+            return false;
+
+        return horizontalOverscrollBehaviorPreventsPropagation();
+    }
+    return true;
 }
 
 bool ScrollingTreeScrollingNode::shouldRubberBand(const PlatformWheelEvent& wheelEvent, EventTargeting eventTargeting) const
 {
-    // We always rubber-band the latched node, or the root node.
-    // The stateless wheel event doesn't trigger rubber-band.
-    // Also rubberband when we should block scroll propagation
-    // at this node, which has overscroll behavior that is not none.
-    auto scrollPropagationInfo = computeScrollPropagation(wheelEvent.delta());
-    return (isLatchedNode() || eventTargeting == EventTargeting::NodeOnly || (isRootNode() && !wheelEvent.isNonGestureEvent()) || ( scrollPropagationInfo.shouldBlockScrollPropagation && scrollPropagationInfo.isHandled && overscrollBehaviorAllowsRubberBand()));
+    if (isLatchedNode())
+        return true;
+
+    if (eventTargeting == EventTargeting::NodeOnly)
+        return true;
+
+    if (wheelEvent.isNonGestureEvent())
+        return false;
+
+    auto pinnedEdges = edgePinnedState();
+
+    auto horizontalSide = ScrollableArea::targetSideForScrollDelta(-wheelEvent.delta(), ScrollEventAxis::Horizontal);
+    if (horizontalSide && shouldRubberBandOnSide(*horizontalSide, pinnedEdges))
+        return true;
+
+    auto verticalSide = ScrollableArea::targetSideForScrollDelta(-wheelEvent.delta(), ScrollEventAxis::Vertical);
+    if (verticalSide && shouldRubberBandOnSide(*verticalSide, pinnedEdges))
+        return true;
+
+    return false;
 }
 
 bool ScrollingTreeScrollingNode::canHandleWheelEvent(const PlatformWheelEvent& wheelEvent, EventTargeting eventTargeting) const
@@ -137,8 +216,8 @@ bool ScrollingTreeScrollingNode::canHandleWheelEvent(const PlatformWheelEvent& w
     if (!canHaveScrollbars())
         return false;
 
-    // MayBegin is used to flash scrollbars; if this node is scrollable, it can handle it.
-    if (wheelEvent.phase() == PlatformWheelEventPhase::MayBegin)
+    // MayBegin and End are used to update scrollbars; if this node is scrollable, it can handle it.
+    if (wheelEvent.phase() == PlatformWheelEventPhase::MayBegin || wheelEvent.phase() == PlatformWheelEventPhase::Ended || wheelEvent.momentumPhase() == PlatformWheelEventPhase::Ended)
         return true;
 
     if (shouldRubberBand(wheelEvent, eventTargeting))
@@ -214,67 +293,126 @@ RectEdges<bool> ScrollingTreeScrollingNode::edgePinnedState() const
 
 bool ScrollingTreeScrollingNode::isUserScrollInProgress() const
 {
-    return scrollingTree().isUserScrollInProgressForNode(scrollingNodeID());
+    return scrollingTree()->isUserScrollInProgressForNode(scrollingNodeID());
 }
 
 void ScrollingTreeScrollingNode::setUserScrollInProgress(bool isUserScrolling)
 {
-    scrollingTree().setUserScrollInProgressForNode(scrollingNodeID(), isUserScrolling);
+    scrollingTree()->setUserScrollInProgressForNode(scrollingNodeID(), isUserScrolling);
 }
 
 bool ScrollingTreeScrollingNode::isScrollSnapInProgress() const
 {
-    return scrollingTree().isScrollSnapInProgressForNode(scrollingNodeID());
+    return scrollingTree()->isScrollSnapInProgressForNode(scrollingNodeID());
 }
 
 void ScrollingTreeScrollingNode::setScrollSnapInProgress(bool isSnapping)
 {
-    scrollingTree().setNodeScrollSnapInProgress(scrollingNodeID(), isSnapping);
+    scrollingTree()->setNodeScrollSnapInProgress(scrollingNodeID(), isSnapping);
 }
 
 void ScrollingTreeScrollingNode::willStartAnimatedScroll()
 {
+    scrollingTree()->scrollingTreeNodeWillStartAnimatedScroll(*this);
 }
 
 void ScrollingTreeScrollingNode::didStopAnimatedScroll()
 {
     LOG_WITH_STREAM(Scrolling, stream << "ScrollingTreeScrollingNode " << scrollingNodeID() << " didStopAnimatedScroll");
-    scrollingTree().scrollingTreeNodeDidStopAnimatedScroll(*this);
+    scrollingTree()->scrollingTreeNodeDidStopAnimatedScroll(*this);
+}
+
+void ScrollingTreeScrollingNode::willStartWheelEventScroll()
+{
+    scrollingTree()->scrollingTreeNodeWillStartWheelEventScroll(*this);
+}
+
+void ScrollingTreeScrollingNode::didStopWheelEventScroll()
+{
+    scrollingTree()->scrollingTreeNodeDidStopWheelEventScroll(*this);
+}
+
+bool ScrollingTreeScrollingNode::startAnimatedScrollToPosition(FloatPoint destinationPosition)
+{
+    return m_delegate ? m_delegate->startAnimatedScrollToPosition(destinationPosition) : false;
+}
+
+void ScrollingTreeScrollingNode::stopAnimatedScroll()
+{
+    if (m_delegate)
+        m_delegate->stopAnimatedScroll();
+}
+
+void ScrollingTreeScrollingNode::serviceScrollAnimation(MonotonicTime currentTime)
+{
+    if (m_delegate)
+        m_delegate->serviceScrollAnimation(currentTime);
 }
 
 void ScrollingTreeScrollingNode::setScrollAnimationInProgress(bool animationInProgress)
 {
-    scrollingTree().setScrollAnimationInProgressForNode(scrollingNodeID(), animationInProgress);
+    scrollingTree()->setScrollAnimationInProgressForNode(scrollingNodeID(), animationInProgress);
+}
+
+void ScrollingTreeScrollingNode::handleKeyboardScrollRequest(const RequestedKeyboardScrollData& scrollData)
+{
+    if (m_delegate)
+        m_delegate->handleKeyboardScrollRequest(scrollData);
+}
+
+void ScrollingTreeScrollingNode::requestKeyboardScroll(const RequestedKeyboardScrollData& scrollData)
+{
+    scrollingTree()->scrollingTreeNodeRequestsKeyboardScroll(scrollingNodeID(), scrollData);
 }
 
 void ScrollingTreeScrollingNode::handleScrollPositionRequest(const RequestedScrollData& requestedScrollData)
 {
-    LOG_WITH_STREAM(Scrolling, stream << "ScrollingTreeScrollingNode " << scrollingNodeID() << " handleScrollPositionRequest() - position " << requestedScrollData.scrollPosition << " animated " << (requestedScrollData.animated == ScrollIsAnimated::Yes));
-
     stopAnimatedScroll();
 
     if (requestedScrollData.requestType == ScrollRequestType::CancelAnimatedScroll) {
-        scrollingTree().removePendingScrollAnimationForNode(scrollingNodeID());
+        ASSERT(!requestedScrollData.requestedDataBeforeAnimatedScroll);
+        LOG_WITH_STREAM(Scrolling, stream << "ScrollingTreeScrollingNode " << scrollingNodeID() << " handleScrollPositionRequest() - cancel animated scroll");
+        scrollingTree()->removePendingScrollAnimationForNode(scrollingNodeID());
         return;
     }
 
-    if (scrollingTree().scrollingTreeNodeRequestsScroll(scrollingNodeID(), requestedScrollData))
+    if (scrollingTree()->scrollingTreeNodeRequestsScroll(scrollingNodeID(), requestedScrollData))
         return;
 
+    LOG_WITH_STREAM(Scrolling, stream << "ScrollingTreeScrollingNode " << scrollingNodeID() << " handleScrollPositionRequest() with data " << requestedScrollData);
+
+    if (requestedScrollData.requestedDataBeforeAnimatedScroll) {
+        auto& [requestType, positionOrDeltaBeforeAnimatedScroll, scrollType, clamping] = *requestedScrollData.requestedDataBeforeAnimatedScroll;
+
+        switch (requestType) {
+        case ScrollRequestType::PositionUpdate:
+        case ScrollRequestType::DeltaUpdate: {
+            auto intermediatePosition = RequestedScrollData::computeDestinationPosition(currentScrollPosition(), requestType, positionOrDeltaBeforeAnimatedScroll);
+            scrollTo(intermediatePosition, scrollType, clamping);
+            break;
+        }
+        case ScrollRequestType::CancelAnimatedScroll:
+            stopAnimatedScroll();
+            break;
+        }
+    }
+
+    auto destinationPosition = requestedScrollData.destinationPosition(currentScrollPosition());
     if (requestedScrollData.animated == ScrollIsAnimated::Yes) {
-        startAnimatedScrollToPosition(requestedScrollData.scrollPosition);
+        startAnimatedScrollToPosition(destinationPosition);
         return;
     }
 
-    scrollTo(requestedScrollData.scrollPosition, requestedScrollData.scrollType, requestedScrollData.clamping);
+    scrollTo(destinationPosition, requestedScrollData.scrollType, requestedScrollData.clamping);
 }
 
 FloatPoint ScrollingTreeScrollingNode::adjustedScrollPosition(const FloatPoint& scrollPosition, ScrollClamping clamping) const
 {
+    auto adjustedPosition = m_delegate ? m_delegate->adjustedScrollPosition(scrollPosition) : scrollPosition;
     if (clamping == ScrollClamping::Clamped)
-        return clampScrollPosition(scrollPosition);
+        return clampScrollPosition(adjustedPosition);
 
-    return scrollPosition;
+    return adjustedPosition;
 }
 
 void ScrollingTreeScrollingNode::scrollBy(const FloatSize& delta, ScrollClamping clamp)
@@ -287,7 +425,7 @@ void ScrollingTreeScrollingNode::scrollTo(const FloatPoint& position, ScrollType
     if (position == m_currentScrollPosition)
         return;
 
-    scrollingTree().setIsHandlingProgrammaticScroll(scrollType == ScrollType::Programmatic);
+    scrollingTree()->setIsHandlingProgrammaticScroll(scrollType == ScrollType::Programmatic);
 
     if (scrollType == ScrollType::Programmatic)
         willDoProgrammaticScroll(position);
@@ -296,18 +434,18 @@ void ScrollingTreeScrollingNode::scrollTo(const FloatPoint& position, ScrollType
     
     LOG_WITH_STREAM(Scrolling, stream << "ScrollingTreeScrollingNode " << scrollingNodeID() << " scrollTo " << position << " adjusted to "
         << m_currentScrollPosition << " (" << scrollType << ", " << clamp << ") (delta from last committed position " << (m_lastCommittedScrollPosition - m_currentScrollPosition) << ")"
-        << " rubberbanding " << scrollingTree().isRubberBandInProgressForNode(scrollingNodeID()));
+        << " rubberbanding " << scrollingTree()->isRubberBandInProgressForNode(scrollingNodeID()));
 
     updateViewportForCurrentScrollPosition();
     currentScrollPositionChanged(scrollType);
 
-    scrollingTree().setIsHandlingProgrammaticScroll(false);
+    scrollingTree()->setIsHandlingProgrammaticScroll(false);
 }
 
 void ScrollingTreeScrollingNode::currentScrollPositionChanged(ScrollType, ScrollingLayerPositionAction action)
 {
     m_scrolledSinceLastCommit = true;
-    scrollingTree().scrollingTreeNodeDidScroll(*this, action);
+    scrollingTree()->scrollingTreeNodeDidScroll(*this, action);
 }
 
 bool ScrollingTreeScrollingNode::scrollPositionAndLayoutViewportMatch(const FloatPoint& position, std::optional<FloatRect>)
@@ -332,9 +470,9 @@ void ScrollingTreeScrollingNode::wasScrolledByDelegatedScrolling(const FloatPoin
 
     repositionRelatedLayers();
 
-    scrollingTree().notifyRelatedNodesAfterScrollPositionChange(*this);
-    scrollingTree().scrollingTreeNodeDidScroll(*this, scrollingLayerPositionAction);
-    scrollingTree().setNeedsApplyLayerPositionsAfterCommit();
+    scrollingTree()->notifyRelatedNodesAfterScrollPositionChange(*this);
+    scrollingTree()->scrollingTreeNodeDidScroll(*this, scrollingLayerPositionAction);
+    scrollingTree()->setNeedsApplyLayerPositionsAfterCommit();
 }
 
 void ScrollingTreeScrollingNode::dumpProperties(TextStream& ts, OptionSet<ScrollingStateTreeAsTextBehavior> behavior) const
@@ -439,6 +577,16 @@ ScrollPropagationInfo ScrollingTreeScrollingNode::computeScrollPropagation(const
     return propagation;
 }
 
+void ScrollingTreeScrollingNode::scrollbarVisibilityDidChange(ScrollbarOrientation orientation, bool isVisible)
+{
+    scrollingTree()->scrollingTreeNodeScrollbarVisibilityDidChange(scrollingNodeID(), orientation, isVisible);
+}
+
+void ScrollingTreeScrollingNode::scrollbarMinimumThumbLengthDidChange(ScrollbarOrientation orientation, int minimumThumbLength)
+{
+    scrollingTree()->scrollingTreeNodeScrollbarMinimumThumbLengthDidChange(scrollingNodeID(), orientation, minimumThumbLength);
+}
+
 } // namespace WebCore
 
 #endif // ENABLE(ASYNC_SCROLLING)
