@@ -31,31 +31,35 @@
 #include "config.h"
 #include "GridPositionsResolver.h"
 
+#include "AncestorSubgridIterator.h"
 #include "GridArea.h"
 #include "RenderBox.h"
 #include "RenderGrid.h"
+#include "RenderStyleInlines.h"
+#include "StyleGridData.h"
 #include <cstdlib>
+#include <wtf/text/MakeString.h>
 
 namespace WebCore {
 
 static inline bool isColumnSide(GridPositionSide side)
 {
-    return side == ColumnStartSide || side == ColumnEndSide;
+    return side == GridPositionSide::ColumnStartSide || side == GridPositionSide::ColumnEndSide;
 }
 
 static inline bool isStartSide(GridPositionSide side)
 {
-    return side == ColumnStartSide || side == RowStartSide;
+    return side == GridPositionSide::ColumnStartSide || side == GridPositionSide::RowStartSide;
 }
 
 static inline GridTrackSizingDirection directionFromSide(GridPositionSide side)
 {
-    return side == ColumnStartSide || side == ColumnEndSide ? ForColumns : ForRows;
+    return side == GridPositionSide::ColumnStartSide || side == GridPositionSide::ColumnEndSide ? GridTrackSizingDirection::ForColumns : GridTrackSizingDirection::ForRows;
 }
 
-static const String implicitNamedGridLineForSide(const String& lineName, GridPositionSide side)
+static String implicitNamedGridLineForSide(const String& lineName, GridPositionSide side)
 {
-    return lineName + (isStartSide(side) ? "-start" : "-end");
+    return makeString(lineName, isStartSide(side) ? "-start"_s : "-end"_s);
 }
 
 static unsigned explicitGridSizeForSide(const RenderGrid& gridContainer, GridPositionSide side)
@@ -66,16 +70,16 @@ static unsigned explicitGridSizeForSide(const RenderGrid& gridContainer, GridPos
 static inline GridPositionSide transposedSide(GridPositionSide side)
 {
     switch (side) {
-    case ColumnStartSide: return RowStartSide;
-    case ColumnEndSide: return RowEndSide;
-    case RowStartSide: return ColumnStartSide;
-    default: return ColumnEndSide;
+    case GridPositionSide::ColumnStartSide: return GridPositionSide::RowStartSide;
+    case GridPositionSide::ColumnEndSide: return GridPositionSide::RowEndSide;
+    case GridPositionSide::RowStartSide: return GridPositionSide::ColumnStartSide;
+    default: return GridPositionSide::ColumnEndSide;
     }
 }
 
 static std::optional<int> clampedImplicitLineForArea(const RenderStyle& style, const String& name, int min, int max, bool isRowAxis, bool isStartSide)
 {
-    const NamedGridAreaMap& areas = style.namedGridArea();
+    auto& areas = style.namedGridArea().map;
     auto gridAreaIt = areas.find(name);
     if (gridAreaIt != areas.end()) {
         const GridArea& gridArea = gridAreaIt->value;
@@ -92,13 +96,13 @@ NamedLineCollectionBase::NamedLineCollectionBase(const RenderGrid& initialGrid,
     auto direction = directionFromSide(side);
     const auto* grid = &initialGrid;
     const auto* gridContainerStyle = &grid->style();
-    bool isRowAxis = direction == ForColumns;
+    bool isRowAxis = direction == GridTrackSizingDirection::ForColumns;
 
     m_lastLine = explicitGridSizeForSide(*grid, side);
 
-    const auto& gridLineNames = isRowAxis ? gridContainerStyle->namedGridColumnLines() : gridContainerStyle->namedGridRowLines();
-    const auto& autoRepeatGridLineNames = isRowAxis ? gridContainerStyle->autoRepeatNamedGridColumnLines() : gridContainerStyle->autoRepeatNamedGridRowLines();
-    const auto& implicitGridLineNames = isRowAxis ? gridContainerStyle->implicitNamedGridColumnLines() : gridContainerStyle->implicitNamedGridRowLines();
+    auto& gridLineNames = (isRowAxis ? gridContainerStyle->namedGridColumnLines() : gridContainerStyle->namedGridRowLines()).map;
+    auto& autoRepeatGridLineNames = (isRowAxis ? gridContainerStyle->autoRepeatNamedGridColumnLines() : gridContainerStyle->autoRepeatNamedGridRowLines()).map;
+    auto& implicitGridLineNames = (isRowAxis ? gridContainerStyle->implicitNamedGridColumnLines() : gridContainerStyle->implicitNamedGridRowLines()).map;
 
     auto linesIterator = gridLineNames.find(lineName);
     m_namedLinesIndices = linesIterator == gridLineNames.end() ? nullptr : &linesIterator->value;
@@ -155,9 +159,9 @@ NamedLineCollectionBase::NamedLineCollectionBase(const RenderGrid& initialGrid,
     }
 
     ASSERT(!m_autoRepeatTotalTracks);
-    m_autoRepeatTrackListLength = (isRowAxis ? gridContainerStyle->autoRepeatOrderedNamedGridColumnLines() : gridContainerStyle->autoRepeatOrderedNamedGridRowLines()).size();
+    m_autoRepeatTrackListLength = (isRowAxis ? gridContainerStyle->autoRepeatOrderedNamedGridColumnLines() : gridContainerStyle->autoRepeatOrderedNamedGridRowLines()).map.size();
     if (m_autoRepeatTrackListLength) {
-        unsigned namedLines = (isRowAxis ? gridContainerStyle->orderedNamedGridColumnLines() : gridContainerStyle->orderedNamedGridRowLines()).size();
+        unsigned namedLines = (isRowAxis ? gridContainerStyle->orderedNamedGridColumnLines() : gridContainerStyle->orderedNamedGridRowLines()).map.size();
         unsigned totalLines = m_lastLine + 1;
         if (namedLines < totalLines) {
             // auto repeat in a subgrid specifies the line names that should be repeated, not
@@ -179,6 +183,8 @@ void NamedLineCollectionBase::ensureInheritedNamedIndices()
 
 bool NamedLineCollectionBase::contains(unsigned line) const
 {
+    ASSERT(hasNamedLines());
+
     if (line > m_lastLine)
         return false;
 
@@ -186,6 +192,9 @@ bool NamedLineCollectionBase::contains(unsigned line) const
         return Indices && Indices->find(line) != notFound;
     };
 
+    if (contains(m_implicitNamedLinesIndices, line))
+        return true;
+
     if (!m_autoRepeatTrackListLength || line < m_insertionPoint)
         return contains(m_namedLinesIndices, line);
 
@@ -226,78 +235,82 @@ NamedLineCollection::NamedLineCollection(const RenderGrid& initialGrid, const St
     auto currentSide = side;
     auto direction = directionFromSide(currentSide);
     bool initialFlipped = GridLayoutFunctions::isFlippedDirection(initialGrid, direction);
-    const RenderGrid* grid = &initialGrid;
-    bool isRowAxis = direction == ForColumns;
+    bool isRowAxis = direction == GridTrackSizingDirection::ForColumns;
 
     // If we're a subgrid, we want to inherit the line names from any ancestor grids.
-    while (isRowAxis ? grid->isSubgridColumns() : grid->isSubgridRows()) {
-        const auto* parent = downcast<RenderGrid>(grid->parent());
+    if (initialGrid.isSubgrid(direction)) {
+        for (auto& currentAncestorSubgrid : AncestorSubgridIterator(initialGrid, direction)) {
+            const auto* currentAncestorSubgridParent = downcast<RenderGrid>(currentAncestorSubgrid.parent());
 
-        // Translate our explicit grid set of lines into the coordinate space of the
-        // parent grid, adjusting direction/side as needed.
-        if (grid->isHorizontalWritingMode() != parent->isHorizontalWritingMode()) {
-            isRowAxis = !isRowAxis;
-            currentSide = transposedSide(currentSide);
-        }
-        direction = directionFromSide(currentSide);
-
-        auto span = parent->gridSpanForChild(*grid, direction);
-        search.translateTo(span, GridLayoutFunctions::isSubgridReversedDirection(*parent, direction, *grid));
-
-        auto convertToInitialSpace = [&](unsigned i) {
-            ASSERT(i >= search.startLine());
-            i -= search.startLine();
-            if (GridLayoutFunctions::isFlippedDirection(*parent, direction) != initialFlipped) {
-                ASSERT(m_lastLine >= i);
-                i = m_lastLine - i;
+            // auto-placed subgrids inside a masonry grid do not inherit any line names
+            if ((currentAncestorSubgridParent->areMasonryRows() && (currentAncestorSubgrid.style().gridItemColumnStart().isAuto() || currentAncestorSubgrid.style().gridItemColumnStart().isSpan())) || (currentAncestorSubgridParent->areMasonryColumns() && (currentAncestorSubgrid.style().gridItemRowStart().isAuto() || currentAncestorSubgrid.style().gridItemRowStart().isSpan())))
+                return;
+            // Translate our explicit grid set of lines into the coordinate space of the
+            // parent grid, adjusting direction/side as needed.
+            if (currentAncestorSubgrid.isHorizontalWritingMode() != currentAncestorSubgridParent->isHorizontalWritingMode()) {
+                isRowAxis = !isRowAxis;
+                currentSide = transposedSide(currentSide);
             }
-            return i;
-        };
+            direction = directionFromSide(currentSide);
 
-        // Create a line collection for the parent grid, and check to see if any of our lines
-        // are present. If we find any, add them to a locally stored line name list (with numbering
-        // relative to our grid).
-        bool appended = false;
-        NamedLineCollectionBase parentCollection(*parent, name, currentSide, nameIsAreaName);
-        for (unsigned i = search.startLine(); i <= search.endLine(); i++) {
-            if (parentCollection.contains(i)) {
-                ensureInheritedNamedIndices();
-                appended = true;
-                m_inheritedNamedLinesIndices.append(convertToInitialSpace(i));
+            auto span = currentAncestorSubgridParent->gridSpanForGridItem(currentAncestorSubgrid, direction);
+            search.translateTo(span, GridLayoutFunctions::isSubgridReversedDirection(*currentAncestorSubgridParent, direction, currentAncestorSubgrid));
+
+            auto convertToInitialSpace = [&](unsigned i) {
+                ASSERT(i >= search.startLine());
+                i -= search.startLine();
+                if (GridLayoutFunctions::isFlippedDirection(*currentAncestorSubgridParent, direction) != initialFlipped) {
+                    ASSERT(m_lastLine >= i);
+                    i = m_lastLine - i;
+                }
+                return i;
+            };
+
+            // Create a line collection for the parent grid, and check to see if any of our lines
+            // are present. If we find any, add them to a locally stored line name list (with numbering
+            // relative to our grid).
+            bool appended = false;
+            NamedLineCollectionBase parentCollection(*currentAncestorSubgridParent, name, currentSide, nameIsAreaName);
+            if (parentCollection.hasNamedLines()) {
+                for (unsigned i = search.startLine(); i <= search.endLine(); i++) {
+                    if (parentCollection.contains(i)) {
+                        ensureInheritedNamedIndices();
+                        appended = true;
+                        m_inheritedNamedLinesIndices.append(convertToInitialSpace(i));
+                    }
+                }
+            }
+
+            if (nameIsAreaName) {
+                // We now need to look at the grid areas for the parent (not the implicit
+                // lines for the parent!), and insert the ones that intersect as implicit
+                // lines (but in our single combined list).
+                auto implicitLine = clampedImplicitLineForArea(currentAncestorSubgridParent->style(), name, search.startLine(), search.endLine(), isRowAxis, isStartSide(side));
+                if (implicitLine) {
+                    ensureInheritedNamedIndices();
+                    appended = true;
+                    m_inheritedNamedLinesIndices.append(convertToInitialSpace(*implicitLine));
+                }
+            }
+
+            if (appended) {
+                // Re-sort m_inheritedNamedLinesIndices
+                std::sort(m_inheritedNamedLinesIndices.begin(), m_inheritedNamedLinesIndices.end());
             }
         }
-
-        if (nameIsAreaName) {
-            // We now need to look at the grid areas for the parent (not the implicit
-            // lines for the parent!), and insert the ones that intersect as implicit
-            // lines (but in our single combined list).
-            auto implicitLine = clampedImplicitLineForArea(parent->style(), name, search.startLine(), search.endLine(), isRowAxis, isStartSide(side));
-            if (implicitLine) {
-                ensureInheritedNamedIndices();
-                appended = true;
-                m_inheritedNamedLinesIndices.append(convertToInitialSpace(*implicitLine));
-            }
-        }
-
-        if (appended) {
-            // Re-sort m_inheritedNamedLinesIndices
-            std::sort(m_inheritedNamedLinesIndices.begin(), m_inheritedNamedLinesIndices.end());
-        }
-
-        grid = parent;
     }
 }
 
-bool NamedLineCollection::hasExplicitNamedLines() const
+bool NamedLineCollectionBase::hasExplicitNamedLines() const
 {
     if (m_namedLinesIndices)
         return true;
     return m_autoRepeatNamedLinesIndices && (!m_isSubgrid || m_autoRepeatLines);
 }
 
-bool NamedLineCollection::hasNamedLines() const
+bool NamedLineCollectionBase::hasNamedLines() const
 {
-    return hasExplicitNamedLines() || m_implicitNamedLinesIndices;
+    return hasExplicitNamedLines() || (m_implicitNamedLinesIndices && !m_implicitNamedLinesIndices->isEmpty());
 }
 
 unsigned NamedLineCollection::lastLine() const
@@ -305,23 +318,6 @@ unsigned NamedLineCollection::lastLine() const
     return m_lastLine;
 }
 
-bool NamedLineCollection::contains(unsigned line) const
-{
-    ASSERT(hasNamedLines());
-
-    if (line > m_lastLine)
-        return false;
-
-    auto contains = [](const Vector<unsigned>* Indices, unsigned line) {
-        return Indices && Indices->find(line) != notFound;
-    };
-
-    if (contains(m_implicitNamedLinesIndices, line))
-        return true;
-
-    return NamedLineCollectionBase::contains(line);
-}
-
 int NamedLineCollection::firstExplicitPosition() const
 {
     ASSERT(hasExplicitNamedLines());
@@ -365,7 +361,7 @@ static bool isIndefiniteSpan(GridPosition& initialPosition, GridPosition& finalP
 
 static void adjustGridPositionsFromStyle(const RenderBox& gridItem, GridTrackSizingDirection direction, GridPosition& initialPosition, GridPosition& finalPosition)
 {
-    bool isForColumns = direction == ForColumns;
+    bool isForColumns = direction == GridTrackSizingDirection::ForColumns;
     initialPosition = isForColumns ? gridItem.style().gridItemColumnStart() : gridItem.style().gridItemRowStart();
     finalPosition = isForColumns ? gridItem.style().gridItemColumnEnd() : gridItem.style().gridItemRowEnd();
 
@@ -380,16 +376,19 @@ static void adjustGridPositionsFromStyle(const RenderBox& gridItem, GridTrackSiz
     if (finalPosition.isAuto() && initialPosition.isSpan() && !initialPosition.namedGridLine().isNull())
         initialPosition.setSpanPosition(1, String());
 
-    if (isIndefiniteSpan(initialPosition, finalPosition) && is<RenderGrid>(gridItem) && downcast<RenderGrid>(gridItem).isSubgrid(direction)) {
-        // Indefinite span for an item that is subgridded in this axis.
-        int lineCount = (isForColumns ? gridItem.style().orderedNamedGridColumnLines() : gridItem.style().orderedNamedGridRowLines()).size();
+    if (isIndefiniteSpan(initialPosition, finalPosition)) {
+        auto* renderGrid = dynamicDowncast<RenderGrid>(gridItem);
+        if (renderGrid && renderGrid->isSubgrid(direction)) {
+            // Indefinite span for an item that is subgridded in this axis.
+            int lineCount = (isForColumns ? gridItem.style().orderedNamedGridColumnLines() : gridItem.style().orderedNamedGridRowLines()).map.size();
 
-        if (initialPosition.isAuto()) {
-            // Set initial position to span <line names - 1>
-            initialPosition.setSpanPosition(std::max(1, lineCount - 1), emptyString());
-        } else {
-            // Set final position to span <line names - 1>
-            finalPosition.setSpanPosition(std::max(1, lineCount - 1), emptyString());
+            if (initialPosition.isAuto()) {
+                // Set initial position to span <line names - 1>
+                initialPosition.setSpanPosition(std::max(1, lineCount - 1), String());
+            } else {
+                // Set final position to span <line names - 1>
+                finalPosition.setSpanPosition(std::max(1, lineCount - 1), String());
+            }
         }
     }
 }
@@ -398,20 +397,20 @@ unsigned GridPositionsResolver::explicitGridColumnCount(const RenderGrid& gridCo
 {
     if (gridContainer.isSubgridColumns()) {
         const RenderGrid& parent = *downcast<RenderGrid>(gridContainer.parent());
-        GridTrackSizingDirection direction = GridLayoutFunctions::flowAwareDirectionForChild(parent, gridContainer, ForColumns);
-        return parent.gridSpanForChild(gridContainer, direction).integerSpan();
+        GridTrackSizingDirection direction = GridLayoutFunctions::flowAwareDirectionForGridItem(parent, gridContainer, GridTrackSizingDirection::ForColumns);
+        return parent.gridSpanForGridItem(gridContainer, direction).integerSpan();
     }
-    return std::min<unsigned>(std::max(gridContainer.style().gridColumns().size() + gridContainer.autoRepeatCountForDirection(ForColumns), gridContainer.style().namedGridAreaColumnCount()), GridPosition::max());
+    return std::min<unsigned>(std::max(gridContainer.style().gridColumnTrackSizes().size() + gridContainer.autoRepeatCountForDirection(GridTrackSizingDirection::ForColumns), gridContainer.style().namedGridAreaColumnCount()), GridPosition::max());
 }
 
 unsigned GridPositionsResolver::explicitGridRowCount(const RenderGrid& gridContainer)
 {
     if (gridContainer.isSubgridRows()) {
         const RenderGrid& parent = *downcast<RenderGrid>(gridContainer.parent());
-        GridTrackSizingDirection direction = GridLayoutFunctions::flowAwareDirectionForChild(parent, gridContainer, ForRows);
-        return parent.gridSpanForChild(gridContainer, direction).integerSpan();
+        GridTrackSizingDirection direction = GridLayoutFunctions::flowAwareDirectionForGridItem(parent, gridContainer, GridTrackSizingDirection::ForRows);
+        return parent.gridSpanForGridItem(gridContainer, direction).integerSpan();
     }
-    return std::min<unsigned>(std::max(gridContainer.style().gridRows().size() + gridContainer.autoRepeatCountForDirection(ForRows), gridContainer.style().namedGridAreaRowCount()), GridPosition::max());
+    return std::min<unsigned>(std::max(gridContainer.style().gridRowTrackSizes().size() + gridContainer.autoRepeatCountForDirection(GridTrackSizingDirection::ForRows), gridContainer.style().namedGridAreaRowCount()), GridPosition::max());
 }
 
 static unsigned lookAheadForNamedGridLine(int start, unsigned numberOfLines, NamedLineCollection& linesCollection)
@@ -468,7 +467,7 @@ static int resolveNamedGridLinePositionFromStyle(const RenderGrid& gridContainer
 static GridSpan definiteGridSpanWithNamedLineSpanAgainstOpposite(int oppositeLine, const GridPosition& position, GridPositionSide side, NamedLineCollection& linesCollection)
 {
     int start, end;
-    if (side == RowStartSide || side == ColumnStartSide) {
+    if (side == GridPositionSide::RowStartSide || side == GridPositionSide::ColumnStartSide) {
         start = lookBackForNamedGridLine(oppositeLine - 1, position.spanPosition(), linesCollection);
         end = oppositeLine;
     } else {
@@ -517,12 +516,12 @@ static GridSpan resolveGridPositionAgainstOppositePosition(const RenderGrid& gri
 
 GridPositionSide GridPositionsResolver::initialPositionSide(GridTrackSizingDirection direction)
 {
-    return direction == ForColumns ? ColumnStartSide : RowStartSide;
+    return direction == GridTrackSizingDirection::ForColumns ? GridPositionSide::ColumnStartSide : GridPositionSide::RowStartSide;
 }
 
 GridPositionSide GridPositionsResolver::finalPositionSide(GridTrackSizingDirection direction)
 {
-    return direction == ForColumns ? ColumnEndSide : RowEndSide;
+    return direction == GridTrackSizingDirection::ForColumns ? GridPositionSide::ColumnEndSide : GridPositionSide::RowEndSide;
 }
 
 unsigned GridPositionsResolver::spanSizeForAutoPlacedItem(const RenderBox& gridItem, GridTrackSizingDirection direction)
@@ -546,7 +545,7 @@ unsigned GridPositionsResolver::spanSizeForAutoPlacedItem(const RenderBox& gridI
 static int resolveGridPositionFromStyle(const RenderGrid& gridContainer, const GridPosition& position, GridPositionSide side)
 {
     switch (position.type()) {
-    case ExplicitPosition: {
+    case GridPositionType::ExplicitPosition: {
         ASSERT(position.integerPosition());
 
         if (!position.namedGridLine().isNull())
@@ -561,7 +560,7 @@ static int resolveGridPositionFromStyle(const RenderGrid& gridContainer, const G
 
         return endOfTrack - resolvedPosition;
     }
-    case NamedGridAreaPosition:
+    case GridPositionType::NamedGridAreaPosition:
     {
         // First attempt to match the grid area's edge to a named grid area: if there is a named line with the name
         // ''<custom-ident>-start (for grid-*-start) / <custom-ident>-end'' (for grid-*-end), contributes the first such
@@ -582,8 +581,8 @@ static int resolveGridPositionFromStyle(const RenderGrid& gridContainer, const G
         // If none of the above works specs mandate to assume that all the lines in the implicit grid have this name.
         return explicitGridSizeForSide(gridContainer, side) + 1;
     }
-    case AutoPosition:
-    case SpanPosition:
+    case GridPositionType::AutoPosition:
+    case GridPositionType::SpanPosition:
         // 'auto' and span depend on the opposite position for resolution (e.g. grid-row: auto / 1 or grid-column: span 3 / "myHeader").
         ASSERT_NOT_REACHED();
         return 0;
