@@ -22,19 +22,19 @@
 #include "config.h"
 #include "SVGViewElement.h"
 
+#include "LegacyRenderSVGResource.h"
 #include "RenderElement.h"
-#include "RenderSVGResource.h"
 #include "SVGNames.h"
 #include "SVGSVGElement.h"
 #include "SVGStringList.h"
-#include <wtf/IsoMallocInlines.h>
+#include <wtf/TZoneMallocInlines.h>
 
 namespace WebCore {
 
-WTF_MAKE_ISO_ALLOCATED_IMPL(SVGViewElement);
+WTF_MAKE_TZONE_OR_ISO_ALLOCATED_IMPL(SVGViewElement);
 
 inline SVGViewElement::SVGViewElement(const QualifiedName& tagName, Document& document)
-    : SVGElement(tagName, document)
+    : SVGElement(tagName, document, makeUniqueRef<PropertyRegistry>(*this))
     , SVGFitToViewBox(this)
 {
     ASSERT(hasTagName(SVGNames::viewTag));
@@ -45,11 +45,11 @@ Ref<SVGViewElement> SVGViewElement::create(const QualifiedName& tagName, Documen
     return adoptRef(*new SVGViewElement(tagName, document));
 }
 
-void SVGViewElement::parseAttribute(const QualifiedName& name, const AtomString& value)
+void SVGViewElement::attributeChanged(const QualifiedName& name, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason attributeModificationReason)
 {
-    SVGElement::parseAttribute(name, value);
-    SVGFitToViewBox::parseAttribute(name, value);
-    SVGZoomAndPan::parseAttribute(name, value);
+    SVGFitToViewBox::parseAttribute(name, newValue);
+    SVGZoomAndPan::parseAttribute(name, newValue);
+    SVGElement::attributeChanged(name, oldValue, newValue, attributeModificationReason);
 }
 
 void SVGViewElement::svgAttributeChanged(const QualifiedName& attrName)
@@ -59,10 +59,11 @@ void SVGViewElement::svgAttributeChanged(const QualifiedName& attrName)
         return;
 
     if (SVGFitToViewBox::isKnownAttribute(attrName)) {
-        if (!m_targetElement)
+        RefPtr targetElement = m_targetElement.get();
+        if (!targetElement)
             return;
-        m_targetElement->inheritViewAttributes(*this);
-        m_targetElement->updateSVGRendererForElementChange();
+        targetElement->inheritViewAttributes(*this);
+        targetElement->updateSVGRendererForElementChange();
         return;
     }
 
