@@ -26,8 +26,6 @@
 #include "config.h"
 #include "SWServerJobQueue.h"
 
-#if ENABLE(SERVICE_WORKER)
-
 #include "ExceptionData.h"
 #include "Logging.h"
 #include "SWServer.h"
@@ -38,9 +36,13 @@
 #include "ServiceWorkerUpdateViaCache.h"
 #include "WorkerFetchResult.h"
 #include "WorkerType.h"
+#include <wtf/TZoneMallocInlines.h>
+#include <wtf/text/MakeString.h>
 
 namespace WebCore {
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(SWServerJobQueue);
+
 SWServerJobQueue::SWServerJobQueue(SWServer& server, const ServiceWorkerRegistrationKey& key)
     : m_jobTimer(*this, &SWServerJobQueue::runNextJobSynchronously)
     , m_server(server)
@@ -59,8 +61,8 @@ bool SWServerJobQueue::isCurrentlyProcessingJob(const ServiceWorkerJobDataIdenti
 
 static bool doCertificatesMatch(const CertificateInfo& first, const CertificateInfo& second)
 {
-#if PLATFORM(COCOA) && HAVE(SEC_TRUST_SERIALIZATION)
-    return first.trust() == second.trust() || certificatesMatch(first.trust(), second.trust());
+#if PLATFORM(COCOA)
+    return first.trust() == second.trust() || certificatesMatch(first.trust().get(), second.trust().get());
 #else
     // FIXME: Add support for certificate matching in CertificateInfo.
     UNUSED_PARAM(first);
@@ -69,26 +71,27 @@ static bool doCertificatesMatch(const CertificateInfo& first, const CertificateI
 #endif
 }
 
-void SWServerJobQueue::scriptFetchFinished(const ServiceWorkerJobDataIdentifier& jobDataIdentifier, WorkerFetchResult&& result)
+void SWServerJobQueue::scriptFetchFinished(const ServiceWorkerJobDataIdentifier& jobDataIdentifier, const std::optional<ProcessIdentifier>& requestingProcessIdentifier, WorkerFetchResult&& result)
 {
     if (!isCurrentlyProcessingJob(jobDataIdentifier))
         return;
 
     auto& job = firstJob();
 
-    auto* registration = m_server.getRegistration(m_registrationKey);
+    Ref server = m_server.get();
+    RefPtr registration = server->getRegistration(m_registrationKey);
     if (!registration)
         return;
 
-    auto* newestWorker = registration->getNewestWorker();
+    RefPtr newestWorker = registration->getNewestWorker();
 
     if (!result.error.isNull()) {
         // Invoke Reject Job Promise with job and TypeError.
-        m_server.rejectJob(job, ExceptionData { TypeError, makeString("Script URL ", job.scriptURL.string(), " fetch resulted in error: ", result.error.localizedDescription()) });
+        server->rejectJob(job, ExceptionData { ExceptionCode::TypeError, makeString("Script URL "_s, job.scriptURL.string(), " fetch resulted in error: "_s, result.error.localizedDescription()) });
 
         // If newestWorker is null, invoke Clear Registration algorithm passing registration as its argument.
         if (!newestWorker)
-            registration->clear();
+            registration->clear(); // Will destroy the registration.
 
         // Invoke Finish Job with job and abort these steps.
         finishCurrentJob();
@@ -105,45 +108,45 @@ void SWServerJobQueue::scriptFetchFinished(const ServiceWorkerJobDataIdentifier&
         auto scriptURLs = newestWorker->importedScriptURLs();
         if (!scriptURLs.isEmpty()) {
             m_workerFetchResult = WTFMove(result);
-            m_server.refreshImportedScripts(job, *registration, scriptURLs);
+            protectedServer()->refreshImportedScripts(job, *registration, scriptURLs, requestingProcessIdentifier);
             return;
         }
 
         // FIXME: for non classic scripts, check the script’s module record's [[ECMAScriptCode]].
 
-        RELEASE_LOG(ServiceWorker, "%p - SWServerJobQueue::scriptFetchFinished, script, certificate and imported scripts are matching for registrationID=%llu", this, registration->identifier().toUInt64());
+        RELEASE_LOG(ServiceWorker, "%p - SWServerJobQueue::scriptFetchFinished, script, certificate and imported scripts are matching for registrationID=%" PRIu64, this, registration->identifier().toUInt64());
         scriptAndImportedScriptsFetchFinished(job, *registration);
         return;
     }
 
-    m_server.updateWorker(job.identifier(), *registration, job.scriptURL, result.script, result.certificateInfo, result.contentSecurityPolicy, result.crossOriginEmbedderPolicy, result.referrerPolicy, job.workerType, { }, job.serviceWorkerPageIdentifier());
+    protectedServer()->updateWorker(job.identifier(), requestingProcessIdentifier, *registration, job.scriptURL, result.script, result.certificateInfo, result.contentSecurityPolicy, result.crossOriginEmbedderPolicy, result.referrerPolicy, job.workerType, { }, job.serviceWorkerPageIdentifier());
 }
 
-void SWServerJobQueue::importedScriptsFetchFinished(const ServiceWorkerJobDataIdentifier& jobDataIdentifier, const Vector<std::pair<URL, ScriptBuffer>>& importedScripts)
+void SWServerJobQueue::importedScriptsFetchFinished(const ServiceWorkerJobDataIdentifier& jobDataIdentifier, const Vector<std::pair<URL, ScriptBuffer>>& importedScripts, const std::optional<ProcessIdentifier>& requestingProcessIdentifier)
 {
     if (!isCurrentlyProcessingJob(jobDataIdentifier))
         return;
 
     auto& job = firstJob();
 
-    auto* registration = m_server.getRegistration(m_registrationKey);
+    RefPtr registration = protectedServer()->getRegistration(m_registrationKey);
     if (!registration)
         return;
 
-    auto* newestWorker = registration->getNewestWorker();
+    RefPtr newestWorker = registration->getNewestWorker();
     if (newestWorker && newestWorker->matchingImportedScripts(importedScripts)) {
-        RELEASE_LOG(ServiceWorker, "%p - SWServerJobQueue::importedScriptsFetchFinished, script, certificate and imported scripts are matching for registrationID=%llu", this, registration->identifier().toUInt64());
+        RELEASE_LOG(ServiceWorker, "%p - SWServerJobQueue::importedScriptsFetchFinished, script, certificate and imported scripts are matching for registrationID=%" PRIu64, this, registration->identifier().toUInt64());
         scriptAndImportedScriptsFetchFinished(job, *registration);
         return;
     }
 
-    m_server.updateWorker(job.identifier(), *registration, job.scriptURL, m_workerFetchResult.script, m_workerFetchResult.certificateInfo, m_workerFetchResult.contentSecurityPolicy, m_workerFetchResult.crossOriginEmbedderPolicy, m_workerFetchResult.referrerPolicy, job.workerType, { }, job.serviceWorkerPageIdentifier());
+    protectedServer()->updateWorker(job.identifier(), requestingProcessIdentifier, *registration, job.scriptURL, m_workerFetchResult.script, m_workerFetchResult.certificateInfo, m_workerFetchResult.contentSecurityPolicy, m_workerFetchResult.crossOriginEmbedderPolicy, m_workerFetchResult.referrerPolicy, job.workerType, { }, job.serviceWorkerPageIdentifier());
 }
 
 void SWServerJobQueue::scriptAndImportedScriptsFetchFinished(const ServiceWorkerJobData& job, SWServerRegistration& registration)
 {
     // Invoke Resolve Job Promise with job and registration.
-    m_server.resolveRegistrationJob(job, registration.data(), ShouldNotifyWhenResolved::No);
+    protectedServer()->resolveRegistrationJob(job, registration.data(), ShouldNotifyWhenResolved::No);
 
     // Invoke Finish Job with job and abort these steps.
     finishCurrentJob();
@@ -156,19 +159,24 @@ void SWServerJobQueue::scriptContextFailedToStart(const ServiceWorkerJobDataIden
         return;
 
     // If an uncaught runtime script error occurs during the above step, then:
-    auto* registration = m_server.getRegistration(m_registrationKey);
+    Ref server = m_server.get();
+    RefPtr registration = server->getRegistration(m_registrationKey);
     ASSERT(registration);
+    if (!registration || !registration->preInstallationWorker()) {
+        RELEASE_LOG_ERROR(ServiceWorker, "SWServerJobQueue::scriptContextFailedToStart registration is null (%d) or pre installation worker is null", !registration);
+        return;
+    }
 
     ASSERT(registration->preInstallationWorker());
-    registration->preInstallationWorker()->terminate();
+    registration->protectedPreInstallationWorker()->terminate();
     registration->setPreInstallationWorker(nullptr);
 
     // Invoke Reject Job Promise with job and TypeError.
-    m_server.rejectJob(firstJob(), { TypeError, message });
+    server->rejectJob(firstJob(), { ExceptionCode::TypeError, message });
 
     // If newestWorker is null, invoke Clear Registration algorithm passing registration as its argument.
     if (!registration->getNewestWorker())
-        registration->clear();
+        registration->clear(); // Will destroy the registation.
 
     // Invoke Finish Job with job and abort these steps.
     finishCurrentJob();
@@ -179,8 +187,12 @@ void SWServerJobQueue::scriptContextStarted(const ServiceWorkerJobDataIdentifier
     if (!isCurrentlyProcessingJob(jobDataIdentifier))
         return;
 
-    auto* registration = m_server.getRegistration(m_registrationKey);
+    RefPtr registration = protectedServer()->getRegistration(m_registrationKey);
     ASSERT(registration);
+    if (!registration) {
+        RELEASE_LOG_ERROR(ServiceWorker, "SWServerJobQueue::scriptContextStarted registration is null");
+        return;
+    }
 
     install(*registration, identifier);
 }
@@ -189,17 +201,18 @@ void SWServerJobQueue::scriptContextStarted(const ServiceWorkerJobDataIdentifier
 void SWServerJobQueue::install(SWServerRegistration& registration, ServiceWorkerIdentifier installingWorker)
 {
     // The Install algorithm should never be invoked with a null worker.
-    auto* worker = m_server.workerByID(installingWorker);
+    Ref server = m_server.get();
+    RefPtr worker = server->workerByID(installingWorker);
     RELEASE_ASSERT(worker);
 
-    ASSERT(registration.preInstallationWorker() == worker);
+    ASSERT(registration.preInstallationWorker() == worker.get());
     registration.setPreInstallationWorker(nullptr);
 
-    registration.updateRegistrationState(ServiceWorkerRegistrationState::Installing, worker);
+    registration.updateRegistrationState(ServiceWorkerRegistrationState::Installing, worker.get());
     registration.updateWorkerState(*worker, ServiceWorkerState::Installing);
 
     // Invoke Resolve Job Promise with job and registration.
-    m_server.resolveRegistrationJob(firstJob(), registration.data(), ShouldNotifyWhenResolved::Yes);
+    server->resolveRegistrationJob(firstJob(), registration.data(), ShouldNotifyWhenResolved::Yes);
 
     // FIXME: https://bugs.webkit.org/show_bug.cgi?id=215122. We do not need to wait for the registration promise to resolve to continue the install steps.
 }
@@ -207,7 +220,8 @@ void SWServerJobQueue::install(SWServerRegistration& registration, ServiceWorker
 // https://w3c.github.io/ServiceWorker/#install (after resolving promise).
 void SWServerJobQueue::didResolveRegistrationPromise()
 {
-    auto* registration = m_server.getRegistration(m_registrationKey);
+    Ref server = m_server.get();
+    RefPtr registration = server->getRegistration(m_registrationKey);
     ASSERT(registration);
     ASSERT(registration->installingWorker());
 
@@ -216,7 +230,7 @@ void SWServerJobQueue::didResolveRegistrationPromise()
         return;
     }
 
-    RELEASE_LOG(ServiceWorker, "%p - SWServerJobQueue::didResolveRegistrationPromise: RegistrationID=%llu. Now proceeding with install", this, registration->identifier().toUInt64());
+    RELEASE_LOG(ServiceWorker, "%p - SWServerJobQueue::didResolveRegistrationPromise: RegistrationID=%" PRIu64 ". Now proceeding with install", this, registration->identifier().toUInt64());
 
     // Queue a task to fire an event named updatefound at all the ServiceWorkerRegistration objects
     // for all the service worker clients whose creation URL matches registration's scope url and
@@ -225,7 +239,7 @@ void SWServerJobQueue::didResolveRegistrationPromise()
 
     // Queue a task to fire the InstallEvent.
     ASSERT(registration->installingWorker());
-    m_server.fireInstallEvent(*registration->installingWorker());
+    server->fireInstallEvent(*registration->protectedInstallingWorker());
 }
 
 // https://w3c.github.io/ServiceWorker/#install
@@ -234,7 +248,7 @@ void SWServerJobQueue::didFinishInstall(const ServiceWorkerJobDataIdentifier& jo
     if (!isCurrentlyProcessingJob(jobDataIdentifier))
         return;
 
-    auto* registration = worker.registration();
+    RefPtr registration = worker.registration();
     ASSERT(registration);
     ASSERT(registration->installingWorker() == &worker);
 
@@ -247,14 +261,14 @@ void SWServerJobQueue::didFinishInstall(const ServiceWorkerJobDataIdentifier& jo
 
         // If newestWorker is null, invoke Clear Registration algorithm passing registration as its argument.
         if (!registration->getNewestWorker())
-            registration->clear();
+            registration->clear(); // Will destroy the registration.
 
         // Invoke Finish Job with job and abort these steps.
         finishCurrentJob();
         return;
     }
 
-    if (auto* waitingWorker = registration->waitingWorker()) {
+    if (RefPtr waitingWorker = registration->waitingWorker()) {
         waitingWorker->terminate();
         registration->updateWorkerState(*waitingWorker, ServiceWorkerState::Redundant);
     }
@@ -303,34 +317,36 @@ void SWServerJobQueue::runNextJobSynchronously()
 void SWServerJobQueue::runRegisterJob(const ServiceWorkerJobData& job)
 {
     ASSERT(job.type == ServiceWorkerJobType::Register);
+    ASSERT(job.registrationOptions);
 
-    if (!job.isFromServiceWorkerPage && !shouldTreatAsPotentiallyTrustworthy(job.scriptURL) && !m_server.canHandleScheme(job.scriptURL.protocol()))
-        return rejectCurrentJob(ExceptionData { SecurityError, "Script URL is not potentially trustworthy"_s });
+    Ref server = m_server.get();
+    if (!job.isFromServiceWorkerPage && !shouldTreatAsPotentiallyTrustworthy(job.scriptURL) && !server->canHandleScheme(job.scriptURL.protocol()))
+        return rejectCurrentJob(ExceptionData { ExceptionCode::SecurityError, "Script URL is not potentially trustworthy"_s });
 
     // If the origin of job's script url is not job's referrer's origin, then:
     if (!protocolHostAndPortAreEqual(job.scriptURL, job.clientCreationURL))
-        return rejectCurrentJob(ExceptionData { SecurityError, "Script origin does not match the registering client's origin"_s });
+        return rejectCurrentJob(ExceptionData { ExceptionCode::SecurityError, "Script origin does not match the registering client's origin"_s });
 
     // If the origin of job's scope url is not job's referrer's origin, then:
     if (!protocolHostAndPortAreEqual(job.scopeURL, job.clientCreationURL))
-        return rejectCurrentJob(ExceptionData { SecurityError, "Scope origin does not match the registering client's origin"_s });
+        return rejectCurrentJob(ExceptionData { ExceptionCode::SecurityError, "Scope origin does not match the registering client's origin"_s });
 
     // If registration is not null (in our parlance "empty"), then:
-    if (auto* registration = m_server.getRegistration(m_registrationKey)) {
-        auto* newestWorker = registration->getNewestWorker();
-        if (newestWorker && equalIgnoringFragmentIdentifier(job.scriptURL, newestWorker->scriptURL()) && job.workerType == newestWorker->type() && job.registrationOptions.updateViaCache == registration->updateViaCache()) {
-            RELEASE_LOG(ServiceWorker, "%p - SWServerJobQueue::runRegisterJob: Found directly reusable registration %llu for job %s (DONE)", this, registration->identifier().toUInt64(), job.identifier().loggingString().utf8().data());
-            m_server.resolveRegistrationJob(job, registration->data(), ShouldNotifyWhenResolved::No);
+    if (RefPtr registration = server->getRegistration(m_registrationKey)) {
+        RefPtr newestWorker = registration->getNewestWorker();
+        if (newestWorker && equalIgnoringFragmentIdentifier(job.scriptURL, newestWorker->scriptURL()) && job.workerType == newestWorker->type() && job.registrationOptions->updateViaCache == registration->updateViaCache()) {
+            RELEASE_LOG(ServiceWorker, "%p - SWServerJobQueue::runRegisterJob: Found directly reusable registration %" PRIu64 " for job %s (DONE)", this, registration->identifier().toUInt64(), job.identifier().loggingString().utf8().data());
+            server->resolveRegistrationJob(job, registration->data(), ShouldNotifyWhenResolved::No);
             finishCurrentJob();
             return;
         }
         // This is not specified yet (https://github.com/w3c/ServiceWorker/issues/1189).
-        if (registration->updateViaCache() != job.registrationOptions.updateViaCache)
-            registration->setUpdateViaCache(job.registrationOptions.updateViaCache);
-        RELEASE_LOG(ServiceWorker, "%p - SWServerJobQueue::runRegisterJob: Found registration %llu for job %s but it needs updating", this, registration->identifier().toUInt64(), job.identifier().loggingString().utf8().data());
+        if (registration->updateViaCache() != job.registrationOptions->updateViaCache)
+            registration->setUpdateViaCache(job.registrationOptions->updateViaCache);
+        RELEASE_LOG(ServiceWorker, "%p - SWServerJobQueue::runRegisterJob: Found registration %" PRIu64 " for job %s but it needs updating", this, registration->identifier().toUInt64(), job.identifier().loggingString().utf8().data());
     } else {
-        auto newRegistration = makeUnique<SWServerRegistration>(m_server, m_registrationKey, job.registrationOptions.updateViaCache, job.scopeURL, job.scriptURL, job.serviceWorkerPageIdentifier(), NavigationPreloadState::defaultValue());
-        m_server.addRegistration(WTFMove(newRegistration));
+        Ref newRegistration = SWServerRegistration::create(server.get(), m_registrationKey, job.registrationOptions->updateViaCache, job.scopeURL, job.scriptURL, job.serviceWorkerPageIdentifier(), NavigationPreloadState::defaultValue());
+        server->addRegistration(WTFMove(newRegistration));
 
         RELEASE_LOG(ServiceWorker, "%p - SWServerJobQueue::runRegisterJob: No existing registration for job %s, constructing a new one.", this, job.identifier().loggingString().utf8().data());
     }
@@ -343,27 +359,28 @@ void SWServerJobQueue::runUnregisterJob(const ServiceWorkerJobData& job)
 {
     // If the origin of job's scope url is not job's client's origin, then:
     if (!protocolHostAndPortAreEqual(job.scopeURL, job.clientCreationURL))
-        return rejectCurrentJob(ExceptionData { SecurityError, "Origin of scope URL does not match the client's origin"_s });
+        return rejectCurrentJob(ExceptionData { ExceptionCode::SecurityError, "Origin of scope URL does not match the client's origin"_s });
 
     // Let registration be the result of running "Get Registration" algorithm passing job's scope url as the argument.
-    auto* registration = m_server.getRegistration(m_registrationKey);
+    Ref server = m_server.get();
+    RefPtr registration = server->getRegistration(m_registrationKey);
 
     // If registration is null, then:
     if (!registration) {
         // Invoke Resolve Job Promise with job and false.
-        m_server.resolveUnregistrationJob(job, m_registrationKey, false);
+        server->resolveUnregistrationJob(job, m_registrationKey, false);
         finishCurrentJob();
         return;
     }
     
     // Remove scope to registration map[job’s scope url].
-    m_server.removeFromScopeToRegistrationMap(m_registrationKey);
+    server->removeFromScopeToRegistrationMap(m_registrationKey);
 
     // Invoke Resolve Job Promise with job and true.
-    m_server.resolveUnregistrationJob(job, m_registrationKey, true);
+    server->resolveUnregistrationJob(job, m_registrationKey, true);
 
     // Invoke Try Clear Registration with registration.
-    registration->tryClear();
+    registration->tryClear(); // This may destroy the registration.
     finishCurrentJob();
 }
 
@@ -371,25 +388,26 @@ void SWServerJobQueue::runUnregisterJob(const ServiceWorkerJobData& job)
 void SWServerJobQueue::runUpdateJob(const ServiceWorkerJobData& job)
 {
     // Let registration be the result of running the Get Registration algorithm passing job's scope url as the argument.
-    auto* registration = m_server.getRegistration(m_registrationKey);
+    Ref server = m_server.get();
+    RefPtr registration = server->getRegistration(m_registrationKey);
 
     // If registration is null (in our parlance "empty") or registration's uninstalling flag is set, then:
     if (!registration)
-        return rejectCurrentJob(ExceptionData { TypeError, "Cannot update a null/nonexistent service worker registration"_s });
+        return rejectCurrentJob(ExceptionData { ExceptionCode::TypeError, "Cannot update a null/nonexistent service worker registration"_s });
 
     // Let newestWorker be the result of running Get Newest Worker algorithm passing registration as the argument.
-    auto* newestWorker = registration->getNewestWorker();
+    RefPtr newestWorker = registration->getNewestWorker();
 
     // If job’s type is update, and newestWorker is not null and its script url does not equal job’s script url, then:
     if (job.type == ServiceWorkerJobType::Update && newestWorker && !equalIgnoringFragmentIdentifier(job.scriptURL, newestWorker->scriptURL()))
-        return rejectCurrentJob(ExceptionData { TypeError, "Cannot update a service worker with a requested script URL whose newest worker has a different script URL"_s });
+        return rejectCurrentJob(ExceptionData { ExceptionCode::TypeError, "Cannot update a service worker with a requested script URL whose newest worker has a different script URL"_s });
 
-    m_server.startScriptFetch(job, *registration);
+    server->startScriptFetch(job, *registration);
 }
 
 void SWServerJobQueue::rejectCurrentJob(const ExceptionData& exceptionData)
 {
-    m_server.rejectJob(firstJob(), exceptionData);
+    protectedServer()->rejectJob(firstJob(), exceptionData);
 
     finishCurrentJob();
 }
@@ -404,7 +422,7 @@ void SWServerJobQueue::finishCurrentJob()
         runNextJob();
 }
 
-void SWServerJobQueue::removeAllJobsMatching(const Function<bool(ServiceWorkerJobData&)>& matches)
+void SWServerJobQueue::removeAllJobsMatching(NOESCAPE const Function<bool(ServiceWorkerJobData&)>& matches)
 {
     bool isFirst = true;
     bool didRemoveFirstJob = false;
@@ -440,5 +458,3 @@ void SWServerJobQueue::cancelJobsFromServiceWorker(ServiceWorkerIdentifier servi
 }
 
 } // namespace WebCore
-
-#endif // ENABLE(SERVICE_WORKER)
