@@ -28,17 +28,18 @@
 
 #include "CDATASection.h"
 #include "Comment.h"
+#include "CommonAtomStrings.h"
 #include "Document.h"
 #include "DocumentFragment.h"
 #include "DocumentType.h"
-#include "ElementAncestorIterator.h"
-#include "Frame.h"
+#include "ElementAncestorIteratorInlines.h"
 #include "FrameLoader.h"
-#include "FrameView.h"
 #include "HTMLLinkElement.h"
 #include "HTMLNames.h"
 #include "HTMLStyleElement.h"
 #include "ImageLoader.h"
+#include "LocalFrame.h"
+#include "LocalFrameView.h"
 #include "PendingScript.h"
 #include "ProcessingInstruction.h"
 #include "ResourceError.h"
@@ -52,26 +53,29 @@
 #include "ScriptSourceCode.h"
 #include "StyleScope.h"
 #include "TextResourceDecoder.h"
-#include "TreeDepthLimit.h"
 #include "XMLNSNames.h"
 #include <wtf/Ref.h>
+#include <wtf/TZoneMallocInlines.h>
 #include <wtf/Threading.h>
 #include <wtf/Vector.h>
 
 namespace WebCore {
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(XMLDocumentParser);
+
 using namespace HTMLNames;
 
+static constexpr unsigned maxXMLTreeDepth = 5000;
+
 void XMLDocumentParser::pushCurrentNode(ContainerNode* n)
 {
     ASSERT(n);
     ASSERT(m_currentNode);
     if (n != document())
         n->ref();
-    m_currentNodeStack.append(m_currentNode);
-    m_currentNode = n;
-    if (m_currentNodeStack.size() > maxDOMTreeDepth)
-        handleError(XMLErrors::fatal, "Excessive node nesting.", textPosition());
+    m_currentNodeStack.append(std::exchange(m_currentNode, n));
+    if (m_currentNodeStack.size() > maxXMLTreeDepth)
+        handleError(XMLErrors::Type::Fatal, "Excessive node nesting.", textPosition());
 }
 
 void XMLDocumentParser::popCurrentNode()
@@ -83,8 +87,7 @@ void XMLDocumentParser::popCurrentNode()
     if (m_currentNode != document())
         m_currentNode->deref();
 
-    m_currentNode = m_currentNodeStack.last();
-    m_currentNodeStack.removeLast();
+    m_currentNode = m_currentNodeStack.takeLast();
 }
 
 void XMLDocumentParser::clearCurrentNodeStack()
@@ -124,19 +127,16 @@ void XMLDocumentParser::append(RefPtr<StringImpl>&& inputSource)
     }
 
     doWrite(source);
-
-    // After parsing, dispatch image beforeload events.
-    ImageLoader::dispatchPendingBeforeLoadEvents(nullptr);
 }
 
-void XMLDocumentParser::handleError(XMLErrors::ErrorType type, const char* m, TextPosition position)
+void XMLDocumentParser::handleError(XMLErrors::Type type, const char* m, TextPosition position)
 {
     if (!m_xmlErrors)
         m_xmlErrors = makeUnique<XMLErrors>(*document());
     m_xmlErrors->handleError(type, m, position);
-    if (type != XMLErrors::warning)
+    if (type != XMLErrors::Type::Warning)
         m_sawError = true;
-    if (type == XMLErrors::fatal)
+    if (type == XMLErrors::Type::Fatal)
         stopParsing();
 }
 
@@ -147,7 +147,7 @@ void XMLDocumentParser::createLeafTextNode()
 
     ASSERT(m_bufferedText.size() == 0);
     ASSERT(!m_leafTextNode);
-    m_leafTextNode = Text::create(m_currentNode->document(), "");
+    m_leafTextNode = Text::create(m_currentNode->document(), String { emptyString() });
     m_currentNode->parserAppendChild(*m_leafTextNode);
 }
 
@@ -159,8 +159,12 @@ bool XMLDocumentParser::updateLeafTextNode()
     if (!m_leafTextNode)
         return true;
 
-    // This operation might fire mutation event, see below.
-    m_leafTextNode->appendData(String::fromUTF8(m_bufferedText.data(), m_bufferedText.size()));
+    if (isXHTMLDocument())
+        m_leafTextNode->parserAppendData(String::fromUTF8(m_bufferedText.span()));
+    else {
+        // This operation might fire mutation event, see below.
+        m_leafTextNode->appendData(String::fromUTF8(m_bufferedText.span()));
+    }
     m_bufferedText = { };
 
     m_leafTextNode = nullptr;
@@ -204,7 +208,7 @@ void XMLDocumentParser::end()
 
     if (isParsing())
         prepareToStopParsing();
-    document()->setReadyState(Document::Interactive);
+    document()->setReadyState(Document::ReadyState::Interactive);
     clearCurrentNodeStack();
     document()->finishedParsing();
 }
@@ -267,18 +271,12 @@ static XMLParsingNamespaces findXMLParsingNamespaces(Element* contextElement)
 
     XMLParsingNamespaces result;
 
-    bool stopLookingForDefaultNamespace = false;
+    result.defaultNamespace = contextElement->lookupNamespaceURI(nullAtom());
 
     for (auto& element : lineageOfType<Element>(*contextElement)) {
-        if (is<SVGForeignObjectElement>(element))
-            stopLookingForDefaultNamespace = true;
-        else if (!stopLookingForDefaultNamespace)
-            result.defaultNamespace = element.namespaceURI();
-
         if (!element.hasAttributes())
             continue;
-
-        for (auto& attribute : element.attributesIterator()) {
+        for (auto& attribute : element.attributes()) {
             if (attribute.prefix() == xmlnsAtom())
                 result.prefixNamespaces.set(attribute.localName(), attribute.value());
         }
@@ -287,7 +285,7 @@ static XMLParsingNamespaces findXMLParsingNamespaces(Element* contextElement)
     return result;
 }
 
-bool XMLDocumentParser::parseDocumentFragment(const String& chunk, DocumentFragment& fragment, Element* contextElement, ParserContentPolicy parserContentPolicy)
+bool XMLDocumentParser::parseDocumentFragment(const String& chunk, DocumentFragment& fragment, Element* contextElement, OptionSet<ParserContentPolicy> parserContentPolicy)
 {
     if (!chunk.length())
         return true;
@@ -296,7 +294,7 @@ bool XMLDocumentParser::parseDocumentFragment(const String& chunk, DocumentFragm
     // http://www.whatwg.org/specs/web-apps/current-work/multipage/the-xhtml-syntax.html#xml-fragment-parsing-algorithm
     // For now we have a hack for script/style innerHTML support:
     if (contextElement && (contextElement->hasLocalName(HTMLNames::scriptTag->localName()) || contextElement->hasLocalName(HTMLNames::styleTag->localName()))) {
-        fragment.parserAppendChild(fragment.document().createTextNode(chunk));
+        fragment.parserAppendChild(fragment.document().createTextNode(String { chunk }));
         return true;
     }
 
