@@ -25,19 +25,21 @@
 
 #pragma once
 
-#include "DataReference.h"
 #include "DownloadID.h"
 #include "DownloadManager.h"
 #include "DownloadMonitor.h"
 #include "MessageSender.h"
 #include "NetworkDataTask.h"
 #include "SandboxExtension.h"
+#include "UseDownloadPlaceholder.h"
 #include <WebCore/AuthenticationChallenge.h>
 #include <WebCore/ResourceRequest.h>
 #include <memory>
 #include <pal/SessionID.h>
 #include <wtf/Noncopyable.h>
+#include <wtf/RefCountedAndCanMakeWeakPtr.h>
 #include <wtf/RetainPtr.h>
+#include <wtf/TZoneMalloc.h>
 #include <wtf/WeakPtr.h>
 
 #if PLATFORM(COCOA)
@@ -50,7 +52,6 @@ class AuthenticationChallenge;
 class BlobDataFileReference;
 class Credential;
 class ResourceError;
-class ResourceHandle;
 class ResourceRequest;
 class ResourceResponse;
 }
@@ -62,23 +63,30 @@ class NetworkDataTask;
 class NetworkSession;
 class WebPage;
 
-class Download : public IPC::MessageSender, public CanMakeWeakPtr<Download> {
-    WTF_MAKE_NONCOPYABLE(Download); WTF_MAKE_FAST_ALLOCATED;
+class Download final : public IPC::MessageSender, public RefCountedAndCanMakeWeakPtr<Download> {
+    WTF_MAKE_TZONE_ALLOCATED(Download);
+    WTF_MAKE_NONCOPYABLE(Download);
 public:
-    Download(DownloadManager&, DownloadID, NetworkDataTask&, NetworkSession&, const String& suggestedFilename = { });
+    static Ref<Download> create(DownloadManager&, DownloadID, NetworkDataTask&, NetworkSession&, const String& suggestedFilename = { });
 #if PLATFORM(COCOA)
-    Download(DownloadManager&, DownloadID, NSURLSessionDownloadTask*, NetworkSession&, const String& suggestedFilename = { });
+    static Ref<Download> create(DownloadManager&, DownloadID, NSURLSessionDownloadTask*, NetworkSession&, const String& suggestedFilename = { });
 #endif
 
     ~Download();
 
-    void resume(const IPC::DataReference& resumeData, const String& path, SandboxExtension::Handle&&);
+    void resume(std::span<const uint8_t> resumeData, const String& path, SandboxExtension::Handle&&, std::span<const uint8_t> activityAccessToken);
     enum class IgnoreDidFailCallback : bool { No, Yes };
-    void cancel(CompletionHandler<void(const IPC::DataReference&)>&&, IgnoreDidFailCallback);
+    void cancel(CompletionHandler<void(std::span<const uint8_t>)>&&, IgnoreDidFailCallback);
 #if PLATFORM(COCOA)
     void publishProgress(const URL&, SandboxExtension::Handle&&);
 #endif
 
+#if HAVE(MODERN_DOWNLOADPROGRESS)
+    void publishProgress(const URL&, std::span<const uint8_t>, WebKit::UseDownloadPlaceholder, std::span<const uint8_t>);
+    void setPlaceholderURL(NSURL *, NSData *);
+    void setFinalURL(NSURL *, NSData *);
+#endif
+
     DownloadID downloadID() const { return m_downloadID; }
     PAL::SessionID sessionID() const { return m_sessionID; }
 
@@ -87,23 +95,38 @@ public:
     void didCreateDestination(const String& path);
     void didReceiveData(uint64_t bytesWritten, uint64_t totalBytesWritten, uint64_t totalBytesExpectedToWrite);
     void didFinish();
-    void didFail(const WebCore::ResourceError&, const IPC::DataReference& resumeData);
+    void didFail(const WebCore::ResourceError&, std::span<const uint8_t> resumeData);
 
-    void applicationDidEnterBackground() { m_monitor.applicationDidEnterBackground(); }
-    void applicationWillEnterForeground() { m_monitor.applicationWillEnterForeground(); }
-    DownloadManager& manager() const { return m_downloadManager; }
+    void applicationDidEnterBackground() { protectedMonitor()->applicationDidEnterBackground(); }
+    void applicationWillEnterForeground() { protectedMonitor()->applicationWillEnterForeground(); }
+    DownloadManager* manager() const { return m_downloadManager.get(); }
+    void clearManager() { m_downloadManager = nullptr; }
 
     unsigned testSpeedMultiplier() const { return m_testSpeedMultiplier; }
 
 private:
+    Download(DownloadManager&, DownloadID, NetworkDataTask&, NetworkSession&, const String& suggestedFilename = { });
+#if PLATFORM(COCOA)
+    Download(DownloadManager&, DownloadID, NSURLSessionDownloadTask*, NetworkSession&, const String& suggestedFilename = { });
+#endif
+
+    Ref<DownloadMonitor> protectedMonitor() { return m_monitor; }
+
     // IPC::MessageSender
     IPC::Connection* messageSenderConnection() const override;
     uint64_t messageSenderDestinationID() const override;
 
-    void platformCancelNetworkLoad(CompletionHandler<void(const IPC::DataReference&)>&&);
+    void platformCancelNetworkLoad(CompletionHandler<void(std::span<const uint8_t>)>&&);
     void platformDestroyDownload();
+    void platformDidFinish(CompletionHandler<void()>&&);
 
-    DownloadManager& m_downloadManager;
+#if HAVE(MODERN_DOWNLOADPROGRESS)
+    void startUpdatingProgress();
+    void updateProgress(uint64_t totalBytesWritten, uint64_t totalBytesExpectedToWrite);
+    static Vector<uint8_t> updateResumeDataWithPlaceholderURL(NSURL *, std::span<const uint8_t> resumeData);
+#endif
+
+    CheckedPtr<DownloadManager> m_downloadManager;
     DownloadID m_downloadID;
     Ref<DownloadManager::Client> m_client;
 
@@ -114,13 +137,21 @@ private:
 #if PLATFORM(COCOA)
     RetainPtr<NSURLSessionDownloadTask> m_downloadTask;
     RetainPtr<NSProgress> m_progress;
+    RetainPtr<NSURL> m_placeholderURL;
+#if HAVE(MODERN_DOWNLOADPROGRESS)
+    RetainPtr<NSData> m_bookmarkData;
+    RetainPtr<NSURL> m_bookmarkURL;
+    bool m_canUpdateProgress { false };
+    std::optional<uint64_t> m_totalBytesWritten;
+    std::optional<uint64_t> m_totalBytesExpectedToWrite;
+#endif
 #endif
     PAL::SessionID m_sessionID;
     bool m_hasReceivedData { false };
     IgnoreDidFailCallback m_ignoreDidFailCallback { IgnoreDidFailCallback::No };
     DownloadMonitor m_monitor { *this };
     unsigned m_testSpeedMultiplier { 1 };
-    CompletionHandler<void(const IPC::DataReference&)> m_cancelCompletionHandler;
+    CompletionHandler<void(std::span<const uint8_t>)> m_cancelCompletionHandler;
 };
 
 } // namespace WebKit
