@@ -4,7 +4,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2,1 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,22 +20,19 @@
 #include "config.h"
 #include "WebKitWebPage.h"
 
-#include "APIArray.h"
-#include "APIDictionary.h"
-#include "APIError.h"
-#include "APINumber.h"
+#include "APIInjectedBundleEditorClient.h"
+#include "APIInjectedBundleFormClient.h"
+#include "APIInjectedBundlePageContextMenuClient.h"
+#include "APIInjectedBundlePageLoaderClient.h"
+#include "APIInjectedBundlePageResourceLoadClient.h"
+#include "APIInjectedBundlePageUIClient.h"
 #include "APIString.h"
-#include "APIURLRequest.h"
-#include "APIURLResponse.h"
-#include "ImageOptions.h"
 #include "InjectedBundle.h"
+#include "InjectedBundlePageContextMenuClient.h"
+#include "MessageSenderInlines.h"
 #include "WebContextMenuItem.h"
-#include "WebImage.h"
 #include "WebKitConsoleMessagePrivate.h"
 #include "WebKitContextMenuPrivate.h"
-#include "WebKitDOMDocumentPrivate.h"
-#include "WebKitDOMElementPrivate.h"
-#include "WebKitDOMNodePrivate.h"
 #include "WebKitFramePrivate.h"
 #include "WebKitPrivate.h"
 #include "WebKitScriptWorldPrivate.h"
@@ -43,33 +40,51 @@
 #include "WebKitURIResponsePrivate.h"
 #include "WebKitUserMessagePrivate.h"
 #include "WebKitWebEditorPrivate.h"
+#include "WebKitWebFormManagerPrivate.h"
 #include "WebKitWebHitTestResultPrivate.h"
 #include "WebKitWebPagePrivate.h"
 #include "WebKitWebProcessEnumTypes.h"
 #include "WebPageProxyMessages.h"
 #include "WebProcess.h"
+#include <WebCore/ContextMenuContext.h>
 #include <WebCore/Document.h>
 #include <WebCore/DocumentLoader.h>
-#include <WebCore/Frame.h>
 #include <WebCore/FrameDestructionObserver.h>
 #include <WebCore/FrameLoader.h>
-#include <WebCore/FrameView.h>
 #include <WebCore/HTMLFormElement.h>
+#include <WebCore/LocalFrame.h>
+#include <WebCore/LocalFrameView.h>
 #include <glib/gi18n-lib.h>
+#include <wtf/HashMap.h>
 #include <wtf/NeverDestroyed.h>
+#include <wtf/TZoneMallocInlines.h>
 #include <wtf/glib/WTFGType.h>
 #include <wtf/text/CString.h>
-#include <wtf/text/StringBuilder.h>
+#include <wtf/text/MakeString.h>
+
+#if !ENABLE(2022_GLIB_API)
+#include "WebKitDOMDocumentPrivate.h"
+#include "WebKitDOMElementPrivate.h"
+#include "WebKitDOMNodePrivate.h"
+#endif
 
 using namespace WebKit;
 using namespace WebCore;
 
+/**
+ * WebKitWebPage:
+ *
+ * A loaded web page.
+ */
+
 enum {
     DOCUMENT_LOADED,
     SEND_REQUEST,
     CONTEXT_MENU,
     CONSOLE_MESSAGE_SENT,
+#if !ENABLE(2022_GLIB_API)
     FORM_CONTROLS_ASSOCIATED,
+#endif
     FORM_CONTROLS_ASSOCIATED_FOR_FRAME,
     WILL_SUBMIT_FORM,
     USER_MESSAGE_RECEIVED,
@@ -79,29 +94,32 @@ enum {
 
 enum {
     PROP_0,
-
-    PROP_URI
+    PROP_URI,
+    N_PROPERTIES,
 };
 
+static std::array<GParamSpec*, N_PROPERTIES> sObjProperties;
+
 struct _WebKitWebPagePrivate {
     WebPage* webPage;
 
     CString uri;
 
     GRefPtr<WebKitWebEditor> webEditor;
+    HashMap<WebKitScriptWorld*, GRefPtr<WebKitWebFormManager>> formManagerMap;
 };
 
-static guint signals[LAST_SIGNAL] = { 0, };
+static std::array<unsigned, LAST_SIGNAL> signals;
 
-WEBKIT_DEFINE_TYPE(WebKitWebPage, webkit_web_page, G_TYPE_OBJECT)
+WEBKIT_DEFINE_FINAL_TYPE(WebKitWebPage, webkit_web_page, G_TYPE_OBJECT, GObject)
 
 static void webFrameDestroyed(WebFrame*);
 
 class WebKitFrameWrapper final: public FrameDestructionObserver {
-    WTF_MAKE_FAST_ALLOCATED;
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(WebKitFrameWrapper);
 public:
     WebKitFrameWrapper(WebFrame& webFrame)
-        : FrameDestructionObserver(webFrame.coreFrame())
+        : FrameDestructionObserver(webFrame.coreLocalFrame())
         , m_webkitFrame(adoptGRef(webkitFrameCreate(&webFrame)))
     {
     }
@@ -126,15 +144,24 @@ static WebFrameMap& webFrameMap()
     return map;
 }
 
-static WebKitFrame* webkitFrameGetOrCreate(WebFrame* webFrame)
+static WebKitFrame* webkitFrameGet(WebFrame* webFrame)
 {
     auto wrapperPtr = webFrameMap().get(webFrame);
     if (wrapperPtr)
         return wrapperPtr->webkitFrame();
 
+    return nullptr;
+}
+
+static WebKitFrame* webkitFrameGetOrCreate(WebFrame* webFrame)
+{
+    if (auto* webKitFrame = webkitFrameGet(webFrame))
+        return webKitFrame;
+
     std::unique_ptr<WebKitFrameWrapper> wrapper = makeUnique<WebKitFrameWrapper>(*webFrame);
-    wrapperPtr = wrapper.get();
+    auto wrapperPtr = wrapper.get();
     webFrameMap().set(webFrame, WTFMove(wrapper));
+
     return wrapperPtr->webkitFrame();
 }
 
@@ -149,13 +176,7 @@ static void webkitWebPageSetURI(WebKitWebPage* webPage, const CString& uri)
         return;
 
     webPage->priv->uri = uri;
-    g_object_notify(G_OBJECT(webPage), "uri");
-}
-
-static void webkitWebPageDidSendConsoleMessage(WebKitWebPage* webPage, MessageSource source, MessageLevel level, const String& message, unsigned lineNumber, const String& sourceID)
-{
-    WebKitConsoleMessage consoleMessage(source, level, message, lineNumber, sourceID);
-    g_signal_emit(webPage, signals[CONSOLE_MESSAGE_SENT], 0, &consoleMessage);
+    g_object_notify_by_pspec(G_OBJECT(webPage), sObjProperties[PROP_URI]);
 }
 
 class PageLoaderClient final : public API::InjectedBundle::PageLoaderClient {
@@ -177,30 +198,62 @@ private:
 
     void didStartProvisionalLoadForFrame(WebPage&, WebFrame& frame, RefPtr<API::Object>&) override
     {
-        if (!frame.isMainFrame())
+        auto* webKitFrame = webkitFrameGet(&frame);
+        if (!webKitFrame && !frame.isMainFrame())
             return;
-        webkitWebPageSetURI(m_webPage, getDocumentLoaderURL(frame.coreFrame()->loader().provisionalDocumentLoader()));
+
+        const auto uri = getDocumentLoaderURL(frame.coreLocalFrame()->loader().provisionalDocumentLoader());
+
+        if (webKitFrame)
+            webkitFrameSetURI(webKitFrame, uri);
+
+        if (frame.isMainFrame())
+            webkitWebPageSetURI(m_webPage, uri);
     }
 
     void didReceiveServerRedirectForProvisionalLoadForFrame(WebPage&, WebFrame& frame, RefPtr<API::Object>&) override
     {
-        if (!frame.isMainFrame())
+        auto* webKitFrame = webkitFrameGet(&frame);
+        if (!webKitFrame && !frame.isMainFrame())
             return;
-        webkitWebPageSetURI(m_webPage, getDocumentLoaderURL(frame.coreFrame()->loader().provisionalDocumentLoader()));
+
+        const auto uri = getDocumentLoaderURL(frame.coreLocalFrame()->loader().provisionalDocumentLoader());
+
+        if (webKitFrame)
+            webkitFrameSetURI(webKitFrame, uri);
+
+        if (frame.isMainFrame())
+            webkitWebPageSetURI(m_webPage, uri);
     }
 
     void didSameDocumentNavigationForFrame(WebPage&, WebFrame& frame, SameDocumentNavigationType, RefPtr<API::Object>&) override
     {
-        if (!frame.isMainFrame())
+        auto* webKitFrame = webkitFrameGet(&frame);
+        if (!webKitFrame && !frame.isMainFrame())
             return;
-        webkitWebPageSetURI(m_webPage, frame.coreFrame()->document()->url().string().utf8());
+
+        const auto uri = frame.coreLocalFrame()->document()->url().string().utf8();
+
+        if (webKitFrame)
+            webkitFrameSetURI(webKitFrame, uri);
+
+        if (frame.isMainFrame())
+            webkitWebPageSetURI(m_webPage, uri);
     }
 
     void didCommitLoadForFrame(WebPage&, WebFrame& frame, RefPtr<API::Object>&) override
     {
-        if (!frame.isMainFrame())
+        auto* webKitFrame = webkitFrameGet(&frame);
+        if (!webKitFrame && !frame.isMainFrame())
             return;
-        webkitWebPageSetURI(m_webPage, getDocumentLoaderURL(frame.coreFrame()->loader().documentLoader()));
+
+        const auto uri = getDocumentLoaderURL(frame.coreLocalFrame()->loader().documentLoader());
+
+        if (webKitFrame)
+            webkitFrameSetURI(webKitFrame, uri);
+
+        if (frame.isMainFrame())
+            webkitWebPageSetURI(m_webPage, uri);
     }
 
     void didFinishDocumentLoadForFrame(WebPage&, WebFrame& frame, RefPtr<API::Object>&) override
@@ -217,9 +270,27 @@ private:
             webkitScriptWorldWindowObjectCleared(wkWorld, m_webPage, webkitFrameGetOrCreate(&frame));
     }
 
+    void globalObjectIsAvailableForFrame(WebPage&, WebFrame& frame, DOMWrapperWorld& world) override
+    {
+        // Force the creation of the JavaScript context for existing WebKitScriptWorlds to
+        // ensure WebKitScriptWorld::window-object-cleared signal is emitted.
+        auto injectedWorld = InjectedBundleScriptWorld::getOrCreate(world);
+        if (webkitScriptWorldGet(injectedWorld.ptr()))
+            frame.jsContextForWorld(injectedWorld.ptr());
+    }
+
+    void serviceWorkerGlobalObjectIsAvailableForFrame(WebPage&, WebFrame&, DOMWrapperWorld&) override
+    {
+    }
+
     WebKitWebPage* m_webPage;
 };
 
+static void webkitWebPageDidSendConsoleMessage(WebKitWebPage* webPage, MessageSource source, MessageLevel level, const String& message, unsigned lineNumber, const String& sourceID)
+{
+    WebKitConsoleMessage consoleMessage(source, level, message, lineNumber, sourceID);
+    g_signal_emit(webPage, signals[CONSOLE_MESSAGE_SENT], 0, &consoleMessage);
+}
 
 class PageResourceLoadClient final : public API::InjectedBundle::ResourceLoadClient {
 public:
@@ -229,17 +300,7 @@ public:
     }
 
 private:
-    void didInitiateLoadForResource(WebPage& page, WebFrame& frame, uint64_t identifier, const ResourceRequest& request, bool) override
-    {
-        API::Dictionary::MapType message;
-        message.set(String::fromUTF8("Page"), &page);
-        message.set(String::fromUTF8("Frame"), &frame);
-        message.set(String::fromUTF8("Identifier"), API::UInt64::create(identifier));
-        message.set(String::fromUTF8("Request"), API::URLRequest::create(request));
-        WebProcess::singleton().injectedBundle()->postMessage(String::fromUTF8("WebPage.DidInitiateLoadForResource"), API::Dictionary::create(WTFMove(message)).ptr());
-    }
-
-    void willSendRequestForFrame(WebPage& page, WebFrame&, uint64_t identifier, ResourceRequest& resourceRequest, const ResourceResponse& redirectResourceResponse) override
+    void willSendRequestForFrame(WebPage& page, WebFrame&, WebCore::ResourceLoaderIdentifier identifier, ResourceRequest& resourceRequest, const ResourceResponse& redirectResourceResponse) override
     {
         GRefPtr<WebKitURIRequest> request = adoptGRef(webkitURIRequestCreateForResourceRequest(resourceRequest));
         GRefPtr<WebKitURIResponse> redirectResponse = !redirectResourceResponse.isNull() ? adoptGRef(webkitURIResponseCreateForResourceResponse(redirectResourceResponse)) : nullptr;
@@ -252,106 +313,30 @@ private:
         }
 
         webkitURIRequestGetResourceRequest(request.get(), resourceRequest);
-        resourceRequest.setInitiatingPageID(page.webPageProxyIdentifier().toUInt64());
-
-        API::Dictionary::MapType message;
-        message.set(String::fromUTF8("Page"), &page);
-        message.set(String::fromUTF8("Identifier"), API::UInt64::create(identifier));
-        message.set(String::fromUTF8("Request"), API::URLRequest::create(resourceRequest));
-        if (!redirectResourceResponse.isNull())
-            message.set(String::fromUTF8("RedirectResponse"), API::URLResponse::create(redirectResourceResponse));
-        WebProcess::singleton().injectedBundle()->postMessage(String::fromUTF8("WebPage.DidSendRequestForResource"), API::Dictionary::create(WTFMove(message)).ptr());
     }
 
-    void didReceiveResponseForResource(WebPage& page, WebFrame&, uint64_t identifier, const ResourceResponse& response) override
+    void didReceiveResponseForResource(WebPage& page, WebFrame&, WebCore::ResourceLoaderIdentifier identifier, const ResourceResponse& response) override
     {
-        API::Dictionary::MapType message;
-        message.set(String::fromUTF8("Page"), &page);
-        message.set(String::fromUTF8("Identifier"), API::UInt64::create(identifier));
-        message.set(String::fromUTF8("Response"), API::URLResponse::create(response));
-        WebProcess::singleton().injectedBundle()->postMessage(String::fromUTF8("WebPage.DidReceiveResponseForResource"), API::Dictionary::create(WTFMove(message)).ptr());
-
         // Post on the console as well to be consistent with the inspector.
         if (response.httpStatusCode() >= 400) {
-            StringBuilder errorMessage;
-            errorMessage.appendLiteral("Failed to load resource: the server responded with a status of ");
-            errorMessage.appendNumber(response.httpStatusCode());
-            errorMessage.appendLiteral(" (");
-            errorMessage.append(response.httpStatusText());
-            errorMessage.append(')');
-            webkitWebPageDidSendConsoleMessage(m_webPage, MessageSource::Network, MessageLevel::Error, errorMessage.toString(), 0, response.url().string());
+            String errorMessage = makeString("Failed to load resource: the server responded with a status of "_s, response.httpStatusCode(), " ("_s, response.httpStatusText(), ')');
+            webkitWebPageDidSendConsoleMessage(m_webPage, MessageSource::Network, MessageLevel::Error, errorMessage, 0, response.url().string());
         }
     }
 
-    void didReceiveContentLengthForResource(WebPage& page, WebFrame&, uint64_t identifier, uint64_t contentLength) override
+    void didFailLoadForResource(WebPage& page, WebFrame&, WebCore::ResourceLoaderIdentifier identifier, const ResourceError& error) override
     {
-        API::Dictionary::MapType message;
-        message.set(String::fromUTF8("Page"), &page);
-        message.set(String::fromUTF8("Identifier"), API::UInt64::create(identifier));
-        message.set(String::fromUTF8("ContentLength"), API::UInt64::create(contentLength));
-        WebProcess::singleton().injectedBundle()->postMessage(String::fromUTF8("WebPage.DidReceiveContentLengthForResource"), API::Dictionary::create(WTFMove(message)).ptr());
-    }
-
-    void didFinishLoadForResource(WebPage& page, WebFrame&, uint64_t identifier) override
-    {
-        API::Dictionary::MapType message;
-        message.set(String::fromUTF8("Page"), &page);
-        message.set(String::fromUTF8("Identifier"), API::UInt64::create(identifier));
-        WebProcess::singleton().injectedBundle()->postMessage(String::fromUTF8("WebPage.DidFinishLoadForResource"), API::Dictionary::create(WTFMove(message)).ptr());
-    }
-
-    void didFailLoadForResource(WebPage& page, WebFrame&, uint64_t identifier, const ResourceError& error) override
-    {
-        API::Dictionary::MapType message;
-        message.set(String::fromUTF8("Page"), &page);
-        message.set(String::fromUTF8("Identifier"), API::UInt64::create(identifier));
-        message.set(String::fromUTF8("Error"), API::Error::create(error));
-        WebProcess::singleton().injectedBundle()->postMessage(String::fromUTF8("WebPage.DidFailLoadForResource"), API::Dictionary::create(WTFMove(message)).ptr());
-
         // Post on the console as well to be consistent with the inspector.
         if (!error.isCancellation()) {
-            StringBuilder errorMessage;
-            errorMessage.appendLiteral("Failed to load resource");
-            if (!error.localizedDescription().isEmpty()) {
-                errorMessage.appendLiteral(": ");
-                errorMessage.append(error.localizedDescription());
-            }
-            webkitWebPageDidSendConsoleMessage(m_webPage, MessageSource::Network, MessageLevel::Error, errorMessage.toString(), 0, error.failingURL().string());
+            auto errorDescription = error.localizedDescription();
+            auto errorMessage = makeString("Failed to load resource"_s, errorDescription.isEmpty() ? ""_s : ": "_s, errorDescription);
+            webkitWebPageDidSendConsoleMessage(m_webPage, MessageSource::Network, MessageLevel::Error, errorMessage, 0, error.failingURL().string());
         }
     }
 
     WebKitWebPage* m_webPage;
 };
 
-class PageContextMenuClient final : public API::InjectedBundle::PageContextMenuClient {
-public:
-    explicit PageContextMenuClient(WebKitWebPage* webPage)
-        : m_webPage(webPage)
-    {
-    }
-
-private:
-    bool getCustomMenuFromDefaultItems(WebPage&, const WebCore::HitTestResult& hitTestResult, const Vector<WebCore::ContextMenuItem>& defaultMenu, Vector<WebContextMenuItemData>& newMenu, RefPtr<API::Object>& userData) override
-    {
-        GRefPtr<WebKitContextMenu> contextMenu = adoptGRef(webkitContextMenuCreate(kitItems(defaultMenu)));
-        GRefPtr<WebKitWebHitTestResult> webHitTestResult = adoptGRef(webkitWebHitTestResultCreate(hitTestResult));
-        gboolean returnValue;
-        g_signal_emit(m_webPage, signals[CONTEXT_MENU], 0, contextMenu.get(), webHitTestResult.get(), &returnValue);
-        if (GVariant* variant = webkit_context_menu_get_user_data(contextMenu.get())) {
-            GUniquePtr<gchar> dataString(g_variant_print(variant, TRUE));
-            userData = API::String::create(String::fromUTF8(dataString.get()));
-        }
-
-        if (!returnValue)
-            return false;
-
-        webkitContextMenuPopulate(contextMenu.get(), newMenu);
-        return true;
-    }
-
-    WebKitWebPage* m_webPage;
-};
-
 class PageUIClient final : public API::InjectedBundle::PageUIClient {
 public:
     explicit PageUIClient(WebKitWebPage* webPage)
@@ -368,6 +353,40 @@ private:
     WebKitWebPage* m_webPage;
 };
 
+#if ENABLE(CONTEXT_MENUS)
+class PageContextMenuClient final : public API::InjectedBundle::PageContextMenuClient {
+public:
+    explicit PageContextMenuClient(WebKitWebPage* webPage)
+        : m_webPage(webPage)
+    {
+    }
+
+private:
+    bool getCustomMenuFromDefaultItems(WebPage&, const WebCore::HitTestResult& hitTestResult, const Vector<WebCore::ContextMenuItem>& defaultMenu, Vector<WebContextMenuItemData>& newMenu, const WebCore::ContextMenuContext& context, RefPtr<API::Object>& userData) override
+    {
+        if (context.type() != ContextMenuContext::Type::ContextMenu)
+            return false;
+
+        GRefPtr<WebKitContextMenu> contextMenu = adoptGRef(webkitContextMenuCreate(kitItems(defaultMenu)));
+        GRefPtr<WebKitWebHitTestResult> webHitTestResult = adoptGRef(webkitWebHitTestResultCreate(hitTestResult));
+        gboolean returnValue;
+        g_signal_emit(m_webPage, signals[CONTEXT_MENU], 0, contextMenu.get(), webHitTestResult.get(), &returnValue);
+        if (GVariant* variant = webkit_context_menu_get_user_data(contextMenu.get())) {
+            GUniquePtr<gchar> dataString(g_variant_print(variant, TRUE));
+            userData = API::String::create(String::fromUTF8(dataString.get()));
+        }
+
+        if (!returnValue)
+            return false;
+
+        webkitContextMenuPopulate(contextMenu.get(), newMenu);
+        return true;
+    }
+
+    WebKitWebPage* m_webPage;
+};
+#endif // ENABLE(CONTEXT_MENUS)
+
 class PageFormClient final : public API::InjectedBundle::FormClient {
 public:
     explicit PageFormClient(WebKitWebPage* webPage)
@@ -377,27 +396,61 @@ public:
 
     void willSubmitForm(WebPage*, HTMLFormElement* formElement, WebFrame* frame, WebFrame* sourceFrame, const Vector<std::pair<String, String>>& values, RefPtr<API::Object>&) override
     {
+        if (!formElement)
+            return;
+
+        if (!m_webPage->priv->formManagerMap.isEmpty()) {
+            auto* wkFrame = webkitFrameGetOrCreate(sourceFrame);
+            auto* wkTargetFrame = webkitFrameGetOrCreate(frame);
+            for (const auto& it : m_webPage->priv->formManagerMap)
+                webkitWebFormManagerWillSubmitForm(it.value.get(), webkitFrameGetJSCValueForElementInWorld(wkFrame, *formElement, it.key), wkFrame, wkTargetFrame);
+            return;
+        }
+#if !ENABLE(2022_GLIB_API)
         fireFormSubmissionEvent(WEBKIT_FORM_SUBMISSION_WILL_COMPLETE, formElement, frame, sourceFrame, values);
+#endif
     }
 
     void willSendSubmitEvent(WebPage*, HTMLFormElement* formElement, WebFrame* frame, WebFrame* sourceFrame, const Vector<std::pair<String, String>>& values) override
     {
+        if (!formElement)
+            return;
+
+        if (!m_webPage->priv->formManagerMap.isEmpty()) {
+            auto* wkFrame = webkitFrameGetOrCreate(sourceFrame);
+            auto* wkTargetFrame = webkitFrameGetOrCreate(frame);
+            for (const auto& it : m_webPage->priv->formManagerMap)
+                webkitWebFormManagerWillSendSubmitEvent(it.value.get(), webkitFrameGetJSCValueForElementInWorld(wkFrame, *formElement, it.key), wkFrame, wkTargetFrame);
+            return;
+        }
+#if !ENABLE(2022_GLIB_API)
         fireFormSubmissionEvent(WEBKIT_FORM_SUBMISSION_WILL_SEND_DOM_EVENT, formElement, frame, sourceFrame, values);
+#endif
     }
 
     void didAssociateFormControls(WebPage*, const Vector<RefPtr<Element>>& elements, WebFrame* frame) override
     {
+        auto* wkFrame = webkitFrameGetOrCreate(frame);
+        if (!m_webPage->priv->formManagerMap.isEmpty()) {
+            for (const auto& it : m_webPage->priv->formManagerMap)
+                webkitWebFormManagerDidAssociateFormControls(it.value.get(), wkFrame, webkitFrameGetJSCValuesForElementsInWorld(wkFrame, elements, it.key));
+            return;
+        }
+
+#if !ENABLE(2022_GLIB_API)
         GRefPtr<GPtrArray> formElements = adoptGRef(g_ptr_array_sized_new(elements.size()));
         for (size_t i = 0; i < elements.size(); ++i)
             g_ptr_array_add(formElements.get(), WebKit::kit(elements[i].get()));
 
         g_signal_emit(m_webPage, signals[FORM_CONTROLS_ASSOCIATED], 0, formElements.get());
-        g_signal_emit(m_webPage, signals[FORM_CONTROLS_ASSOCIATED_FOR_FRAME], 0, formElements.get(), webkitFrameGetOrCreate(frame));
+        g_signal_emit(m_webPage, signals[FORM_CONTROLS_ASSOCIATED_FOR_FRAME], 0, formElements.get(), wkFrame);
+#endif
     }
 
     bool shouldNotifyOnFormChanges(WebPage*) override { return true; }
 
 private:
+#if !ENABLE(2022_GLIB_API)
     void fireFormSubmissionEvent(WebKitFormSubmissionStep step, HTMLFormElement* formElement, WebFrame* frame, WebFrame* sourceFrame, const Vector<std::pair<String, String>>& values)
     {
         WebKitFrame* webkitTargetFrame = webkitFrameGetOrCreate(frame);
@@ -410,12 +463,30 @@ private:
             g_ptr_array_add(textFieldValues.get(), g_strdup(pair.second.utf8().data()));
         }
 
+        ALLOW_DEPRECATED_DECLARATIONS_BEGIN
         g_signal_emit(m_webPage, signals[WILL_SUBMIT_FORM], 0, WEBKIT_DOM_ELEMENT(WebKit::kit(static_cast<Node*>(formElement))), step, webkitSourceFrame, webkitTargetFrame, textFieldNames.get(), textFieldValues.get());
+        ALLOW_DEPRECATED_DECLARATIONS_END
     }
+#endif
 
     WebKitWebPage* m_webPage;
 };
 
+static void worldDestroyed(WebKitWebPage* webPage, WebKitScriptWorld* world)
+{
+    webPage->priv->formManagerMap.remove(world);
+}
+
+static void webkitWebPageDispose(GObject* object)
+{
+    auto* priv = WEBKIT_WEB_PAGE(object)->priv;
+    for (auto* world : priv->formManagerMap.keys())
+        g_object_weak_unref(G_OBJECT(world), reinterpret_cast<GWeakNotify>(worldDestroyed), object);
+    priv->formManagerMap.clear();
+
+    G_OBJECT_CLASS(webkit_web_page_parent_class)->dispose(object);
+}
+
 static void webkitWebPageGetProperty(GObject* object, guint propId, GValue* value, GParamSpec* paramSpec)
 {
     WebKitWebPage* webPage = WEBKIT_WEB_PAGE(object);
@@ -432,7 +503,7 @@ static void webkitWebPageGetProperty(GObject* object, guint propId, GValue* valu
 static void webkit_web_page_class_init(WebKitWebPageClass* klass)
 {
     GObjectClass* gObjectClass = G_OBJECT_CLASS(klass);
-
+    gObjectClass->dispose = webkitWebPageDispose;
     gObjectClass->get_property = webkitWebPageGetProperty;
 
     /**
@@ -440,15 +511,14 @@ static void webkit_web_page_class_init(WebKitWebPageClass* klass)
      *
      * The current active URI of the #WebKitWebPage.
      */
-    g_object_class_install_property(
-        gObjectClass,
-        PROP_URI,
+    sObjProperties[PROP_URI] =
         g_param_spec_string(
             "uri",
-            _("URI"),
-            _("The current active URI of the web page"),
+            nullptr, nullptr,
             0,
-            WEBKIT_PARAM_READABLE));
+            WEBKIT_PARAM_READABLE);
+
+    g_object_class_install_properties(gObjectClass, N_PROPERTIES, sObjProperties.data());
 
     /**
      * WebKitWebPage::document-loaded:
@@ -457,8 +527,7 @@ static void webkit_web_page_class_init(WebKitWebPageClass* klass)
      * This signal is emitted when the DOM document of a #WebKitWebPage has been
      * loaded.
      *
-     * You can wait for this signal to get the DOM document with
-     * webkit_web_page_get_dom_document().
+     * You can wait for this signal to get the DOM document
      */
     signals[DOCUMENT_LOADED] = g_signal_new(
         "document-loaded",
@@ -514,7 +583,7 @@ static void webkit_web_page_class_init(WebKitWebPageClass* klass)
      * build its own context menu or pass user data to the UI Process.
      * This signal is useful when the information available in the UI Process
      * is not enough to build or customize the context menu, for example, to
-     * add menu entries depending on the #WebKitDOMNode at the coordinates of the
+     * add menu entries depending on the node at the coordinates of the
      * @hit_test_result. Otherwise, it's recommended to use #WebKitWebView::context-menu
      * signal instead.
      *
@@ -530,7 +599,11 @@ static void webkit_web_page_class_init(WebKitWebPageClass* klass)
         g_signal_accumulator_true_handled, nullptr,
         g_cclosure_marshal_generic,
         G_TYPE_BOOLEAN, 2,
+#if ENABLE(CONTEXT_MENUS)
         WEBKIT_TYPE_CONTEXT_MENU,
+#else
+        G_TYPE_OBJECT,
+#endif // ENABLE(CONTEXT_MENUS)
         WEBKIT_TYPE_WEB_HIT_TEST_RESULT);
 
     /**
@@ -554,6 +627,8 @@ static void webkit_web_page_class_init(WebKitWebPageClass* klass)
         G_TYPE_NONE, 1,
         WEBKIT_TYPE_CONSOLE_MESSAGE | G_SIGNAL_TYPE_STATIC_SCOPE);
 
+#if !ENABLE(2022_GLIB_API)
+ALLOW_DEPRECATED_DECLARATIONS_BEGIN
     /**
      * WebKitWebPage::form-controls-associated:
      * @web_page: the #WebKitWebPage on which the signal is emitted
@@ -601,6 +676,8 @@ static void webkit_web_page_class_init(WebKitWebPageClass* klass)
      * keep them alive after the signal handler returns.
      *
      * Since: 2.26
+     *
+     * Deprecated: 2.40: Use #WebKitWebFormManager::form-controls-associated instead.
      */
     signals[FORM_CONTROLS_ASSOCIATED_FOR_FRAME] = g_signal_new(
         "form-controls-associated-for-frame",
@@ -656,6 +733,8 @@ static void webkit_web_page_class_init(WebKitWebPageClass* klass)
      * emitted.
      *
      * Since: 2.20
+     *
+     * Deprecated: 2.40: Use #WebKitWebFormManager::will-send-submit-event and #WebKitWebFormManager::will-submit-form instead.
      */
     signals[WILL_SUBMIT_FORM] = g_signal_new(
         "will-submit-form",
@@ -670,6 +749,8 @@ static void webkit_web_page_class_init(WebKitWebPageClass* klass)
         WEBKIT_TYPE_FRAME,
         G_TYPE_PTR_ARRAY,
         G_TYPE_PTR_ARRAY);
+ALLOW_DEPRECATED_DECLARATIONS_END
+#endif
 
     /**
      * WebKitWebPage::user-message-received:
@@ -712,58 +793,15 @@ WebKitWebPage* webkitWebPageCreate(WebPage* webPage)
 
     webPage->setInjectedBundleResourceLoadClient(makeUnique<PageResourceLoadClient>(page));
     webPage->setInjectedBundlePageLoaderClient(makeUnique<PageLoaderClient>(page));
+#if ENABLE(CONTEXT_MENUS)
     webPage->setInjectedBundleContextMenuClient(makeUnique<PageContextMenuClient>(page));
-    webPage->setInjectedBundleUIClient(makeUnique<PageUIClient>(page));
+#endif // ENABLE(CONTEXT_MENUS)
     webPage->setInjectedBundleFormClient(makeUnique<PageFormClient>(page));
+    webPage->setInjectedBundleUIClient(makeUnique<PageUIClient>(page));
 
     return page;
 }
 
-void webkitWebPageDidReceiveMessage(WebKitWebPage* page, const String& messageName, API::Dictionary& message)
-{
-#if PLATFORM(GTK)
-    if (messageName == String("GetSnapshot")) {
-        SnapshotOptions snapshotOptions = static_cast<SnapshotOptions>(static_cast<API::UInt64*>(message.get("SnapshotOptions"))->value());
-        uint64_t callbackID = static_cast<API::UInt64*>(message.get("CallbackID"))->value();
-        SnapshotRegion region = static_cast<SnapshotRegion>(static_cast<API::UInt64*>(message.get("SnapshotRegion"))->value());
-        bool transparentBackground = static_cast<API::Boolean*>(message.get("TransparentBackground"))->value();
-
-        RefPtr<WebImage> snapshotImage;
-        WebPage* webPage = page->priv->webPage;
-        if (WebCore::FrameView* frameView = webPage->mainFrameView()) {
-            WebCore::IntRect snapshotRect;
-            switch (region) {
-            case SnapshotRegionVisible:
-                snapshotRect = frameView->visibleContentRect();
-                break;
-            case SnapshotRegionFullDocument:
-                snapshotRect = WebCore::IntRect(WebCore::IntPoint(0, 0), frameView->contentsSize());
-                break;
-            default:
-                ASSERT_NOT_REACHED();
-            }
-            if (!snapshotRect.isEmpty()) {
-                Color savedBackgroundColor;
-                if (transparentBackground) {
-                    savedBackgroundColor = frameView->baseBackgroundColor();
-                    frameView->setBaseBackgroundColor(Color::transparentBlack);
-                }
-                snapshotImage = webPage->scaledSnapshotWithOptions(snapshotRect, 1, snapshotOptions | SnapshotOptionsShareable);
-                if (transparentBackground)
-                    frameView->setBaseBackgroundColor(savedBackgroundColor);
-            }
-        }
-
-        API::Dictionary::MapType messageReply;
-        messageReply.set("Page", webPage);
-        messageReply.set("CallbackID", API::UInt64::create(callbackID));
-        messageReply.set("Snapshot", snapshotImage);
-        WebProcess::singleton().injectedBundle()->postMessage("WebPage.DidGetSnapshot", API::Dictionary::create(WTFMove(messageReply)).ptr());
-    } else
-#endif
-        ASSERT_NOT_REACHED();
-}
-
 void webkitWebPageDidReceiveUserMessage(WebKitWebPage* webPage, UserMessage&& message, CompletionHandler<void(UserMessage&&)>&& completionHandler)
 {
     // Sink the floating ref.
@@ -772,6 +810,7 @@ void webkitWebPageDidReceiveUserMessage(WebKitWebPage* webPage, UserMessage&& me
     g_signal_emit(webPage, signals[USER_MESSAGE_RECEIVED], 0, userMessage.get(), &returnValue);
 }
 
+#if !ENABLE(2022_GLIB_API)
 /**
  * webkit_web_page_get_dom_document:
  * @web_page: a #WebKitWebPage
@@ -780,16 +819,19 @@ void webkitWebPageDidReceiveUserMessage(WebKitWebPage* webPage, UserMessage&& me
  *
  * Returns: (transfer none): the #WebKitDOMDocument currently loaded, or %NULL
  *    if no document is currently loaded.
+ *
+ * Deprecated: 2.40. Use JavaScriptCore API instead.
  */
 WebKitDOMDocument* webkit_web_page_get_dom_document(WebKitWebPage* webPage)
 {
     g_return_val_if_fail(WEBKIT_IS_WEB_PAGE(webPage), nullptr);
 
-    if (auto* coreFrame = webPage->priv->webPage->mainFrame())
+    if (auto* coreFrame = dynamicDowncast<WebCore::LocalFrame>(webPage->priv->webPage->mainFrame()))
         return kit(coreFrame->document());
 
     return nullptr;
 }
+#endif
 
 /**
  * webkit_web_page_get_id:
@@ -834,6 +876,7 @@ const gchar* webkit_web_page_get_uri(WebKitWebPage* webPage)
  * Returns: (transfer none): the #WebKitFrame that is the main frame of @web_page
  *
  * Since: 2.2
+ * Deprecated: 2.48
  */
 WebKitFrame* webkit_web_page_get_main_frame(WebKitWebPage* webPage)
 {
@@ -867,8 +910,8 @@ WebKitWebEditor* webkit_web_page_get_editor(WebKitWebPage* webPage)
  * @web_page: a #WebKitWebPage
  * @message: a #WebKitUserMessage
  * @cancellable: (nullable): a #GCancellable or %NULL to ignore
- * @callback: (scope async): (nullable): A #GAsyncReadyCallback to call when the request is satisfied or %NULL
- * @user_data: (closure): the data to pass to callback function
+ * @callback: (scope async) (nullable): A #GAsyncReadyCallback to call when the request is satisfied or %NULL
+ * @user_data: the data to pass to callback function
  *
  * Send @message to the #WebKitWebView corresponding to @web_page. If @message is floating, it's consumed.
  *
@@ -926,3 +969,31 @@ WebKitUserMessage* webkit_web_page_send_message_to_view_finish(WebKitWebPage* we
 
     return WEBKIT_USER_MESSAGE(g_task_propagate_pointer(G_TASK(result), error));
 }
+
+/**
+ * webkit_web_page_get_form_manager:
+ * @web_page: a #WebKitWebPage
+ * @world: (nullable): a #WebKitScriptWorld
+ *
+ * Get the #WebKitWebFormManager of @web_page in @world.
+ *
+ * Returns: (transfer none): a #WebKitWebFormManager
+ *
+ * Since: 2.40
+ */
+WebKitWebFormManager* webkit_web_page_get_form_manager(WebKitWebPage* webPage, WebKitScriptWorld* world)
+{
+    g_return_val_if_fail(WEBKIT_IS_WEB_PAGE(webPage), nullptr);
+    g_return_val_if_fail(!world || WEBKIT_IS_SCRIPT_WORLD(world), nullptr);
+
+    if (!world)
+        world = webkit_script_world_get_default();
+
+    auto addResult = webPage->priv->formManagerMap.ensure(world, [] {
+        return adoptGRef(webkitWebFormManagerCreate());
+    });
+    if (addResult.isNewEntry)
+        g_object_weak_ref(G_OBJECT(world), reinterpret_cast<GWeakNotify>(worldDestroyed), webPage);
+
+    return addResult.iterator->value.get();
+}
