@@ -26,17 +26,18 @@
 #include "config.h"
 #include "ShareableBitmapUtilities.h"
 
-#include "ShareableBitmap.h"
 #include <WebCore/CachedImage.h>
-#include <WebCore/Frame.h>
 #include <WebCore/FrameSnapshotting.h>
 #include <WebCore/GeometryUtilities.h>
 #include <WebCore/GraphicsContext.h>
 #include <WebCore/ImageBuffer.h>
 #include <WebCore/IntSize.h>
+#include <WebCore/LocalFrame.h>
 #include <WebCore/PlatformScreen.h>
+#include <WebCore/RenderElementInlines.h>
 #include <WebCore/RenderImage.h>
 #include <WebCore/RenderVideo.h>
+#include <WebCore/ShareableBitmap.h>
 
 namespace WebKit {
 using namespace WebCore;
@@ -44,37 +45,36 @@ using namespace WebCore;
 RefPtr<ShareableBitmap> createShareableBitmap(RenderImage& renderImage, CreateShareableBitmapFromImageOptions&& options)
 {
     Ref frame = renderImage.frame();
-    auto colorSpaceForBitmap = screenColorSpace(frame->mainFrame().view());
-    if (!renderImage.isMedia() && !renderImage.opacity() && options.useSnapshotForTransparentImages == UseSnapshotForTransparentImages::Yes) {
+    auto colorSpaceForBitmap = screenColorSpace(frame->mainFrame().virtualView());
+    if (!renderImage.isRenderMedia() && !renderImage.opacity() && options.useSnapshotForTransparentImages == UseSnapshotForTransparentImages::Yes) {
         auto snapshotRect = renderImage.absoluteBoundingBoxRect();
         if (snapshotRect.isEmpty())
             return { };
 
         OptionSet<SnapshotFlags> snapshotFlags { SnapshotFlags::ExcludeSelectionHighlighting, SnapshotFlags::PaintEverythingExcludingSelection };
-        auto imageBuffer = snapshotFrameRect(frame.get(), snapshotRect, { snapshotFlags, PixelFormat::BGRA8, DestinationColorSpace::SRGB() });
+        auto imageBuffer = snapshotFrameRect(frame.get(), snapshotRect, { snapshotFlags, ImageBufferPixelFormat::BGRA8, DestinationColorSpace::SRGB() });
         if (!imageBuffer)
             return { };
 
-        auto snapshotImage = ImageBuffer::sinkIntoImage(WTFMove(imageBuffer), PreserveResolution::Yes);
+        auto snapshotImage = ImageBuffer::sinkIntoNativeImage(WTFMove(imageBuffer));
         if (!snapshotImage)
             return { };
 
-        auto bitmap = ShareableBitmap::create(snapshotRect.size(), { WTFMove(colorSpaceForBitmap) });
+        auto bitmap = ShareableBitmap::create({ snapshotImage->size(), WTFMove(colorSpaceForBitmap) });
         if (!bitmap)
             return { };
 
         auto context = bitmap->createGraphicsContext();
         if (!context)
             return { };
-
-        context->drawImage(*snapshotImage, { FloatPoint::zero(), snapshotRect.size() });
+        FloatRect imageRect { { }, snapshotImage->size() };
+        context->drawNativeImage(*snapshotImage, imageRect, imageRect);
         return bitmap;
     }
 
 #if ENABLE(VIDEO)
-    if (is<RenderVideo>(renderImage)) {
-        auto& renderVideo = downcast<RenderVideo>(renderImage);
-        Ref video = renderVideo.videoElement();
+    if (auto* renderVideo = dynamicDowncast<RenderVideo>(renderImage)) {
+        Ref video = renderVideo->videoElement();
         auto image = video->nativeImageForCurrentTime();
         if (!image)
             return { };
@@ -83,7 +83,7 @@ RefPtr<ShareableBitmap> createShareableBitmap(RenderImage& renderImage, CreateSh
         if (imageSize.isEmpty() || imageSize.width() <= 1 || imageSize.height() <= 1)
             return { };
 
-        auto bitmap = ShareableBitmap::create(imageSize, { WTFMove(colorSpaceForBitmap) });
+        auto bitmap = ShareableBitmap::create({ imageSize, WTFMove(colorSpaceForBitmap) });
         if (!bitmap)
             return { };
 
@@ -91,7 +91,7 @@ RefPtr<ShareableBitmap> createShareableBitmap(RenderImage& renderImage, CreateSh
         if (!context)
             return { };
 
-        context->drawNativeImage(*image, imageSize, FloatRect { { }, imageSize }, FloatRect { { }, imageSize });
+        context->drawNativeImage(*image, FloatRect { { }, imageSize }, FloatRect { { }, imageSize });
         return bitmap;
     }
 #endif // ENABLE(VIDEO)
@@ -114,7 +114,7 @@ RefPtr<ShareableBitmap> createShareableBitmap(RenderImage& renderImage, CreateSh
     }
 
     // FIXME: Only select ExtendedColor on images known to need wide gamut.
-    auto sharedBitmap = ShareableBitmap::create(IntSize(bitmapSize), { WTFMove(colorSpaceForBitmap) });
+    auto sharedBitmap = ShareableBitmap::create({ IntSize(bitmapSize), WTFMove(colorSpaceForBitmap) });
     if (!sharedBitmap)
         return { };
 
